// -*- mode: js; indent-tabs-mode: nil; js-basic-offset: 4 -*-
//
// This file is part of Genie
//
// Copyright 2020 The Board of Trustees of the Leland Stanford Junior University
//
// Licensed under the Apache License, Version 2.0 (the "License");
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
//
//    http://www.apache.org/licenses/LICENSE-2.0
//
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// See the License for the specific language governing permissions and
// limitations under the License.
//
// Author: Giovanni Campagna <gcampagn@cs.stanford.edu>

// Coref question templates: questions about the current result, referred by coreference ("it", "that", "the restaurant")

import assert from 'assert';
import { Ast } from 'thingtalk';
import * as C from '../../ast_manip';
import * as D from '../../dialogue_acts';
import ThingpediaLoader from '../../load-thingpedia';
import type * as $Genie from "genie-toolkit";
export default async function($runtime : typeof $Genie.SentenceGeneratorRuntime, $ttUtils : typeof $Genie.ThingTalkUtils, $options : $Genie.SentenceGeneratorTypes.GrammarOptions, $locale : $Genie.I18n.LanguagePack, $grammar : $Genie.SentenceGenerator<any, any>, $loader : ThingpediaLoader) : Promise<void> {
   $grammar.declareSymbol("out_param_list_for_coref_question");
   $grammar.addRule("out_param_list_for_coref_question", [new $runtime.NonTerminal("out_param_Any", undefined), "and", new $runtime.NonTerminal("out_param_Any", ["functionName" as ("functionName" extends keyof ReturnType<typeof C.paramKeyFn> ? string : void), 0, "functionName" as ("functionName" extends keyof ReturnType<typeof C.paramKeyFn> ? string : void)])], ((p1 : C.ParamSlot, $0 : undefined, p2 : C.ParamSlot) : (C.ParamSlot[])|null => {
        if (p1.name === 'id' || p2.name === 'id' || p1.name === p2.name)
            return null;
        assert(C.isSameFunction(p1.schema, p2.schema));
        return [p1, p2];
    }), C.paramArrayKeyFn, {  });
   $grammar.addRule("out_param_list_for_coref_question", [new $runtime.NonTerminal("out_param_Any", undefined), ",", new $runtime.NonTerminal("out_param_Any", ["functionName" as ("functionName" extends keyof ReturnType<typeof C.paramKeyFn> ? string : void), 0, "functionName" as ("functionName" extends keyof ReturnType<typeof C.paramKeyFn> ? string : void)]), new $runtime.Choice([",", ""]), "and", new $runtime.NonTerminal("out_param_Any", ["functionName" as ("functionName" extends keyof ReturnType<typeof C.paramKeyFn> ? string : void), 0, "functionName" as ("functionName" extends keyof ReturnType<typeof C.paramKeyFn> ? string : void)])], ((p1 : C.ParamSlot, $0 : undefined, p2 : C.ParamSlot, $1 : undefined, $2 : undefined, p3 : C.ParamSlot) : (C.ParamSlot[])|null => {
        if (p1.name === 'id' || p2.name === 'id' || p3.name === 'id')
            return null;
        if (p1.name === p2.name || p2.name === p3.name || p1.name === p3.name)
            return null;
        assert(C.isSameFunction(p1.schema, p2.schema));
        assert(C.isSameFunction(p1.schema, p3.schema));
        return [p1, p2, p3];
    }), C.paramArrayKeyFn, {  });
   $grammar.declareSymbol("simple_user_search_question");
   $grammar.addRule("simple_user_search_question", [new $runtime.Choice(["can you tell me", ""]), "what is", new $runtime.Choice(["the", "its"]), new $runtime.NonTerminal("out_param_Any", undefined), "?"], (($0 : undefined, $1 : undefined, $2 : undefined, param : C.ParamSlot, $3 : undefined) : (C.ParamSlot[])|null => {
        if (param.name === 'id')
            return null;
        return [param];
    }), C.paramArrayKeyFn, {  });
   $grammar.addRule("simple_user_search_question", ["can you tell me", new $runtime.Choice(["the", "its"]), new $runtime.NonTerminal("out_param_Any", undefined), "?"], (($0 : undefined, $1 : undefined, param : C.ParamSlot, $2 : undefined) : (C.ParamSlot[])|null => {
        if (param.name === 'id')
            return null;
        return [param];
    }), C.paramArrayKeyFn, {  });
   $grammar.addRule("simple_user_search_question", [new $runtime.Choice(["can you tell me", ""]), "what", new $runtime.NonTerminal("out_param_Any", undefined), "does it have ?"], (($0 : undefined, $1 : undefined, param : C.ParamSlot, $2 : undefined) : (C.ParamSlot[])|null => {
        if (param.name === 'id')
            return null;
        return [param];
    }), C.paramArrayKeyFn, {  });
   $grammar.addRule("simple_user_search_question", ["i would like to", new $runtime.Choice(["know", "see"]), new $runtime.Choice(["the", "its"]), new $runtime.NonTerminal("out_param_Any", undefined), new $runtime.Choice(["first", ""]), "."], (($0 : undefined, $1 : undefined, $2 : undefined, param : C.ParamSlot, $3 : undefined, $4 : undefined) : (C.ParamSlot[])|null => {
        if (param.name === 'id')
            return null;
        return [param];
    }), C.paramArrayKeyFn, {  });
   $grammar.addRule("simple_user_search_question", ["i need", new $runtime.Choice(["the", "its"]), new $runtime.NonTerminal("out_param_Any", undefined), new $runtime.Choice(["first", ""]), "."], (($0 : undefined, $1 : undefined, param : C.ParamSlot, $2 : undefined, $3 : undefined) : (C.ParamSlot[])|null => {
        if (param.name === 'id')
            return null;
        return [param];
    }), C.paramArrayKeyFn, {  });
   $grammar.addRule("simple_user_search_question", ["i just need", new $runtime.Choice(["the", "its"]), new $runtime.NonTerminal("out_param_Any", undefined), "."], (($0 : undefined, $1 : undefined, param : C.ParamSlot, $2 : undefined) : (C.ParamSlot[])|null => {
        if (param.name === 'id')
            return null;
        return [param];
    }), C.paramArrayKeyFn, {  });
   $grammar.addRule("simple_user_search_question", [new $runtime.Choice(["can you tell me", ""]), "what", new $runtime.Choice(["is", "are"]), new $runtime.Choice(["the", "its"]), new $runtime.NonTerminal("out_param_list_for_coref_question", undefined), "?"], (($0 : undefined, $1 : undefined, $2 : undefined, $3 : undefined, params : C.ParamSlot[], $4 : undefined) : (C.ParamSlot[])|null => params), C.paramArrayKeyFn, {  });
   $grammar.addRule("simple_user_search_question", ["can you tell me", new $runtime.Choice(["the", "its"]), new $runtime.NonTerminal("out_param_list_for_coref_question", undefined), "?"], (($0 : undefined, $1 : undefined, params : C.ParamSlot[], $2 : undefined) : (C.ParamSlot[])|null => params), C.paramArrayKeyFn, {  });
   $grammar.addRule("simple_user_search_question", ["i would like to", new $runtime.Choice(["know", "see"]), new $runtime.Choice(["the", "its"]), new $runtime.NonTerminal("out_param_list_for_coref_question", undefined), new $runtime.Choice(["first", ""]), "."], (($0 : undefined, $1 : undefined, $2 : undefined, params : C.ParamSlot[], $3 : undefined, $4 : undefined) : (C.ParamSlot[])|null => params), C.paramArrayKeyFn, {  });
   $grammar.addRule("simple_user_search_question", ["i need", new $runtime.Choice(["the", "its"]), new $runtime.NonTerminal("out_param_list_for_coref_question", undefined), new $runtime.Choice(["first", ""]), "."], (($0 : undefined, $1 : undefined, params : C.ParamSlot[], $2 : undefined, $3 : undefined) : (C.ParamSlot[])|null => params), C.paramArrayKeyFn, {  });
   $grammar.addRule("simple_user_search_question", ["i just need", new $runtime.Choice(["the", "its"]), new $runtime.NonTerminal("out_param_list_for_coref_question", undefined), "."], (($0 : undefined, $1 : undefined, params : C.ParamSlot[], $2 : undefined) : (C.ParamSlot[])|null => params), C.paramArrayKeyFn, {  });
   $grammar.addRule("simple_user_search_question", [new $runtime.Choice(["can you tell me", ""]), "what is", "the", new $runtime.NonTerminal("out_param_Any", undefined), "of the", new $runtime.NonTerminal("base_table", ["functionName" as ("functionName" extends keyof ReturnType<typeof C.expressionKeyFn> ? string : void), 3, "functionName" as ("functionName" extends keyof ReturnType<typeof C.paramKeyFn> ? string : void)]), "?"], (($0 : undefined, $1 : undefined, $2 : undefined, param : C.ParamSlot, $3 : undefined, base : Ast.InvocationExpression, $4 : undefined) : (C.ParamSlot[])|null => {
        assert(C.isSameFunction(base.schema!, param.schema));
        if (param.name === 'id')
            return null;
        return [param];
    }), C.paramArrayKeyFn, {  });
   $grammar.addRule("simple_user_search_question", ["can you tell me", "the", new $runtime.NonTerminal("out_param_Any", undefined), "of the", new $runtime.NonTerminal("base_table", ["functionName" as ("functionName" extends keyof ReturnType<typeof C.expressionKeyFn> ? string : void), 2, "functionName" as ("functionName" extends keyof ReturnType<typeof C.paramKeyFn> ? string : void)]), "?"], (($0 : undefined, $1 : undefined, param : C.ParamSlot, $2 : undefined, base : Ast.InvocationExpression, $3 : undefined) : (C.ParamSlot[])|null => {
        assert(C.isSameFunction(base.schema!, param.schema));
        if (param.name === 'id')
            return null;
        return [param];
    }), C.paramArrayKeyFn, {  });
   $grammar.addRule("simple_user_search_question", [new $runtime.Choice(["can you tell me", ""]), "what", new $runtime.NonTerminal("out_param_Any", undefined), "does the", new $runtime.NonTerminal("base_table", ["functionName" as ("functionName" extends keyof ReturnType<typeof C.expressionKeyFn> ? string : void), 2, "functionName" as ("functionName" extends keyof ReturnType<typeof C.paramKeyFn> ? string : void)]), "have ?"], (($0 : undefined, $1 : undefined, param : C.ParamSlot, $2 : undefined, base : Ast.InvocationExpression, $3 : undefined) : (C.ParamSlot[])|null => {
        assert(C.isSameFunction(base.schema!, param.schema));
        if (param.name === 'id')
            return null;
        return [param];
    }), C.paramArrayKeyFn, {  });
   $grammar.addRule("simple_user_search_question", ["i would like to", new $runtime.Choice(["know", "see"]), "the", new $runtime.NonTerminal("out_param_Any", undefined), "of the", new $runtime.NonTerminal("base_table", ["functionName" as ("functionName" extends keyof ReturnType<typeof C.expressionKeyFn> ? string : void), 3, "functionName" as ("functionName" extends keyof ReturnType<typeof C.paramKeyFn> ? string : void)]), new $runtime.Choice(["first", ""]), "."], (($0 : undefined, $1 : undefined, $2 : undefined, param : C.ParamSlot, $3 : undefined, base : Ast.InvocationExpression, $4 : undefined, $5 : undefined) : (C.ParamSlot[])|null => {
        assert(C.isSameFunction(base.schema!, param.schema));
        if (param.name === 'id')
            return null;
        return [param];
    }), C.paramArrayKeyFn, {  });
   $grammar.addRule("simple_user_search_question", ["i just need", new $runtime.Choice(["the", "its"]), "the", new $runtime.NonTerminal("out_param_Any", undefined), "of the", new $runtime.NonTerminal("base_table", ["functionName" as ("functionName" extends keyof ReturnType<typeof C.expressionKeyFn> ? string : void), 3, "functionName" as ("functionName" extends keyof ReturnType<typeof C.paramKeyFn> ? string : void)]), "."], (($0 : undefined, $1 : undefined, $2 : undefined, param : C.ParamSlot, $3 : undefined, base : Ast.InvocationExpression, $4 : undefined) : (C.ParamSlot[])|null => {
        assert(C.isSameFunction(base.schema!, param.schema));
        if (param.name === 'id')
            return null;
        return [param];
    }), C.paramArrayKeyFn, {  });
   $grammar.addRule("simple_user_search_question", [new $runtime.Choice(["can you tell me", ""]), "what is", "the", new $runtime.NonTerminal("base_table", undefined), "'s", new $runtime.NonTerminal("out_param_Any", ["functionName" as ("functionName" extends keyof ReturnType<typeof C.paramKeyFn> ? string : void), 3, "functionName" as ("functionName" extends keyof ReturnType<typeof C.expressionKeyFn> ? string : void)]), "?"], (($0 : undefined, $1 : undefined, $2 : undefined, base : Ast.InvocationExpression, $3 : undefined, param : C.ParamSlot, $4 : undefined) : (C.ParamSlot[])|null => {
        assert(C.isSameFunction(base.schema!, param.schema));
        if (param.name === 'id')
            return null;
        return [param];
    }), C.paramArrayKeyFn, {  });
   $grammar.addRule("simple_user_search_question", ["can you tell me", "the", new $runtime.NonTerminal("base_table", undefined), "'s", new $runtime.NonTerminal("out_param_Any", ["functionName" as ("functionName" extends keyof ReturnType<typeof C.paramKeyFn> ? string : void), 2, "functionName" as ("functionName" extends keyof ReturnType<typeof C.expressionKeyFn> ? string : void)]), "?"], (($0 : undefined, $1 : undefined, base : Ast.InvocationExpression, $2 : undefined, param : C.ParamSlot, $3 : undefined) : (C.ParamSlot[])|null => {
        assert(C.isSameFunction(base.schema!, param.schema));
        if (param.name === 'id')
            return null;
        return [param];
    }), C.paramArrayKeyFn, {  });
   $grammar.addRule("simple_user_search_question", ["i would like to", new $runtime.Choice(["know", "see"]), "the", new $runtime.NonTerminal("base_table", undefined), "'s", new $runtime.NonTerminal("out_param_Any", ["functionName" as ("functionName" extends keyof ReturnType<typeof C.paramKeyFn> ? string : void), 3, "functionName" as ("functionName" extends keyof ReturnType<typeof C.expressionKeyFn> ? string : void)]), new $runtime.Choice(["first", ""]), "."], (($0 : undefined, $1 : undefined, $2 : undefined, base : Ast.InvocationExpression, $3 : undefined, param : C.ParamSlot, $4 : undefined, $5 : undefined) : (C.ParamSlot[])|null => {
        assert(C.isSameFunction(base.schema!, param.schema));
        if (param.name === 'id')
            return null;
        return [param];
    }), C.paramArrayKeyFn, {  });
   $grammar.addRule("simple_user_search_question", ["i just need", new $runtime.Choice(["the", "its"]), "the", new $runtime.NonTerminal("base_table", undefined), "'s", new $runtime.NonTerminal("out_param_Any", ["functionName" as ("functionName" extends keyof ReturnType<typeof C.paramKeyFn> ? string : void), 3, "functionName" as ("functionName" extends keyof ReturnType<typeof C.expressionKeyFn> ? string : void)]), "."], (($0 : undefined, $1 : undefined, $2 : undefined, base : Ast.InvocationExpression, $3 : undefined, param : C.ParamSlot, $4 : undefined) : (C.ParamSlot[])|null => {
        assert(C.isSameFunction(base.schema!, param.schema));
        if (param.name === 'id')
            return null;
        return [param];
    }), C.paramArrayKeyFn, {  });
   $grammar.addRule("simple_user_search_question", [new $runtime.Choice(["can you tell me", ""]), "what", new $runtime.Choice(["is", "are"]), "the", new $runtime.NonTerminal("out_param_list_for_coref_question", undefined), "of the", new $runtime.NonTerminal("base_table", ["functionName" as ("functionName" extends keyof ReturnType<typeof C.expressionKeyFn> ? string : void), 4, "functionName" as ("functionName" extends keyof ReturnType<typeof C.paramArrayKeyFn> ? string : void)]), "?"], (($0 : undefined, $1 : undefined, $2 : undefined, $3 : undefined, params : C.ParamSlot[], $4 : undefined, base : Ast.InvocationExpression, $5 : undefined) : (C.ParamSlot[])|null => {
        if (!params.every((p) => C.isSameFunction(base.schema!, p.schema)))
            return null;
        return params;
    }), C.paramArrayKeyFn, {  });
   $grammar.addRule("simple_user_search_question", ["can you tell me", "the", new $runtime.NonTerminal("out_param_list_for_coref_question", undefined), "of the", new $runtime.NonTerminal("base_table", ["functionName" as ("functionName" extends keyof ReturnType<typeof C.expressionKeyFn> ? string : void), 2, "functionName" as ("functionName" extends keyof ReturnType<typeof C.paramArrayKeyFn> ? string : void)]), "?"], (($0 : undefined, $1 : undefined, params : C.ParamSlot[], $2 : undefined, base : Ast.InvocationExpression, $3 : undefined) : (C.ParamSlot[])|null => {
        if (!params.every((p) => C.isSameFunction(base.schema!, p.schema)))
            return null;
        return params;
    }), C.paramArrayKeyFn, {  });
   $grammar.addRule("simple_user_search_question", [new $runtime.Choice(["can you tell me", ""]), "what", new $runtime.NonTerminal("out_param_list_for_coref_question", undefined), "does the", new $runtime.NonTerminal("base_table", ["functionName" as ("functionName" extends keyof ReturnType<typeof C.expressionKeyFn> ? string : void), 2, "functionName" as ("functionName" extends keyof ReturnType<typeof C.paramArrayKeyFn> ? string : void)]), "have ?"], (($0 : undefined, $1 : undefined, params : C.ParamSlot[], $2 : undefined, base : Ast.InvocationExpression, $3 : undefined) : (C.ParamSlot[])|null => {
        if (!params.every((p) => C.isSameFunction(base.schema!, p.schema)))
            return null;
        return params;
    }), C.paramArrayKeyFn, {  });
   $grammar.addRule("simple_user_search_question", ["i would like to", new $runtime.Choice(["know", "see"]), "the", new $runtime.NonTerminal("out_param_list_for_coref_question", undefined), "of the", new $runtime.NonTerminal("base_table", ["functionName" as ("functionName" extends keyof ReturnType<typeof C.expressionKeyFn> ? string : void), 3, "functionName" as ("functionName" extends keyof ReturnType<typeof C.paramArrayKeyFn> ? string : void)]), new $runtime.Choice(["first", ""]), "."], (($0 : undefined, $1 : undefined, $2 : undefined, params : C.ParamSlot[], $3 : undefined, base : Ast.InvocationExpression, $4 : undefined, $5 : undefined) : (C.ParamSlot[])|null => {
        if (!params.every((p) => C.isSameFunction(base.schema!, p.schema)))
            return null;
        return params;
    }), C.paramArrayKeyFn, {  });
   $grammar.addRule("simple_user_search_question", ["i just need", new $runtime.Choice(["the", "its"]), "the", new $runtime.NonTerminal("out_param_list_for_coref_question", undefined), "of the", new $runtime.NonTerminal("base_table", ["functionName" as ("functionName" extends keyof ReturnType<typeof C.expressionKeyFn> ? string : void), 3, "functionName" as ("functionName" extends keyof ReturnType<typeof C.paramArrayKeyFn> ? string : void)]), "."], (($0 : undefined, $1 : undefined, $2 : undefined, params : C.ParamSlot[], $3 : undefined, base : Ast.InvocationExpression, $4 : undefined) : (C.ParamSlot[])|null => {
        if (!params.every((p) => C.isSameFunction(base.schema!, p.schema)))
            return null;
        return params;
    }), C.paramArrayKeyFn, {  });
   $grammar.addRule("simple_user_search_question", [new $runtime.Choice(["can you tell me", ""]), "what is", "the", new $runtime.NonTerminal("base_table", undefined), "'s", new $runtime.NonTerminal("out_param_list_for_coref_question", ["functionName" as ("functionName" extends keyof ReturnType<typeof C.paramArrayKeyFn> ? string : void), 3, "functionName" as ("functionName" extends keyof ReturnType<typeof C.expressionKeyFn> ? string : void)]), "?"], (($0 : undefined, $1 : undefined, $2 : undefined, base : Ast.InvocationExpression, $3 : undefined, params : C.ParamSlot[], $4 : undefined) : (C.ParamSlot[])|null => {
        if (!params.every((p) => C.isSameFunction(base.schema!, p.schema)))
            return null;
        return params;
    }), C.paramArrayKeyFn, {  });
   $grammar.addRule("simple_user_search_question", ["can you tell me", "the", new $runtime.NonTerminal("base_table", undefined), "'s", new $runtime.NonTerminal("out_param_list_for_coref_question", ["functionName" as ("functionName" extends keyof ReturnType<typeof C.paramArrayKeyFn> ? string : void), 2, "functionName" as ("functionName" extends keyof ReturnType<typeof C.expressionKeyFn> ? string : void)]), "?"], (($0 : undefined, $1 : undefined, base : Ast.InvocationExpression, $2 : undefined, params : C.ParamSlot[], $3 : undefined) : (C.ParamSlot[])|null => {
        if (!params.every((p) => C.isSameFunction(base.schema!, p.schema)))
            return null;
        return params;
    }), C.paramArrayKeyFn, {  });
   $grammar.addRule("simple_user_search_question", ["i would like to", new $runtime.Choice(["know", "see"]), "the", new $runtime.NonTerminal("base_table", undefined), "'s", new $runtime.NonTerminal("out_param_list_for_coref_question", ["functionName" as ("functionName" extends keyof ReturnType<typeof C.paramArrayKeyFn> ? string : void), 3, "functionName" as ("functionName" extends keyof ReturnType<typeof C.expressionKeyFn> ? string : void)]), new $runtime.Choice(["first", ""]), "."], (($0 : undefined, $1 : undefined, $2 : undefined, base : Ast.InvocationExpression, $3 : undefined, params : C.ParamSlot[], $4 : undefined, $5 : undefined) : (C.ParamSlot[])|null => {
        if (!params.every((p) => C.isSameFunction(base.schema!, p.schema)))
            return null;
        return params;
    }), C.paramArrayKeyFn, {  });
   $grammar.addRule("simple_user_search_question", ["i just need", new $runtime.Choice(["the", "its"]), "the", new $runtime.NonTerminal("base_table", undefined), "'s", new $runtime.NonTerminal("out_param_list_for_coref_question", ["functionName" as ("functionName" extends keyof ReturnType<typeof C.paramArrayKeyFn> ? string : void), 3, "functionName" as ("functionName" extends keyof ReturnType<typeof C.expressionKeyFn> ? string : void)]), "."], (($0 : undefined, $1 : undefined, $2 : undefined, base : Ast.InvocationExpression, $3 : undefined, params : C.ParamSlot[], $4 : undefined) : (C.ParamSlot[])|null => {
        if (!params.every((p) => C.isSameFunction(base.schema!, p.schema)))
            return null;
        return params;
    }), C.paramArrayKeyFn, {  });
   $grammar.addRule("simple_user_search_question", [new $runtime.Choice(["can you tell me", ""]), "does", new $runtime.Choice(["it", "this", "that"]), "have", new $runtime.NonTerminal("npp_boolean_projection", undefined), "?"], (($0 : undefined, $1 : undefined, $2 : undefined, $3 : undefined, param : C.ParamSlot, $4 : undefined) : (C.ParamSlot[])|null => {
        return [param];
    }), C.paramArrayKeyFn, {  });
   $grammar.addRule("simple_user_search_question", [new $runtime.Choice(["can you tell me", ""]), "is", "there", new $runtime.NonTerminal("npp_boolean_projection", undefined), "?"], (($0 : undefined, $1 : undefined, $2 : undefined, param : C.ParamSlot, $3 : undefined) : (C.ParamSlot[])|null => {
        return [param];
    }), C.paramArrayKeyFn, {  });
   $grammar.addRule("simple_user_search_question", ["i would like to", new $runtime.Choice(["know", "see"]), new $runtime.Choice(["if", "whether"]), new $runtime.Choice(["it", "this", "that"]), "has", new $runtime.NonTerminal("npp_boolean_projection", undefined), new $runtime.Choice(["first", ""]), "."], (($0 : undefined, $1 : undefined, $2 : undefined, $3 : undefined, $4 : undefined, param : C.ParamSlot, $5 : undefined, $6 : undefined) : (C.ParamSlot[])|null => {
        return [param];
    }), C.paramArrayKeyFn, {  });
   $grammar.addRule("simple_user_search_question", ["i would like to", new $runtime.Choice(["know", "see"]), new $runtime.Choice(["if", "whether"]), "there", "is", new $runtime.NonTerminal("npp_boolean_projection", undefined), new $runtime.Choice(["first", ""]), "."], (($0 : undefined, $1 : undefined, $2 : undefined, $3 : undefined, $4 : undefined, param : C.ParamSlot, $5 : undefined, $6 : undefined) : (C.ParamSlot[])|null => {
        return [param];
    }), C.paramArrayKeyFn, {  });
   $grammar.addRule("simple_user_search_question", [new $runtime.Choice(["can you tell me", ""]), "does", new $runtime.Choice(["it", "this", "that"]), new $runtime.NonTerminal("avp_boolean_projection", undefined), "?"], (($0 : undefined, $1 : undefined, $2 : undefined, param : C.ParamSlot, $3 : undefined) : (C.ParamSlot[])|null => {
        return [param];
    }), C.paramArrayKeyFn, {  });
   $grammar.addRule("simple_user_search_question", ["i would like to", new $runtime.Choice(["know", "see"]), new $runtime.Choice(["if", "whether"]), new $runtime.Choice(["it", "this", "that"]), new $runtime.NonTerminal("avp_boolean_projection", undefined), new $runtime.Choice(["first", ""]), "."], (($0 : undefined, $1 : undefined, $2 : undefined, $3 : undefined, param : C.ParamSlot, $4 : undefined, $5 : undefined) : (C.ParamSlot[])|null => {
        return [param];
    }), C.paramArrayKeyFn, {  });
   $grammar.addRule("simple_user_search_question", [new $runtime.Choice(["can you tell me", ""]), "is", new $runtime.Choice(["it", "this", "that"]), new $runtime.NonTerminal("pvp_boolean_projection", undefined), "?"], (($0 : undefined, $1 : undefined, $2 : undefined, param : C.ParamSlot, $3 : undefined) : (C.ParamSlot[])|null => {
        return [param];
    }), C.paramArrayKeyFn, {  });
   $grammar.addRule("simple_user_search_question", ["i would like to", new $runtime.Choice(["know", "see"]), new $runtime.Choice(["if", "whether"]), new $runtime.Choice(["it", "this", "that"]), "is", new $runtime.NonTerminal("pvp_boolean_projection", undefined), new $runtime.Choice(["first", ""]), "."], (($0 : undefined, $1 : undefined, $2 : undefined, $3 : undefined, $4 : undefined, param : C.ParamSlot, $5 : undefined, $6 : undefined) : (C.ParamSlot[])|null => {
        return [param];
    }), C.paramArrayKeyFn, {  });
   $grammar.addRule("simple_user_search_question", [new $runtime.Choice(["can you tell me", ""]), "is", new $runtime.Choice(["it", "this", "that"]), new $runtime.NonTerminal("preposition_boolean_projection", undefined), "?"], (($0 : undefined, $1 : undefined, $2 : undefined, param : C.ParamSlot, $3 : undefined) : (C.ParamSlot[])|null => {
        return [param];
    }), C.paramArrayKeyFn, {  });
   $grammar.addRule("simple_user_search_question", ["i would like to", new $runtime.Choice(["know", "see"]), new $runtime.Choice(["if", "whether"]), new $runtime.Choice(["it", "this", "that"]), "is", new $runtime.NonTerminal("preposition_boolean_projection", undefined), new $runtime.Choice(["first", ""]), "."], (($0 : undefined, $1 : undefined, $2 : undefined, $3 : undefined, $4 : undefined, param : C.ParamSlot, $5 : undefined, $6 : undefined) : (C.ParamSlot[])|null => {
        return [param];
    }), C.paramArrayKeyFn, {  });
   $grammar.addRule("simple_user_search_question", [new $runtime.Choice(["can you tell me", ""]), "is", "it", new $runtime.NonTerminal("npi_boolean_projection", undefined), "?"], (($0 : undefined, $1 : undefined, $2 : undefined, param : C.ParamSlot, $3 : undefined) : (C.ParamSlot[])|null => {
        return [param];
    }), C.paramArrayKeyFn, {  });
   $grammar.addRule("simple_user_search_question", ["i would like to", new $runtime.Choice(["know", "see"]), new $runtime.Choice(["if", "whether"]), new $runtime.Choice(["it", "this", "that"]), "is", new $runtime.NonTerminal("npi_boolean_projection", undefined), new $runtime.Choice(["first", ""]), "."], (($0 : undefined, $1 : undefined, $2 : undefined, $3 : undefined, $4 : undefined, param : C.ParamSlot, $5 : undefined, $6 : undefined) : (C.ParamSlot[])|null => {
        return [param];
    }), C.paramArrayKeyFn, {  });
   $grammar.addRule("simple_user_search_question", [new $runtime.Choice(["can you tell me", ""]), "is", "it", new $runtime.NonTerminal("npv_boolean_projection", undefined), "?"], (($0 : undefined, $1 : undefined, $2 : undefined, param : C.ParamSlot, $3 : undefined) : (C.ParamSlot[])|null => {
        return [param];
    }), C.paramArrayKeyFn, {  });
   $grammar.addRule("simple_user_search_question", ["i would like to", new $runtime.Choice(["know", "see"]), new $runtime.Choice(["if", "whether"]), new $runtime.Choice(["it", "this", "that"]), "is", new $runtime.NonTerminal("npv_boolean_projection", undefined), new $runtime.Choice(["first", ""]), "."], (($0 : undefined, $1 : undefined, $2 : undefined, $3 : undefined, $4 : undefined, param : C.ParamSlot, $5 : undefined, $6 : undefined) : (C.ParamSlot[])|null => {
        return [param];
    }), C.paramArrayKeyFn, {  });
   $grammar.addRule("simple_user_search_question", [new $runtime.Choice(["can you tell me", ""]), "is", "it", new $runtime.NonTerminal("apv_boolean_projection", undefined), "?"], (($0 : undefined, $1 : undefined, $2 : undefined, param : C.ParamSlot, $3 : undefined) : (C.ParamSlot[])|null => {
        return [param];
    }), C.paramArrayKeyFn, {  });
   $grammar.addRule("simple_user_search_question", ["i would like to", new $runtime.Choice(["know", "see"]), new $runtime.Choice(["if", "whether"]), new $runtime.Choice(["it", "this", "that"]), "is", new $runtime.NonTerminal("apv_boolean_projection", undefined), new $runtime.Choice(["first", ""]), "."], (($0 : undefined, $1 : undefined, $2 : undefined, $3 : undefined, $4 : undefined, param : C.ParamSlot, $5 : undefined, $6 : undefined) : (C.ParamSlot[])|null => {
        return [param];
    }), C.paramArrayKeyFn, {  });
   $grammar.addRule("simple_user_search_question", [new $runtime.Choice(["can you tell me", ""]), "does", "the", new $runtime.NonTerminal("base_table", undefined), "have", new $runtime.NonTerminal("npp_boolean_projection", ["functionName" as ("functionName" extends keyof ReturnType<typeof C.paramKeyFn> ? string : void), 3, "functionName" as ("functionName" extends keyof ReturnType<typeof C.expressionKeyFn> ? string : void)]), "?"], (($0 : undefined, $1 : undefined, $2 : undefined, base : Ast.InvocationExpression, $3 : undefined, param : C.ParamSlot, $4 : undefined) : (C.ParamSlot[])|null => {
        assert(C.isSameFunction(base.schema!, param.schema));
        return [param];
    }), C.paramArrayKeyFn, {  });
   $grammar.addRule("simple_user_search_question", ["i would like to", new $runtime.Choice(["know", "see"]), new $runtime.Choice(["if", "whether"]), "the", new $runtime.NonTerminal("base_table", undefined), "has", new $runtime.NonTerminal("npp_boolean_projection", ["functionName" as ("functionName" extends keyof ReturnType<typeof C.paramKeyFn> ? string : void), 4, "functionName" as ("functionName" extends keyof ReturnType<typeof C.expressionKeyFn> ? string : void)]), new $runtime.Choice(["first", ""]), "."], (($0 : undefined, $1 : undefined, $2 : undefined, $3 : undefined, base : Ast.InvocationExpression, $4 : undefined, param : C.ParamSlot, $5 : undefined, $6 : undefined) : (C.ParamSlot[])|null => {
        assert(C.isSameFunction(base.schema!, param.schema));
        return [param];
    }), C.paramArrayKeyFn, {  });
   $grammar.addRule("simple_user_search_question", [new $runtime.Choice(["can you tell me", ""]), "does", "the", new $runtime.NonTerminal("base_table", undefined), new $runtime.NonTerminal("avp_boolean_projection", ["functionName" as ("functionName" extends keyof ReturnType<typeof C.paramKeyFn> ? string : void), 3, "functionName" as ("functionName" extends keyof ReturnType<typeof C.expressionKeyFn> ? string : void)]), "?"], (($0 : undefined, $1 : undefined, $2 : undefined, base : Ast.InvocationExpression, param : C.ParamSlot, $3 : undefined) : (C.ParamSlot[])|null => {
        assert(C.isSameFunction(base.schema!, param.schema));
        return [param];
    }), C.paramArrayKeyFn, {  });
   $grammar.addRule("simple_user_search_question", ["i would like to", new $runtime.Choice(["know", "see"]), new $runtime.Choice(["if", "whether"]), "the", new $runtime.NonTerminal("base_table", undefined), new $runtime.NonTerminal("avp_boolean_projection", ["functionName" as ("functionName" extends keyof ReturnType<typeof C.paramKeyFn> ? string : void), 4, "functionName" as ("functionName" extends keyof ReturnType<typeof C.expressionKeyFn> ? string : void)]), new $runtime.Choice(["first", ""]), "."], (($0 : undefined, $1 : undefined, $2 : undefined, $3 : undefined, base : Ast.InvocationExpression, param : C.ParamSlot, $4 : undefined, $5 : undefined) : (C.ParamSlot[])|null => {
        assert(C.isSameFunction(base.schema!, param.schema));
        return [param];
    }), C.paramArrayKeyFn, {  });
   $grammar.addRule("simple_user_search_question", [new $runtime.Choice(["can you tell me", ""]), "is", "the", new $runtime.NonTerminal("base_table", undefined), new $runtime.NonTerminal("pvp_boolean_projection", ["functionName" as ("functionName" extends keyof ReturnType<typeof C.paramKeyFn> ? string : void), 3, "functionName" as ("functionName" extends keyof ReturnType<typeof C.expressionKeyFn> ? string : void)]), "?"], (($0 : undefined, $1 : undefined, $2 : undefined, base : Ast.InvocationExpression, param : C.ParamSlot, $3 : undefined) : (C.ParamSlot[])|null => {
        assert(C.isSameFunction(base.schema!, param.schema));
        return [param];
    }), C.paramArrayKeyFn, {  });
   $grammar.addRule("simple_user_search_question", ["i would like to", new $runtime.Choice(["know", "see"]), new $runtime.Choice(["if", "whether"]), "the", new $runtime.NonTerminal("base_table", undefined), "is", new $runtime.NonTerminal("pvp_boolean_projection", ["functionName" as ("functionName" extends keyof ReturnType<typeof C.paramKeyFn> ? string : void), 4, "functionName" as ("functionName" extends keyof ReturnType<typeof C.expressionKeyFn> ? string : void)]), new $runtime.Choice(["first", ""]), "."], (($0 : undefined, $1 : undefined, $2 : undefined, $3 : undefined, base : Ast.InvocationExpression, $4 : undefined, param : C.ParamSlot, $5 : undefined, $6 : undefined) : (C.ParamSlot[])|null => {
        assert(C.isSameFunction(base.schema!, param.schema));
        return [param];
    }), C.paramArrayKeyFn, {  });
   $grammar.addRule("simple_user_search_question", [new $runtime.Choice(["can you tell me", ""]), "is", "the", new $runtime.NonTerminal("base_table", undefined), new $runtime.NonTerminal("preposition_boolean_projection", ["functionName" as ("functionName" extends keyof ReturnType<typeof C.paramKeyFn> ? string : void), 3, "functionName" as ("functionName" extends keyof ReturnType<typeof C.expressionKeyFn> ? string : void)]), "?"], (($0 : undefined, $1 : undefined, $2 : undefined, base : Ast.InvocationExpression, param : C.ParamSlot, $3 : undefined) : (C.ParamSlot[])|null => {
        assert(C.isSameFunction(base.schema!, param.schema));
        return [param];
    }), C.paramArrayKeyFn, {  });
   $grammar.addRule("simple_user_search_question", ["i would like to", new $runtime.Choice(["know", "see"]), new $runtime.Choice(["if", "whether"]), "the", new $runtime.NonTerminal("base_table", undefined), "is", new $runtime.NonTerminal("preposition_boolean_projection", ["functionName" as ("functionName" extends keyof ReturnType<typeof C.paramKeyFn> ? string : void), 4, "functionName" as ("functionName" extends keyof ReturnType<typeof C.expressionKeyFn> ? string : void)]), new $runtime.Choice(["first", ""]), "."], (($0 : undefined, $1 : undefined, $2 : undefined, $3 : undefined, base : Ast.InvocationExpression, $4 : undefined, param : C.ParamSlot, $5 : undefined, $6 : undefined) : (C.ParamSlot[])|null => {
        assert(C.isSameFunction(base.schema!, param.schema));
        return [param];
    }), C.paramArrayKeyFn, {  });
   $grammar.addRule("simple_user_search_question", [new $runtime.Choice(["can you tell me", ""]), "is", "the", new $runtime.NonTerminal("base_table", undefined), new $runtime.NonTerminal("npi_boolean_projection", ["functionName" as ("functionName" extends keyof ReturnType<typeof C.paramKeyFn> ? string : void), 3, "functionName" as ("functionName" extends keyof ReturnType<typeof C.expressionKeyFn> ? string : void)]), "?"], (($0 : undefined, $1 : undefined, $2 : undefined, base : Ast.InvocationExpression, param : C.ParamSlot, $3 : undefined) : (C.ParamSlot[])|null => {
        assert(C.isSameFunction(base.schema!, param.schema));
        return [param];
    }), C.paramArrayKeyFn, {  });
   $grammar.addRule("simple_user_search_question", ["i would like to", new $runtime.Choice(["know", "see"]), new $runtime.Choice(["if", "whether"]), "the", new $runtime.NonTerminal("base_table", undefined), "is", new $runtime.NonTerminal("npi_boolean_projection", ["functionName" as ("functionName" extends keyof ReturnType<typeof C.paramKeyFn> ? string : void), 4, "functionName" as ("functionName" extends keyof ReturnType<typeof C.expressionKeyFn> ? string : void)]), new $runtime.Choice(["first", ""]), "."], (($0 : undefined, $1 : undefined, $2 : undefined, $3 : undefined, base : Ast.InvocationExpression, $4 : undefined, param : C.ParamSlot, $5 : undefined, $6 : undefined) : (C.ParamSlot[])|null => {
        assert(C.isSameFunction(base.schema!, param.schema));
        return [param];
    }), C.paramArrayKeyFn, {  });
   $grammar.addRule("simple_user_search_question", [new $runtime.Choice(["can you tell me", ""]), "is", "the", new $runtime.NonTerminal("base_table", undefined), new $runtime.NonTerminal("npv_boolean_projection", ["functionName" as ("functionName" extends keyof ReturnType<typeof C.paramKeyFn> ? string : void), 3, "functionName" as ("functionName" extends keyof ReturnType<typeof C.expressionKeyFn> ? string : void)]), "?"], (($0 : undefined, $1 : undefined, $2 : undefined, base : Ast.InvocationExpression, param : C.ParamSlot, $3 : undefined) : (C.ParamSlot[])|null => {
        assert(C.isSameFunction(base.schema!, param.schema));
        return [param];
    }), C.paramArrayKeyFn, {  });
   $grammar.addRule("simple_user_search_question", ["i would like to", new $runtime.Choice(["know", "see"]), new $runtime.Choice(["if", "whether"]), "the", new $runtime.NonTerminal("base_table", undefined), "is", new $runtime.NonTerminal("npv_boolean_projection", ["functionName" as ("functionName" extends keyof ReturnType<typeof C.paramKeyFn> ? string : void), 4, "functionName" as ("functionName" extends keyof ReturnType<typeof C.expressionKeyFn> ? string : void)]), new $runtime.Choice(["first", ""]), "."], (($0 : undefined, $1 : undefined, $2 : undefined, $3 : undefined, base : Ast.InvocationExpression, $4 : undefined, param : C.ParamSlot, $5 : undefined, $6 : undefined) : (C.ParamSlot[])|null => {
        assert(C.isSameFunction(base.schema!, param.schema));
        return [param];
    }), C.paramArrayKeyFn, {  });
   $grammar.addRule("simple_user_search_question", [new $runtime.Choice(["can you tell me", ""]), "is", "the", new $runtime.NonTerminal("base_table", undefined), new $runtime.NonTerminal("apv_boolean_projection", ["functionName" as ("functionName" extends keyof ReturnType<typeof C.paramKeyFn> ? string : void), 3, "functionName" as ("functionName" extends keyof ReturnType<typeof C.expressionKeyFn> ? string : void)]), "?"], (($0 : undefined, $1 : undefined, $2 : undefined, base : Ast.InvocationExpression, param : C.ParamSlot, $3 : undefined) : (C.ParamSlot[])|null => {
        assert(C.isSameFunction(base.schema!, param.schema));
        return [param];
    }), C.paramArrayKeyFn, {  });
   $grammar.addRule("simple_user_search_question", ["i would like to", new $runtime.Choice(["know", "see"]), new $runtime.Choice(["if", "whether"]), "the", new $runtime.NonTerminal("base_table", undefined), "is", new $runtime.NonTerminal("apv_boolean_projection", ["functionName" as ("functionName" extends keyof ReturnType<typeof C.paramKeyFn> ? string : void), 4, "functionName" as ("functionName" extends keyof ReturnType<typeof C.expressionKeyFn> ? string : void)]), new $runtime.Choice(["first", ""]), "."], (($0 : undefined, $1 : undefined, $2 : undefined, $3 : undefined, base : Ast.InvocationExpression, $4 : undefined, param : C.ParamSlot, $5 : undefined, $6 : undefined) : (C.ParamSlot[])|null => {
        assert(C.isSameFunction(base.schema!, param.schema));
        return [param];
    }), C.paramArrayKeyFn, {  });
   for (const proj of $loader.projections) {
       if (proj.category === 'pvp' || proj.category === 'preposition') {
           $grammar.declareSymbol("simple_user_search_question");
           $grammar.addRule("simple_user_search_question", [new $runtime.Choice(["can you tell me", ""]), proj.pronoun, proj.base, new $runtime.Choice(["is", "was", "are", "were"]), new $runtime.Choice(["it", "this", "that"]), proj.canonical, "?"], (($0 : undefined, $1 : undefined, $2 : undefined, $3 : undefined, $4 : undefined, $5 : undefined, $6 : undefined) : (C.ParamSlot[])|null => {
            if (proj.pname === 'id')
                return null;
            return [proj.pslot];
        }), C.paramArrayKeyFn, {  });
           $grammar.addRule("simple_user_search_question", ["i would like to", new $runtime.Choice(["know", "see"]), proj.pronoun, proj.base, new $runtime.Choice(["it", "this", "that"]), new $runtime.Choice(["is", "was", "are", "were"]), proj.canonical, new $runtime.Choice(["first", ""]), "."], (($0 : undefined, $1 : undefined, $2 : undefined, $3 : undefined, $4 : undefined, $5 : undefined, $6 : undefined, $7 : undefined, $8 : undefined) : (C.ParamSlot[])|null => {
            if (proj.pname === 'id')
                return null;
            return [proj.pslot];
        }), C.paramArrayKeyFn, {  });
           $grammar.declareSymbol("simple_user_search_question");
           $grammar.addRule("simple_user_search_question", [new $runtime.Choice(["can you tell me", ""]), proj.pronoun, proj.base, new $runtime.Choice(["is", "was", "are", "were"]), "the", new $runtime.NonTerminal("base_table", undefined), proj.canonical, "?"], (($0 : undefined, $1 : undefined, $2 : undefined, $3 : undefined, $4 : undefined, base : Ast.InvocationExpression, $5 : undefined, $6 : undefined) : (C.ParamSlot[])|null => {
            if (!C.isSameFunction(base.schema!, proj.pslot.schema))
                return null;
            if (proj.pname === 'id')
                return null;
            return [proj.pslot];
        }), C.paramArrayKeyFn, {  });
           $grammar.addRule("simple_user_search_question", ["i would like to", new $runtime.Choice(["know", "see"]), proj.pronoun, proj.base, "the", new $runtime.NonTerminal("base_table", ["functionName" as ("functionName" extends keyof ReturnType<typeof C.expressionKeyFn> ? string : void), ((proj.pslot.schema.qualifiedName)) as ReturnType<typeof C.expressionKeyFn>["functionName"]]), new $runtime.Choice(["is", "was", "are", "were"]), proj.canonical, new $runtime.Choice(["first", ""]), "."], (($0 : undefined, $1 : undefined, $2 : undefined, $3 : undefined, $4 : undefined, base : Ast.InvocationExpression, $5 : undefined, $6 : undefined, $7 : undefined, $8 : undefined) : (C.ParamSlot[])|null => {
            if (!C.isSameFunction(base.schema!, proj.pslot.schema))
                return null;
            if (proj.pname === 'id')
                return null;
            return [proj.pslot];
        }), C.paramArrayKeyFn, {  });
           if (proj.base) {
               $grammar.declareSymbol("simple_user_search_question");
               $grammar.addRule("simple_user_search_question", ["i would like to", new $runtime.Choice(["know", "see"]), "the", proj.base, new $runtime.Choice(["that", "which", ""]), new $runtime.Choice(["it", "this", "that"]), new $runtime.Choice(["is", "was", "are", "were"]), proj.canonical, new $runtime.Choice(["first", ""]), "."], (($0 : undefined, $1 : undefined, $2 : undefined, $3 : undefined, $4 : undefined, $5 : undefined, $6 : undefined, $7 : undefined, $8 : undefined, $9 : undefined) : (C.ParamSlot[])|null => {
                if (proj.pname === 'id')
                    return null;
                return [proj.pslot];
            }), C.paramArrayKeyFn, {  });
               $grammar.declareSymbol("simple_user_search_question");
               $grammar.addRule("simple_user_search_question", ["i would like to", new $runtime.Choice(["know", "see"]), "the", proj.base, new $runtime.Choice(["that", "which", ""]), "the", new $runtime.NonTerminal("base_table", ["functionName" as ("functionName" extends keyof ReturnType<typeof C.expressionKeyFn> ? string : void), ((proj.pslot.schema.qualifiedName)) as ReturnType<typeof C.expressionKeyFn>["functionName"]]), new $runtime.Choice(["is", "was", "are", "were"]), proj.canonical, new $runtime.Choice(["first", ""]), "."], (($0 : undefined, $1 : undefined, $2 : undefined, $3 : undefined, $4 : undefined, $5 : undefined, base : Ast.InvocationExpression, $6 : undefined, $7 : undefined, $8 : undefined, $9 : undefined) : (C.ParamSlot[])|null => {
                if (!C.isSameFunction(base.schema!, proj.pslot.schema))
                    return null;
                if (proj.pname === 'id')
                    return null;
                return [proj.pslot];
            }), C.paramArrayKeyFn, {  });
           }
       }
       if (proj.category === 'avp') {
           $grammar.declareSymbol("simple_user_search_question");
           $grammar.addRule("simple_user_search_question", [new $runtime.Choice(["can you tell me", ""]), proj.pronoun, proj.base, new $runtime.Choice(["do", "does", "did"]), new $runtime.Choice(["it", "this", "that"]), proj.canonical, "?"], (($0 : undefined, $1 : undefined, $2 : undefined, $3 : undefined, $4 : undefined, $5 : undefined, $6 : undefined) : (C.ParamSlot[])|null => {
            if (proj.pname === 'id')
                return null;
            return [proj.pslot];
        }), C.paramArrayKeyFn, {  });
           $grammar.addRule("simple_user_search_question", ["i would like to", new $runtime.Choice(["know", "see"]), proj.pronoun, proj.base, new $runtime.Choice(["it", "this", "that"]), proj.canonical, new $runtime.Choice(["first", ""]), "."], (($0 : undefined, $1 : undefined, $2 : undefined, $3 : undefined, $4 : undefined, $5 : undefined, $6 : undefined, $7 : undefined) : (C.ParamSlot[])|null => {
            if (proj.pname === 'id')
                return null;
            return [proj.pslot];
        }), C.paramArrayKeyFn, {  });
           $grammar.declareSymbol("simple_user_search_question");
           $grammar.addRule("simple_user_search_question", [new $runtime.Choice(["can you tell me", ""]), proj.pronoun, proj.base, new $runtime.Choice(["do", "does", "did"]), "the", new $runtime.NonTerminal("base_table", ["functionName" as ("functionName" extends keyof ReturnType<typeof C.expressionKeyFn> ? string : void), ((proj.pslot.schema.qualifiedName)) as ReturnType<typeof C.expressionKeyFn>["functionName"]]), proj.canonical, "?"], (($0 : undefined, $1 : undefined, $2 : undefined, $3 : undefined, $4 : undefined, base : Ast.InvocationExpression, $5 : undefined, $6 : undefined) : (C.ParamSlot[])|null => {
            if (!C.isSameFunction(base.schema!, proj.pslot.schema))
                return null;
            if (proj.pname === 'id')
                return null;
            return [proj.pslot];
        }), C.paramArrayKeyFn, {  });
           $grammar.addRule("simple_user_search_question", ["i would like to", new $runtime.Choice(["know", "see"]), proj.pronoun, proj.base, "the", new $runtime.NonTerminal("base_table", undefined), proj.canonical, new $runtime.Choice(["first", ""]), "."], (($0 : undefined, $1 : undefined, $2 : undefined, $3 : undefined, $4 : undefined, base : Ast.InvocationExpression, $5 : undefined, $6 : undefined, $7 : undefined) : (C.ParamSlot[])|null => {
            if (!C.isSameFunction(base.schema!, proj.pslot.schema))
                return null;
            if (proj.pname === 'id')
                return null;
            return [proj.pslot];
        }), C.paramArrayKeyFn, {  });
           if (proj.base) {
               $grammar.declareSymbol("simple_user_search_question");
               $grammar.addRule("simple_user_search_question", ["i would like to", new $runtime.Choice(["know", "see"]), "the", proj.base, new $runtime.Choice(["that", "which", ""]), new $runtime.Choice(["it", "this", "that"]), proj.canonical, new $runtime.Choice(["first", ""]), "."], (($0 : undefined, $1 : undefined, $2 : undefined, $3 : undefined, $4 : undefined, $5 : undefined, $6 : undefined, $7 : undefined, $8 : undefined) : (C.ParamSlot[])|null => {
                if (proj.pname === 'id')
                    return null;
                return [proj.pslot];
            }), C.paramArrayKeyFn, {  });
               $grammar.declareSymbol("simple_user_search_question");
               $grammar.addRule("simple_user_search_question", ["i would like to", new $runtime.Choice(["know", "see"]), "the", proj.base, new $runtime.Choice(["that", "which", ""]), "the", new $runtime.NonTerminal("base_table", ["functionName" as ("functionName" extends keyof ReturnType<typeof C.expressionKeyFn> ? string : void), ((proj.pslot.schema.qualifiedName)) as ReturnType<typeof C.expressionKeyFn>["functionName"]]), proj.canonical, new $runtime.Choice(["first", ""]), "."], (($0 : undefined, $1 : undefined, $2 : undefined, $3 : undefined, $4 : undefined, $5 : undefined, base : Ast.InvocationExpression, $6 : undefined, $7 : undefined, $8 : undefined) : (C.ParamSlot[])|null => {
                if (!C.isSameFunction(base.schema!, proj.pslot.schema))
                    return null;
                if (proj.pname === 'id')
                    return null;
                return [proj.pslot];
            }), C.paramArrayKeyFn, {  });
           }
       }
       if (proj.category === 'reverse_verb') {
           $grammar.declareSymbol("simple_user_search_question");
           $grammar.addRule("simple_user_search_question", [new $runtime.Choice(["can you tell me", ""]), proj.pronoun, proj.base, proj.canonical, new $runtime.Choice(["it", "this", "that"]), "?"], (($0 : undefined, $1 : undefined, $2 : undefined, $3 : undefined, $4 : undefined, $5 : undefined) : (C.ParamSlot[])|null => {
            if (proj.pname === 'id')
                return null;
            return [proj.pslot];
        }), C.paramArrayKeyFn, {  });
           $grammar.addRule("simple_user_search_question", ["i would like to", new $runtime.Choice(["know", "see"]), proj.pronoun, proj.base, proj.canonical, new $runtime.Choice(["it", "this", "that"]), new $runtime.Choice(["first", ""]), "."], (($0 : undefined, $1 : undefined, $2 : undefined, $3 : undefined, $4 : undefined, $5 : undefined, $6 : undefined, $7 : undefined) : (C.ParamSlot[])|null => {
            if (proj.pname === 'id')
                return null;
            return [proj.pslot];
        }), C.paramArrayKeyFn, {  });
           $grammar.declareSymbol("simple_user_search_question");
           $grammar.addRule("simple_user_search_question", [new $runtime.Choice(["can you tell me", ""]), proj.pronoun, proj.base, proj.canonical, "the", new $runtime.NonTerminal("base_table", ["functionName" as ("functionName" extends keyof ReturnType<typeof C.expressionKeyFn> ? string : void), ((proj.pslot.schema.qualifiedName)) as ReturnType<typeof C.expressionKeyFn>["functionName"]]), "?"], (($0 : undefined, $1 : undefined, $2 : undefined, $3 : undefined, $4 : undefined, base : Ast.InvocationExpression, $5 : undefined) : (C.ParamSlot[])|null => {
            if (!C.isSameFunction(base.schema!, proj.pslot.schema))
                return null;
            if (proj.pname === 'id')
                return null;
            return [proj.pslot];
        }), C.paramArrayKeyFn, {  });
           $grammar.addRule("simple_user_search_question", ["i would like to", new $runtime.Choice(["know", "see"]), proj.pronoun, proj.base, proj.canonical, "the", new $runtime.NonTerminal("base_table", undefined), new $runtime.Choice(["first", ""]), "."], (($0 : undefined, $1 : undefined, $2 : undefined, $3 : undefined, $4 : undefined, $5 : undefined, base : Ast.InvocationExpression, $6 : undefined, $7 : undefined) : (C.ParamSlot[])|null => {
            if (!C.isSameFunction(base.schema!, proj.pslot.schema))
                return null;
            if (proj.pname === 'id')
                return null;
            return [proj.pslot];
        }), C.paramArrayKeyFn, {  });
           if (proj.base) {
               $grammar.declareSymbol("simple_user_search_question");
               $grammar.addRule("simple_user_search_question", ["i would like to", new $runtime.Choice(["know", "see"]), "the", proj.base, new $runtime.Choice(["that", "which", ""]), proj.canonical, new $runtime.Choice(["it", "this", "that"]), new $runtime.Choice(["first", ""]), "."], (($0 : undefined, $1 : undefined, $2 : undefined, $3 : undefined, $4 : undefined, $5 : undefined, $6 : undefined, $7 : undefined, $8 : undefined) : (C.ParamSlot[])|null => {
                if (proj.pname === 'id')
                    return null;
                return [proj.pslot];
            }), C.paramArrayKeyFn, {  });
               $grammar.declareSymbol("simple_user_search_question");
               $grammar.addRule("simple_user_search_question", ["i would like to", new $runtime.Choice(["know", "see"]), "the", proj.base, new $runtime.Choice(["that", "which", ""]), proj.canonical, "the", new $runtime.NonTerminal("base_table", ["functionName" as ("functionName" extends keyof ReturnType<typeof C.expressionKeyFn> ? string : void), ((proj.pslot.schema.qualifiedName)) as ReturnType<typeof C.expressionKeyFn>["functionName"]]), new $runtime.Choice(["first", ""]), "."], (($0 : undefined, $1 : undefined, $2 : undefined, $3 : undefined, $4 : undefined, $5 : undefined, $6 : undefined, base : Ast.InvocationExpression, $7 : undefined, $8 : undefined) : (C.ParamSlot[])|null => {
                if (!C.isSameFunction(base.schema!, proj.pslot.schema))
                    return null;
                if (proj.pname === 'id')
                    return null;
                return [proj.pslot];
            }), C.paramArrayKeyFn, {  });
           }
       }
   }
   $grammar.declareSymbol("simple_user_search_question_for_list");
   $grammar.addRule("simple_user_search_question_for_list", [new $runtime.Choice(["can you tell me", ""]), "what are", new $runtime.Choice(["the", "their"]), new $runtime.NonTerminal("out_param_Any", undefined), "?"], (($0 : undefined, $1 : undefined, $2 : undefined, param : C.ParamSlot, $3 : undefined) : (C.ParamSlot[])|null => {
        if (param.name === 'id')
            return null;
        return [param];
    }), C.paramArrayKeyFn, {  });
   $grammar.addRule("simple_user_search_question_for_list", ["can you tell me", new $runtime.Choice(["the", "their"]), new $runtime.NonTerminal("out_param_Any", undefined), "?"], (($0 : undefined, $1 : undefined, param : C.ParamSlot, $2 : undefined) : (C.ParamSlot[])|null => {
        if (param.name === 'id')
            return null;
        return [param];
    }), C.paramArrayKeyFn, {  });
   $grammar.addRule("simple_user_search_question_for_list", ["can you tell me", "the", new $runtime.NonTerminal("out_param_Any", undefined), new $runtime.Choice(["for both of them", "for all of them"]), "?"], (($0 : undefined, $1 : undefined, param : C.ParamSlot, $2 : undefined, $3 : undefined) : (C.ParamSlot[])|null => {
        if (param.name === 'id')
            return null;
        return [param];
    }), C.paramArrayKeyFn, {  });
   $grammar.addRule("simple_user_search_question_for_list", [new $runtime.Choice(["can you tell me", ""]), "what", new $runtime.NonTerminal("out_param_Any", undefined), "does it have ?"], (($0 : undefined, $1 : undefined, param : C.ParamSlot, $2 : undefined) : (C.ParamSlot[])|null => {
        if (param.name === 'id')
            return null;
        return [param];
    }), C.paramArrayKeyFn, {  });
   $grammar.addRule("simple_user_search_question_for_list", ["i would like to", new $runtime.Choice(["know", "see"]), new $runtime.Choice(["both of", ""]), new $runtime.Choice(["the", "their"]), new $runtime.NonTerminal("out_param_Any", undefined), new $runtime.Choice(["first", ""]), "."], (($0 : undefined, $1 : undefined, $2 : undefined, $3 : undefined, param : C.ParamSlot, $4 : undefined, $5 : undefined) : (C.ParamSlot[])|null => {
        if (param.name === 'id')
            return null;
        return [param];
    }), C.paramArrayKeyFn, {  });
   $grammar.addRule("simple_user_search_question_for_list", ["i need", new $runtime.Choice(["the", "their"]), new $runtime.NonTerminal("out_param_Any", undefined), new $runtime.Choice(["first", ""]), "."], (($0 : undefined, $1 : undefined, param : C.ParamSlot, $2 : undefined, $3 : undefined) : (C.ParamSlot[])|null => {
        if (param.name === 'id')
            return null;
        return [param];
    }), C.paramArrayKeyFn, {  });
   $grammar.addRule("simple_user_search_question_for_list", ["i just need", new $runtime.Choice(["the", "their"]), new $runtime.NonTerminal("out_param_Any", undefined), "."], (($0 : undefined, $1 : undefined, param : C.ParamSlot, $2 : undefined) : (C.ParamSlot[])|null => {
        if (param.name === 'id')
            return null;
        return [param];
    }), C.paramArrayKeyFn, {  });
   $grammar.addRule("simple_user_search_question_for_list", [new $runtime.Choice(["can you tell me", ""]), "do", new $runtime.Choice(["they", "these", "those"]), "have", new $runtime.NonTerminal("npp_boolean_projection", undefined), "?"], (($0 : undefined, $1 : undefined, $2 : undefined, $3 : undefined, param : C.ParamSlot, $4 : undefined) : (C.ParamSlot[])|null => {
        return [param];
    }), C.paramArrayKeyFn, {  });
   $grammar.addRule("simple_user_search_question_for_list", [new $runtime.Choice(["can you tell me", ""]), "are", "there", new $runtime.NonTerminal("npp_boolean_projection", undefined), "?"], (($0 : undefined, $1 : undefined, $2 : undefined, param : C.ParamSlot, $3 : undefined) : (C.ParamSlot[])|null => {
        return [param];
    }), C.paramArrayKeyFn, {  });
   $grammar.addRule("simple_user_search_question_for_list", ["i would like to", new $runtime.Choice(["know", "see"]), new $runtime.Choice(["if", "whether"]), new $runtime.Choice(["they", "these", "those"]), "have", new $runtime.NonTerminal("npp_boolean_projection", undefined), new $runtime.Choice(["first", ""]), "."], (($0 : undefined, $1 : undefined, $2 : undefined, $3 : undefined, $4 : undefined, param : C.ParamSlot, $5 : undefined, $6 : undefined) : (C.ParamSlot[])|null => {
        return [param];
    }), C.paramArrayKeyFn, {  });
   $grammar.addRule("simple_user_search_question_for_list", ["i would like to", new $runtime.Choice(["know", "see"]), new $runtime.Choice(["if", "whether"]), "there", "are", new $runtime.NonTerminal("npp_boolean_projection", undefined), new $runtime.Choice(["first", ""]), "."], (($0 : undefined, $1 : undefined, $2 : undefined, $3 : undefined, $4 : undefined, param : C.ParamSlot, $5 : undefined, $6 : undefined) : (C.ParamSlot[])|null => {
        return [param];
    }), C.paramArrayKeyFn, {  });
   $grammar.addRule("simple_user_search_question_for_list", [new $runtime.Choice(["can you tell me", ""]), "do", new $runtime.Choice(["they", "these", "those"]), new $runtime.NonTerminal("avp_boolean_projection", undefined), "?"], (($0 : undefined, $1 : undefined, $2 : undefined, param : C.ParamSlot, $3 : undefined) : (C.ParamSlot[])|null => {
        return [param];
    }), C.paramArrayKeyFn, {  });
   $grammar.addRule("simple_user_search_question_for_list", ["i would like to", new $runtime.Choice(["know", "see"]), new $runtime.Choice(["if", "whether"]), new $runtime.Choice(["they", "these", "those"]), new $runtime.NonTerminal("avp_boolean_projection", undefined), new $runtime.Choice(["first", ""]), "."], (($0 : undefined, $1 : undefined, $2 : undefined, $3 : undefined, param : C.ParamSlot, $4 : undefined, $5 : undefined) : (C.ParamSlot[])|null => {
        return [param];
    }), C.paramArrayKeyFn, {  });
   $grammar.addRule("simple_user_search_question_for_list", [new $runtime.Choice(["can you tell me", ""]), "are", new $runtime.Choice(["they", "these", "those"]), new $runtime.NonTerminal("pvp_boolean_projection", undefined), "?"], (($0 : undefined, $1 : undefined, $2 : undefined, param : C.ParamSlot, $3 : undefined) : (C.ParamSlot[])|null => {
        return [param];
    }), C.paramArrayKeyFn, {  });
   $grammar.addRule("simple_user_search_question_for_list", ["i would like to", new $runtime.Choice(["know", "see"]), new $runtime.Choice(["if", "whether"]), new $runtime.Choice(["they", "these", "those"]), "are", new $runtime.NonTerminal("pvp_boolean_projection", undefined), new $runtime.Choice(["first", ""]), "."], (($0 : undefined, $1 : undefined, $2 : undefined, $3 : undefined, $4 : undefined, param : C.ParamSlot, $5 : undefined, $6 : undefined) : (C.ParamSlot[])|null => {
        return [param];
    }), C.paramArrayKeyFn, {  });
   $grammar.addRule("simple_user_search_question_for_list", [new $runtime.Choice(["can you tell me", ""]), "are", new $runtime.Choice(["they", "these", "those"]), new $runtime.NonTerminal("preposition_boolean_projection", undefined), "?"], (($0 : undefined, $1 : undefined, $2 : undefined, param : C.ParamSlot, $3 : undefined) : (C.ParamSlot[])|null => {
        return [param];
    }), C.paramArrayKeyFn, {  });
   $grammar.addRule("simple_user_search_question_for_list", ["i would like to", new $runtime.Choice(["know", "see"]), new $runtime.Choice(["if", "whether"]), new $runtime.Choice(["they", "these", "those"]), "are", new $runtime.NonTerminal("preposition_boolean_projection", undefined), new $runtime.Choice(["first", ""]), "."], (($0 : undefined, $1 : undefined, $2 : undefined, $3 : undefined, $4 : undefined, param : C.ParamSlot, $5 : undefined, $6 : undefined) : (C.ParamSlot[])|null => {
        return [param];
    }), C.paramArrayKeyFn, {  });
   $grammar.addRule("simple_user_search_question_for_list", [new $runtime.Choice(["can you tell me", ""]), "are", new $runtime.Choice(["they", "these", "those"]), new $runtime.NonTerminal("npi_boolean_projection", undefined), "?"], (($0 : undefined, $1 : undefined, $2 : undefined, param : C.ParamSlot, $3 : undefined) : (C.ParamSlot[])|null => {
        return [param];
    }), C.paramArrayKeyFn, {  });
   $grammar.addRule("simple_user_search_question_for_list", ["i would like to", new $runtime.Choice(["know", "see"]), new $runtime.Choice(["if", "whether"]), new $runtime.Choice(["they", "these", "those"]), "are", new $runtime.NonTerminal("npi_boolean_projection", undefined), new $runtime.Choice(["first", ""]), "."], (($0 : undefined, $1 : undefined, $2 : undefined, $3 : undefined, $4 : undefined, param : C.ParamSlot, $5 : undefined, $6 : undefined) : (C.ParamSlot[])|null => {
        return [param];
    }), C.paramArrayKeyFn, {  });
   $grammar.addRule("simple_user_search_question_for_list", [new $runtime.Choice(["can you tell me", ""]), "are", new $runtime.Choice(["they", "these", "those"]), new $runtime.NonTerminal("npv_boolean_projection", undefined), "?"], (($0 : undefined, $1 : undefined, $2 : undefined, param : C.ParamSlot, $3 : undefined) : (C.ParamSlot[])|null => {
        return [param];
    }), C.paramArrayKeyFn, {  });
   $grammar.addRule("simple_user_search_question_for_list", ["i would like to", new $runtime.Choice(["know", "see"]), new $runtime.Choice(["if", "whether"]), new $runtime.Choice(["they", "these", "those"]), "are", new $runtime.NonTerminal("npv_boolean_projection", undefined), new $runtime.Choice(["first", ""]), "."], (($0 : undefined, $1 : undefined, $2 : undefined, $3 : undefined, $4 : undefined, param : C.ParamSlot, $5 : undefined, $6 : undefined) : (C.ParamSlot[])|null => {
        return [param];
    }), C.paramArrayKeyFn, {  });
   $grammar.addRule("simple_user_search_question_for_list", [new $runtime.Choice(["can you tell me", ""]), "are", new $runtime.Choice(["they", "these", "those"]), new $runtime.NonTerminal("apv_boolean_projection", undefined), "?"], (($0 : undefined, $1 : undefined, $2 : undefined, param : C.ParamSlot, $3 : undefined) : (C.ParamSlot[])|null => {
        return [param];
    }), C.paramArrayKeyFn, {  });
   $grammar.addRule("simple_user_search_question_for_list", ["i would like to", new $runtime.Choice(["know", "see"]), new $runtime.Choice(["if", "whether"]), new $runtime.Choice(["they", "these", "those"]), "are", new $runtime.NonTerminal("apv_boolean_projection", undefined), new $runtime.Choice(["first", ""]), "."], (($0 : undefined, $1 : undefined, $2 : undefined, $3 : undefined, $4 : undefined, param : C.ParamSlot, $5 : undefined, $6 : undefined) : (C.ParamSlot[])|null => {
        return [param];
    }), C.paramArrayKeyFn, {  });
   $grammar.declareSymbol("boolean_user_search_question");
   $grammar.addRule("boolean_user_search_question", ["does", new $runtime.Choice(["that", "it", "this"]), "have", new $runtime.NonTerminal("npp_filter", undefined), "?"], (($0 : undefined, $1 : undefined, $2 : undefined, param : C.FilterSlot, $3 : undefined) : (C.ParamSlot[])|null => D.booleanQuestion(null, param)), C.paramArrayKeyFn, {  });
   $grammar.addRule("boolean_user_search_question", ["does", new $runtime.Choice(["that", "it", "this"]), new $runtime.NonTerminal("avp_filter", undefined), "?"], (($0 : undefined, $1 : undefined, param : C.FilterSlot, $2 : undefined) : (C.ParamSlot[])|null => D.booleanQuestion(null, param)), C.paramArrayKeyFn, {  });
   $grammar.addRule("boolean_user_search_question", ["is", new $runtime.Choice(["that", "it", "this"]), "a", new $runtime.NonTerminal("apv_filter", undefined), new $runtime.NonTerminal("base_table", ["functionName" as ("functionName" extends keyof ReturnType<typeof C.expressionKeyFn> ? string : void), 3, "functionName" as ("functionName" extends keyof ReturnType<typeof C.filterKeyFn> ? string : void)]), "?"], (($0 : undefined, $1 : undefined, $2 : undefined, param : C.FilterSlot, base : Ast.InvocationExpression, $3 : undefined) : (C.ParamSlot[])|null => {
        if (!base.schema!.is_list)
            return null;
        return D.booleanQuestion(base, param);
    }), C.paramArrayKeyFn, {  });
   $grammar.addRule("boolean_user_search_question", ["is", new $runtime.Choice(["the", "this", "that"]), new $runtime.NonTerminal("base_table", ["is_list" as ("is_list" extends keyof ReturnType<typeof C.expressionKeyFn> ? string : void), (true) as ReturnType<typeof C.expressionKeyFn>["is_list"]]), new $runtime.NonTerminal("apv_filter", ["functionName" as ("functionName" extends keyof ReturnType<typeof C.filterKeyFn> ? string : void), 2, "functionName" as ("functionName" extends keyof ReturnType<typeof C.expressionKeyFn> ? string : void)]), "?"], (($0 : undefined, $1 : undefined, base : Ast.InvocationExpression, param : C.FilterSlot, $2 : undefined) : (C.ParamSlot[])|null => {
        if (!base.schema!.is_list)
            return null;
        return D.booleanQuestion(base, param);
    }), C.paramArrayKeyFn, {  });
   $grammar.addRule("boolean_user_search_question", ["is", new $runtime.Choice(["that", "it", "this"]), "a", new $runtime.NonTerminal("base_table", ["is_list" as ("is_list" extends keyof ReturnType<typeof C.expressionKeyFn> ? string : void), (true) as ReturnType<typeof C.expressionKeyFn>["is_list"]]), new $runtime.NonTerminal("pvp_filter", ["functionName" as ("functionName" extends keyof ReturnType<typeof C.filterKeyFn> ? string : void), 3, "functionName" as ("functionName" extends keyof ReturnType<typeof C.expressionKeyFn> ? string : void)]), "?"], (($0 : undefined, $1 : undefined, $2 : undefined, base : Ast.InvocationExpression, param : C.FilterSlot, $3 : undefined) : (C.ParamSlot[])|null => {
        if (!base.schema!.is_list)
            return null;
        return D.booleanQuestion(base, param);
    }), C.paramArrayKeyFn, {  });
   $grammar.addRule("boolean_user_search_question", ["is", new $runtime.Choice(["the", "this", "that"]), new $runtime.NonTerminal("base_table", ["is_list" as ("is_list" extends keyof ReturnType<typeof C.expressionKeyFn> ? string : void), (true) as ReturnType<typeof C.expressionKeyFn>["is_list"]]), new $runtime.NonTerminal("pvp_filter", ["functionName" as ("functionName" extends keyof ReturnType<typeof C.filterKeyFn> ? string : void), 2, "functionName" as ("functionName" extends keyof ReturnType<typeof C.expressionKeyFn> ? string : void)]), "?"], (($0 : undefined, $1 : undefined, base : Ast.InvocationExpression, param : C.FilterSlot, $2 : undefined) : (C.ParamSlot[])|null => {
        if (!base.schema!.is_list)
            return null;
        return D.booleanQuestion(base, param);
    }), C.paramArrayKeyFn, {  });
   $grammar.addRule("boolean_user_search_question", ["is", new $runtime.Choice(["that", "it", "this"]), "a", new $runtime.NonTerminal("base_table", ["is_list" as ("is_list" extends keyof ReturnType<typeof C.expressionKeyFn> ? string : void), (true) as ReturnType<typeof C.expressionKeyFn>["is_list"]]), new $runtime.NonTerminal("preposition_filter", ["functionName" as ("functionName" extends keyof ReturnType<typeof C.filterKeyFn> ? string : void), 3, "functionName" as ("functionName" extends keyof ReturnType<typeof C.expressionKeyFn> ? string : void)]), "?"], (($0 : undefined, $1 : undefined, $2 : undefined, base : Ast.InvocationExpression, param : C.FilterSlot, $3 : undefined) : (C.ParamSlot[])|null => {
        if (!base.schema!.is_list)
            return null;
        return D.booleanQuestion(base, param);
    }), C.paramArrayKeyFn, {  });
   $grammar.addRule("boolean_user_search_question", ["is", new $runtime.Choice(["the", "this", "that"]), new $runtime.NonTerminal("base_table", ["is_list" as ("is_list" extends keyof ReturnType<typeof C.expressionKeyFn> ? string : void), (true) as ReturnType<typeof C.expressionKeyFn>["is_list"]]), new $runtime.NonTerminal("preposition_filter", ["functionName" as ("functionName" extends keyof ReturnType<typeof C.filterKeyFn> ? string : void), 2, "functionName" as ("functionName" extends keyof ReturnType<typeof C.expressionKeyFn> ? string : void)]), "?"], (($0 : undefined, $1 : undefined, base : Ast.InvocationExpression, param : C.FilterSlot, $2 : undefined) : (C.ParamSlot[])|null => {
        if (!base.schema!.is_list)
            return null;
        return D.booleanQuestion(base, param);
    }), C.paramArrayKeyFn, {  });
   $grammar.addRule("boolean_user_search_question", ["is", new $runtime.Choice(["that", "it", "this"]), "a", new $runtime.NonTerminal("base_table", ["is_list" as ("is_list" extends keyof ReturnType<typeof C.expressionKeyFn> ? string : void), (true) as ReturnType<typeof C.expressionKeyFn>["is_list"]]), new $runtime.Choice(["that", "which"]), new $runtime.NonTerminal("avp_filter", ["functionName" as ("functionName" extends keyof ReturnType<typeof C.filterKeyFn> ? string : void), 3, "functionName" as ("functionName" extends keyof ReturnType<typeof C.expressionKeyFn> ? string : void)]), "?"], (($0 : undefined, $1 : undefined, $2 : undefined, base : Ast.InvocationExpression, $3 : undefined, param : C.FilterSlot, $4 : undefined) : (C.ParamSlot[])|null => {
        if (!base.schema!.is_list)
            return null;
        return D.booleanQuestion(base, param);
    }), C.paramArrayKeyFn, {  });
   $grammar.addRule("boolean_user_search_question", ["does", new $runtime.Choice(["the", "this", "that"]), new $runtime.NonTerminal("base_table", ["is_list" as ("is_list" extends keyof ReturnType<typeof C.expressionKeyFn> ? string : void), (true) as ReturnType<typeof C.expressionKeyFn>["is_list"]]), new $runtime.NonTerminal("avp_filter", ["functionName" as ("functionName" extends keyof ReturnType<typeof C.filterKeyFn> ? string : void), 2, "functionName" as ("functionName" extends keyof ReturnType<typeof C.expressionKeyFn> ? string : void)]), "?"], (($0 : undefined, $1 : undefined, base : Ast.InvocationExpression, param : C.FilterSlot, $2 : undefined) : (C.ParamSlot[])|null => {
        if (!base.schema!.is_list)
            return null;
        return D.booleanQuestion(base, param);
    }), C.paramArrayKeyFn, {  });
   $grammar.addRule("boolean_user_search_question", ["is", new $runtime.Choice(["that", "it", "this"]), "a", new $runtime.NonTerminal("base_table", ["is_list" as ("is_list" extends keyof ReturnType<typeof C.expressionKeyFn> ? string : void), (true) as ReturnType<typeof C.expressionKeyFn>["is_list"]]), new $runtime.Choice(["that has", "with"]), new $runtime.NonTerminal("npp_filter", ["functionName" as ("functionName" extends keyof ReturnType<typeof C.filterKeyFn> ? string : void), 3, "functionName" as ("functionName" extends keyof ReturnType<typeof C.expressionKeyFn> ? string : void)]), "?"], (($0 : undefined, $1 : undefined, $2 : undefined, base : Ast.InvocationExpression, $3 : undefined, param : C.FilterSlot, $4 : undefined) : (C.ParamSlot[])|null => {
        if (!base.schema!.is_list)
            return null;
        return D.booleanQuestion(base, param);
    }), C.paramArrayKeyFn, {  });
   $grammar.addRule("boolean_user_search_question", ["does", new $runtime.Choice(["the", "this", "that"]), new $runtime.NonTerminal("base_table", ["is_list" as ("is_list" extends keyof ReturnType<typeof C.expressionKeyFn> ? string : void), (true) as ReturnType<typeof C.expressionKeyFn>["is_list"]]), "have", new $runtime.NonTerminal("npp_filter", ["functionName" as ("functionName" extends keyof ReturnType<typeof C.filterKeyFn> ? string : void), 2, "functionName" as ("functionName" extends keyof ReturnType<typeof C.expressionKeyFn> ? string : void)]), "?"], (($0 : undefined, $1 : undefined, base : Ast.InvocationExpression, $2 : undefined, param : C.FilterSlot, $3 : undefined) : (C.ParamSlot[])|null => {
        if (!base.schema!.is_list)
            return null;
        return D.booleanQuestion(base, param);
    }), C.paramArrayKeyFn, {  });
   $grammar.declareSymbol("boolean_user_search_question_for_list");
   $grammar.addRule("boolean_user_search_question_for_list", [new $runtime.Choice(["does either of", "does any of"]), new $runtime.Choice(["them", "these", "those"]), "have", new $runtime.NonTerminal("npp_filter", undefined), "?"], (($0 : undefined, $1 : undefined, $2 : undefined, param : C.FilterSlot, $3 : undefined) : (C.ParamSlot[])|null => D.booleanQuestion(null, param)), C.paramArrayKeyFn, {  });
   $grammar.addRule("boolean_user_search_question_for_list", ["do", "they", "have", new $runtime.NonTerminal("npp_filter", undefined), "?"], (($0 : undefined, $1 : undefined, $2 : undefined, param : C.FilterSlot, $3 : undefined) : (C.ParamSlot[])|null => D.booleanQuestion(null, param)), C.paramArrayKeyFn, {  });
   $grammar.addRule("boolean_user_search_question_for_list", [new $runtime.Choice(["does either of", "does any of"]), new $runtime.Choice(["them", "these", "those"]), new $runtime.NonTerminal("avp_filter", undefined), "?"], (($0 : undefined, $1 : undefined, param : C.FilterSlot, $2 : undefined) : (C.ParamSlot[])|null => D.booleanQuestion(null, param)), C.paramArrayKeyFn, {  });
   $grammar.addRule("boolean_user_search_question_for_list", ["do they", new $runtime.NonTerminal("avp_filter", undefined), "?"], (($0 : undefined, param : C.FilterSlot, $1 : undefined) : (C.ParamSlot[])|null => D.booleanQuestion(null, param)), C.paramArrayKeyFn, {  });
   $grammar.addRule("boolean_user_search_question_for_list", [new $runtime.Choice(["is either of", "is any of"]), new $runtime.Choice(["them", "these", "those"]), "a", new $runtime.NonTerminal("apv_filter", undefined), new $runtime.NonTerminal("base_table", ["functionName" as ("functionName" extends keyof ReturnType<typeof C.expressionKeyFn> ? string : void), 3, "functionName" as ("functionName" extends keyof ReturnType<typeof C.filterKeyFn> ? string : void)]), "?"], (($0 : undefined, $1 : undefined, $2 : undefined, param : C.FilterSlot, base : Ast.InvocationExpression, $3 : undefined) : (C.ParamSlot[])|null => D.booleanQuestion(base, param)), C.paramArrayKeyFn, {  });
   $grammar.addRule("boolean_user_search_question_for_list", ["are they", new $runtime.NonTerminal("apv_filter", undefined), new $runtime.NonTerminal("base_table", ["functionName" as ("functionName" extends keyof ReturnType<typeof C.expressionKeyFn> ? string : void), 1, "functionName" as ("functionName" extends keyof ReturnType<typeof C.filterKeyFn> ? string : void)]), "?"], (($0 : undefined, param : C.FilterSlot, base : Ast.InvocationExpression, $1 : undefined) : (C.ParamSlot[])|null => D.booleanQuestion(base, param)), C.paramArrayKeyFn, {  });
   $grammar.addRule("boolean_user_search_question_for_list", ["are", new $runtime.Choice(["the", "these", "those"]), new $runtime.NonTerminal("base_table", undefined), new $runtime.NonTerminal("apv_filter", ["functionName" as ("functionName" extends keyof ReturnType<typeof C.filterKeyFn> ? string : void), 2, "functionName" as ("functionName" extends keyof ReturnType<typeof C.expressionKeyFn> ? string : void)]), "?"], (($0 : undefined, $1 : undefined, base : Ast.InvocationExpression, param : C.FilterSlot, $2 : undefined) : (C.ParamSlot[])|null => D.booleanQuestion(base, param)), C.paramArrayKeyFn, {  });
   $grammar.addRule("boolean_user_search_question_for_list", [new $runtime.Choice(["is either of", "is any of"]), new $runtime.Choice(["them", "these", "those"]), "a", new $runtime.NonTerminal("base_table", undefined), new $runtime.NonTerminal("pvp_filter", ["functionName" as ("functionName" extends keyof ReturnType<typeof C.filterKeyFn> ? string : void), 3, "functionName" as ("functionName" extends keyof ReturnType<typeof C.expressionKeyFn> ? string : void)]), "?"], (($0 : undefined, $1 : undefined, $2 : undefined, base : Ast.InvocationExpression, param : C.FilterSlot, $3 : undefined) : (C.ParamSlot[])|null => D.booleanQuestion(base, param)), C.paramArrayKeyFn, {  });
   $grammar.addRule("boolean_user_search_question_for_list", ["are they", new $runtime.NonTerminal("base_table", undefined), new $runtime.NonTerminal("pvp_filter", ["functionName" as ("functionName" extends keyof ReturnType<typeof C.filterKeyFn> ? string : void), 1, "functionName" as ("functionName" extends keyof ReturnType<typeof C.expressionKeyFn> ? string : void)]), "?"], (($0 : undefined, base : Ast.InvocationExpression, param : C.FilterSlot, $1 : undefined) : (C.ParamSlot[])|null => D.booleanQuestion(base, param)), C.paramArrayKeyFn, {  });
   $grammar.addRule("boolean_user_search_question_for_list", ["are", new $runtime.Choice(["the", "these", "those"]), new $runtime.NonTerminal("base_table", undefined), new $runtime.NonTerminal("pvp_filter", ["functionName" as ("functionName" extends keyof ReturnType<typeof C.filterKeyFn> ? string : void), 2, "functionName" as ("functionName" extends keyof ReturnType<typeof C.expressionKeyFn> ? string : void)]), "?"], (($0 : undefined, $1 : undefined, base : Ast.InvocationExpression, param : C.FilterSlot, $2 : undefined) : (C.ParamSlot[])|null => D.booleanQuestion(base, param)), C.paramArrayKeyFn, {  });
   $grammar.addRule("boolean_user_search_question_for_list", [new $runtime.Choice(["is either of", "is any of"]), new $runtime.Choice(["them", "these", "those"]), "a", new $runtime.NonTerminal("base_table", undefined), new $runtime.NonTerminal("preposition_filter", ["functionName" as ("functionName" extends keyof ReturnType<typeof C.filterKeyFn> ? string : void), 3, "functionName" as ("functionName" extends keyof ReturnType<typeof C.expressionKeyFn> ? string : void)]), "?"], (($0 : undefined, $1 : undefined, $2 : undefined, base : Ast.InvocationExpression, param : C.FilterSlot, $3 : undefined) : (C.ParamSlot[])|null => D.booleanQuestion(base, param)), C.paramArrayKeyFn, {  });
   $grammar.addRule("boolean_user_search_question_for_list", ["are they", new $runtime.NonTerminal("base_table", undefined), new $runtime.NonTerminal("preposition_filter", ["functionName" as ("functionName" extends keyof ReturnType<typeof C.filterKeyFn> ? string : void), 1, "functionName" as ("functionName" extends keyof ReturnType<typeof C.expressionKeyFn> ? string : void)]), "?"], (($0 : undefined, base : Ast.InvocationExpression, param : C.FilterSlot, $1 : undefined) : (C.ParamSlot[])|null => D.booleanQuestion(base, param)), C.paramArrayKeyFn, {  });
   $grammar.addRule("boolean_user_search_question_for_list", ["are", new $runtime.Choice(["the", "these", "those"]), new $runtime.NonTerminal("base_table", undefined), new $runtime.NonTerminal("preposition_filter", ["functionName" as ("functionName" extends keyof ReturnType<typeof C.filterKeyFn> ? string : void), 2, "functionName" as ("functionName" extends keyof ReturnType<typeof C.expressionKeyFn> ? string : void)]), "?"], (($0 : undefined, $1 : undefined, base : Ast.InvocationExpression, param : C.FilterSlot, $2 : undefined) : (C.ParamSlot[])|null => D.booleanQuestion(base, param)), C.paramArrayKeyFn, {  });
   $grammar.addRule("boolean_user_search_question_for_list", [new $runtime.Choice(["is either of", "is any of"]), new $runtime.Choice(["them", "these", "those"]), "a", new $runtime.NonTerminal("base_table", undefined), new $runtime.Choice(["that", "which"]), new $runtime.NonTerminal("avp_filter", ["functionName" as ("functionName" extends keyof ReturnType<typeof C.filterKeyFn> ? string : void), 3, "functionName" as ("functionName" extends keyof ReturnType<typeof C.expressionKeyFn> ? string : void)]), "?"], (($0 : undefined, $1 : undefined, $2 : undefined, base : Ast.InvocationExpression, $3 : undefined, param : C.FilterSlot, $4 : undefined) : (C.ParamSlot[])|null => D.booleanQuestion(base, param)), C.paramArrayKeyFn, {  });
   $grammar.addRule("boolean_user_search_question_for_list", ["are they", new $runtime.NonTerminal("base_table", undefined), new $runtime.Choice(["that", "which"]), new $runtime.NonTerminal("avp_filter", ["functionName" as ("functionName" extends keyof ReturnType<typeof C.filterKeyFn> ? string : void), 1, "functionName" as ("functionName" extends keyof ReturnType<typeof C.expressionKeyFn> ? string : void)]), "?"], (($0 : undefined, base : Ast.InvocationExpression, $1 : undefined, param : C.FilterSlot, $2 : undefined) : (C.ParamSlot[])|null => D.booleanQuestion(base, param)), C.paramArrayKeyFn, {  });
   $grammar.addRule("boolean_user_search_question_for_list", ["do", new $runtime.Choice(["the", "these", "those"]), new $runtime.NonTerminal("base_table", undefined), new $runtime.NonTerminal("avp_filter", ["functionName" as ("functionName" extends keyof ReturnType<typeof C.filterKeyFn> ? string : void), 2, "functionName" as ("functionName" extends keyof ReturnType<typeof C.expressionKeyFn> ? string : void)]), "?"], (($0 : undefined, $1 : undefined, base : Ast.InvocationExpression, param : C.FilterSlot, $2 : undefined) : (C.ParamSlot[])|null => D.booleanQuestion(base, param)), C.paramArrayKeyFn, {  });
   $grammar.addRule("boolean_user_search_question_for_list", [new $runtime.Choice(["is either of", "is any of"]), new $runtime.Choice(["them", "these", "those"]), "a", new $runtime.NonTerminal("base_table", undefined), new $runtime.Choice(["that has", "with"]), new $runtime.NonTerminal("npp_filter", ["functionName" as ("functionName" extends keyof ReturnType<typeof C.filterKeyFn> ? string : void), 3, "functionName" as ("functionName" extends keyof ReturnType<typeof C.expressionKeyFn> ? string : void)]), "?"], (($0 : undefined, $1 : undefined, $2 : undefined, base : Ast.InvocationExpression, $3 : undefined, param : C.FilterSlot, $4 : undefined) : (C.ParamSlot[])|null => D.booleanQuestion(base, param)), C.paramArrayKeyFn, {  });
   $grammar.addRule("boolean_user_search_question_for_list", ["are they", new $runtime.NonTerminal("base_table", undefined), new $runtime.Choice(["that has", "with"]), new $runtime.NonTerminal("npp_filter", ["functionName" as ("functionName" extends keyof ReturnType<typeof C.filterKeyFn> ? string : void), 1, "functionName" as ("functionName" extends keyof ReturnType<typeof C.expressionKeyFn> ? string : void)]), "?"], (($0 : undefined, base : Ast.InvocationExpression, $1 : undefined, param : C.FilterSlot, $2 : undefined) : (C.ParamSlot[])|null => D.booleanQuestion(base, param)), C.paramArrayKeyFn, {  });
   $grammar.addRule("boolean_user_search_question_for_list", ["do", new $runtime.Choice(["the", "these", "those"]), new $runtime.NonTerminal("base_table", undefined), "have", new $runtime.NonTerminal("npp_filter", ["functionName" as ("functionName" extends keyof ReturnType<typeof C.filterKeyFn> ? string : void), 2, "functionName" as ("functionName" extends keyof ReturnType<typeof C.expressionKeyFn> ? string : void)]), "?"], (($0 : undefined, $1 : undefined, base : Ast.InvocationExpression, $2 : undefined, param : C.FilterSlot, $3 : undefined) : (C.ParamSlot[])|null => D.booleanQuestion(base, param)), C.paramArrayKeyFn, {  });
   $grammar.declareSymbol("user_search_question_with_name");
   $grammar.addRule("user_search_question_with_name", [new $runtime.Choice(["can you tell me", ""]), "what is", new $runtime.NonTerminal("constant_name", undefined), "'s", new $runtime.NonTerminal("out_param_Any", ["associatedIdType" as ("associatedIdType" extends keyof ReturnType<typeof C.paramKeyFn> ? string : void), 2, "type" as ("type" extends keyof ReturnType<typeof C.entityOrNumberValueKeyFn> ? string : void)]), "?"], (($0 : undefined, $1 : undefined, name : Ast.EntityValue|Ast.NumberValue, $2 : undefined, param : C.ParamSlot, $3 : undefined) : ([Ast.EntityValue|Ast.NumberValue, C.ParamSlot[]])|null => {
        if (param.name === 'id')
            return null;
        return [name, [param]];
    }), undefined, {  });
   $grammar.addRule("user_search_question_with_name", [new $runtime.Choice(["can you tell me", ""]), "what is", "the", new $runtime.NonTerminal("out_param_Any", undefined), "of", new $runtime.NonTerminal("constant_name", ["type" as ("type" extends keyof ReturnType<typeof C.entityOrNumberValueKeyFn> ? string : void), 3, "associatedIdType" as ("associatedIdType" extends keyof ReturnType<typeof C.paramKeyFn> ? string : void)]), "?"], (($0 : undefined, $1 : undefined, $2 : undefined, param : C.ParamSlot, $3 : undefined, name : Ast.EntityValue|Ast.NumberValue, $4 : undefined) : ([Ast.EntityValue|Ast.NumberValue, C.ParamSlot[]])|null => {
        if (param.name === 'id')
            return null;
        return [name, [param]];
    }), undefined, {  });
   $grammar.addRule("user_search_question_with_name", ["can you tell me", new $runtime.NonTerminal("constant_name", undefined), "'s", new $runtime.NonTerminal("out_param_Any", ["associatedIdType" as ("associatedIdType" extends keyof ReturnType<typeof C.paramKeyFn> ? string : void), 1, "type" as ("type" extends keyof ReturnType<typeof C.entityOrNumberValueKeyFn> ? string : void)]), "?"], (($0 : undefined, name : Ast.EntityValue|Ast.NumberValue, $1 : undefined, param : C.ParamSlot, $2 : undefined) : ([Ast.EntityValue|Ast.NumberValue, C.ParamSlot[]])|null => {
        if (param.name === 'id')
            return null;
        return [name, [param]];
    }), undefined, {  });
   $grammar.addRule("user_search_question_with_name", ["can you tell me", "the", new $runtime.NonTerminal("out_param_Any", undefined), "of", new $runtime.NonTerminal("constant_name", ["type" as ("type" extends keyof ReturnType<typeof C.entityOrNumberValueKeyFn> ? string : void), 2, "associatedIdType" as ("associatedIdType" extends keyof ReturnType<typeof C.paramKeyFn> ? string : void)]), "?"], (($0 : undefined, $1 : undefined, param : C.ParamSlot, $2 : undefined, name : Ast.EntityValue|Ast.NumberValue, $3 : undefined) : ([Ast.EntityValue|Ast.NumberValue, C.ParamSlot[]])|null => {
        if (param.name === 'id')
            return null;
        return [name, [param]];
    }), undefined, {  });
   $grammar.addRule("user_search_question_with_name", ["i would like to", new $runtime.Choice(["know", "see"]), new $runtime.NonTerminal("constant_name", undefined), "'s", new $runtime.NonTerminal("out_param_Any", ["associatedIdType" as ("associatedIdType" extends keyof ReturnType<typeof C.paramKeyFn> ? string : void), 2, "type" as ("type" extends keyof ReturnType<typeof C.entityOrNumberValueKeyFn> ? string : void)]), new $runtime.Choice(["first", ""]), "."], (($0 : undefined, $1 : undefined, name : Ast.EntityValue|Ast.NumberValue, $2 : undefined, param : C.ParamSlot, $3 : undefined, $4 : undefined) : ([Ast.EntityValue|Ast.NumberValue, C.ParamSlot[]])|null => {
        if (param.name === 'id')
            return null;
        return [name, [param]];
    }), undefined, {  });
   $grammar.addRule("user_search_question_with_name", ["i would like to", new $runtime.Choice(["know", "see"]), "the", new $runtime.NonTerminal("out_param_Any", undefined), "of", new $runtime.NonTerminal("constant_name", ["type" as ("type" extends keyof ReturnType<typeof C.entityOrNumberValueKeyFn> ? string : void), 3, "associatedIdType" as ("associatedIdType" extends keyof ReturnType<typeof C.paramKeyFn> ? string : void)]), new $runtime.Choice(["first", ""]), "."], (($0 : undefined, $1 : undefined, $2 : undefined, param : C.ParamSlot, $3 : undefined, name : Ast.EntityValue|Ast.NumberValue, $4 : undefined, $5 : undefined) : ([Ast.EntityValue|Ast.NumberValue, C.ParamSlot[]])|null => {
        if (param.name === 'id')
            return null;
        return [name, [param]];
    }), undefined, {  });
   $grammar.addRule("user_search_question_with_name", [new $runtime.Choice(["can you tell me", ""]), "what is", new $runtime.NonTerminal("constant_name", undefined), "'s", new $runtime.NonTerminal("out_param_list_for_coref_question", ["associatedIdType" as ("associatedIdType" extends keyof ReturnType<typeof C.paramArrayKeyFn> ? string : void), 2, "type" as ("type" extends keyof ReturnType<typeof C.entityOrNumberValueKeyFn> ? string : void)]), "?"], (($0 : undefined, $1 : undefined, name : Ast.EntityValue|Ast.NumberValue, $2 : undefined, params : C.ParamSlot[], $3 : undefined) : ([Ast.EntityValue|Ast.NumberValue, C.ParamSlot[]])|null => {
        return [name, params];
    }), undefined, {  });
   $grammar.addRule("user_search_question_with_name", [new $runtime.Choice(["can you tell me", ""]), "what is", "the", new $runtime.NonTerminal("out_param_list_for_coref_question", undefined), "of", new $runtime.NonTerminal("constant_name", ["type" as ("type" extends keyof ReturnType<typeof C.entityOrNumberValueKeyFn> ? string : void), 3, "associatedIdType" as ("associatedIdType" extends keyof ReturnType<typeof C.paramArrayKeyFn> ? string : void)]), "?"], (($0 : undefined, $1 : undefined, $2 : undefined, params : C.ParamSlot[], $3 : undefined, name : Ast.EntityValue|Ast.NumberValue, $4 : undefined) : ([Ast.EntityValue|Ast.NumberValue, C.ParamSlot[]])|null => {
        return [name, params];
    }), undefined, {  });
   $grammar.addRule("user_search_question_with_name", ["can you tell me", new $runtime.NonTerminal("constant_name", undefined), "'s", new $runtime.NonTerminal("out_param_list_for_coref_question", ["associatedIdType" as ("associatedIdType" extends keyof ReturnType<typeof C.paramArrayKeyFn> ? string : void), 1, "type" as ("type" extends keyof ReturnType<typeof C.entityOrNumberValueKeyFn> ? string : void)]), "?"], (($0 : undefined, name : Ast.EntityValue|Ast.NumberValue, $1 : undefined, params : C.ParamSlot[], $2 : undefined) : ([Ast.EntityValue|Ast.NumberValue, C.ParamSlot[]])|null => {
        return [name, params];
    }), undefined, {  });
   $grammar.addRule("user_search_question_with_name", ["can you tell me", "the", new $runtime.NonTerminal("out_param_list_for_coref_question", undefined), "of", new $runtime.NonTerminal("constant_name", ["type" as ("type" extends keyof ReturnType<typeof C.entityOrNumberValueKeyFn> ? string : void), 2, "associatedIdType" as ("associatedIdType" extends keyof ReturnType<typeof C.paramArrayKeyFn> ? string : void)]), "?"], (($0 : undefined, $1 : undefined, params : C.ParamSlot[], $2 : undefined, name : Ast.EntityValue|Ast.NumberValue, $3 : undefined) : ([Ast.EntityValue|Ast.NumberValue, C.ParamSlot[]])|null => {
        return [name, params];
    }), undefined, {  });
   $grammar.addRule("user_search_question_with_name", ["i would like to", new $runtime.Choice(["know", "see"]), new $runtime.NonTerminal("constant_name", undefined), "'s", new $runtime.NonTerminal("out_param_list_for_coref_question", ["associatedIdType" as ("associatedIdType" extends keyof ReturnType<typeof C.paramArrayKeyFn> ? string : void), 2, "type" as ("type" extends keyof ReturnType<typeof C.entityOrNumberValueKeyFn> ? string : void)]), new $runtime.Choice(["first", ""]), "."], (($0 : undefined, $1 : undefined, name : Ast.EntityValue|Ast.NumberValue, $2 : undefined, params : C.ParamSlot[], $3 : undefined, $4 : undefined) : ([Ast.EntityValue|Ast.NumberValue, C.ParamSlot[]])|null => {
        return [name, params];
    }), undefined, {  });
   $grammar.addRule("user_search_question_with_name", ["i would like to", new $runtime.Choice(["know", "see"]), "the", new $runtime.NonTerminal("out_param_list_for_coref_question", undefined), "of", new $runtime.NonTerminal("constant_name", ["type" as ("type" extends keyof ReturnType<typeof C.entityOrNumberValueKeyFn> ? string : void), 3, "associatedIdType" as ("associatedIdType" extends keyof ReturnType<typeof C.paramArrayKeyFn> ? string : void)]), new $runtime.Choice(["first", ""]), "."], (($0 : undefined, $1 : undefined, $2 : undefined, params : C.ParamSlot[], $3 : undefined, name : Ast.EntityValue|Ast.NumberValue, $4 : undefined, $5 : undefined) : ([Ast.EntityValue|Ast.NumberValue, C.ParamSlot[]])|null => {
        return [name, params];
    }), undefined, {  });
   $grammar.declareSymbol("boolean_user_search_question_with_name");
   $grammar.addRule("boolean_user_search_question_with_name", ["does", new $runtime.NonTerminal("constant_name", undefined), "have", new $runtime.NonTerminal("npp_filter", ["associatedIdType" as ("associatedIdType" extends keyof ReturnType<typeof C.filterKeyFn> ? string : void), 1, "type" as ("type" extends keyof ReturnType<typeof C.entityOrNumberValueKeyFn> ? string : void)]), "?"], (($0 : undefined, name : Ast.EntityValue|Ast.NumberValue, $1 : undefined, param : C.FilterSlot, $2 : undefined) : ([Ast.EntityValue|Ast.NumberValue, C.ParamSlot[]])|null => {
        const idArg = param.schema.getArgument('id');
        if (!idArg || !idArg.type.equals(name.getType()))
            return null;
        const questions = D.booleanQuestion(null, param);
        if (!questions)
            return null;
        return [name, questions];
    }), undefined, {  });
   $grammar.addRule("boolean_user_search_question_with_name", ["does", new $runtime.NonTerminal("constant_name", undefined), new $runtime.NonTerminal("avp_filter", ["associatedIdType" as ("associatedIdType" extends keyof ReturnType<typeof C.filterKeyFn> ? string : void), 1, "type" as ("type" extends keyof ReturnType<typeof C.entityOrNumberValueKeyFn> ? string : void)]), "?"], (($0 : undefined, name : Ast.EntityValue|Ast.NumberValue, param : C.FilterSlot, $1 : undefined) : ([Ast.EntityValue|Ast.NumberValue, C.ParamSlot[]])|null => {
        const idArg = param.schema.getArgument('id');
        if (!idArg || !idArg.type.equals(name.getType()))
            return null;
        const questions = D.booleanQuestion(null, param);
        if (!questions)
            return null;
        return [name, questions];
    }), undefined, {  });
   $grammar.addRule("boolean_user_search_question_with_name", ["is", new $runtime.NonTerminal("constant_name", undefined), new $runtime.NonTerminal("apv_filter", ["associatedIdType" as ("associatedIdType" extends keyof ReturnType<typeof C.filterKeyFn> ? string : void), 1, "type" as ("type" extends keyof ReturnType<typeof C.entityOrNumberValueKeyFn> ? string : void)]), "?"], (($0 : undefined, name : Ast.EntityValue|Ast.NumberValue, param : C.FilterSlot, $1 : undefined) : ([Ast.EntityValue|Ast.NumberValue, C.ParamSlot[]])|null => {
        const idArg = param.schema.getArgument('id');
        if (!idArg || !idArg.type.equals(name.getType()))
            return null;
        const questions = D.booleanQuestion(null, param);
        if (!questions)
            return null;
        return [name, questions];
    }), undefined, {  });
   $grammar.addRule("boolean_user_search_question_with_name", ["is", new $runtime.NonTerminal("constant_name", undefined), new $runtime.NonTerminal("pvp_filter", ["associatedIdType" as ("associatedIdType" extends keyof ReturnType<typeof C.filterKeyFn> ? string : void), 1, "type" as ("type" extends keyof ReturnType<typeof C.entityOrNumberValueKeyFn> ? string : void)]), "?"], (($0 : undefined, name : Ast.EntityValue|Ast.NumberValue, param : C.FilterSlot, $1 : undefined) : ([Ast.EntityValue|Ast.NumberValue, C.ParamSlot[]])|null => {
        const idArg = param.schema.getArgument('id');
        if (!idArg || !idArg.type.equals(name.getType()))
            return null;
        const questions = D.booleanQuestion(null, param);
        if (!questions)
            return null;
        return [name, questions];
    }), undefined, {  });
   $grammar.addRule("boolean_user_search_question_with_name", ["is", new $runtime.NonTerminal("constant_name", undefined), new $runtime.NonTerminal("preposition_filter", ["associatedIdType" as ("associatedIdType" extends keyof ReturnType<typeof C.filterKeyFn> ? string : void), 1, "type" as ("type" extends keyof ReturnType<typeof C.entityOrNumberValueKeyFn> ? string : void)]), "?"], (($0 : undefined, name : Ast.EntityValue|Ast.NumberValue, param : C.FilterSlot, $1 : undefined) : ([Ast.EntityValue|Ast.NumberValue, C.ParamSlot[]])|null => {
        const idArg = param.schema.getArgument('id');
        if (!idArg || !idArg.type.equals(name.getType()))
            return null;
        const questions = D.booleanQuestion(null, param);
        if (!questions)
            return null;
        return [name, questions];
    }), undefined, {  });
   $grammar.addRule("boolean_user_search_question_with_name", ["is", new $runtime.NonTerminal("constant_name", undefined), "a", new $runtime.NonTerminal("apv_filter", ["associatedIdType" as ("associatedIdType" extends keyof ReturnType<typeof C.filterKeyFn> ? string : void), 1, "type" as ("type" extends keyof ReturnType<typeof C.entityOrNumberValueKeyFn> ? string : void)]), new $runtime.NonTerminal("base_table", ["functionName" as ("functionName" extends keyof ReturnType<typeof C.expressionKeyFn> ? string : void), 3, "functionName" as ("functionName" extends keyof ReturnType<typeof C.filterKeyFn> ? string : void)]), "?"], (($0 : undefined, name : Ast.EntityValue|Ast.NumberValue, $1 : undefined, param : C.FilterSlot, base : Ast.InvocationExpression, $2 : undefined) : ([Ast.EntityValue|Ast.NumberValue, C.ParamSlot[]])|null => {
        const idArg = param.schema.getArgument('id');
        if (!idArg || !idArg.type.equals(name.getType()))
            return null;
        const questions = D.booleanQuestion(base, param);
        if (!questions)
            return null;
        return [name, questions];
    }), undefined, {  });
   $grammar.addRule("boolean_user_search_question_with_name", ["is", new $runtime.NonTerminal("constant_name", undefined), "a", new $runtime.NonTerminal("base_table", ["idType" as ("idType" extends keyof ReturnType<typeof C.expressionKeyFn> ? string : void), 1, "type" as ("type" extends keyof ReturnType<typeof C.entityOrNumberValueKeyFn> ? string : void)]), new $runtime.NonTerminal("pvp_filter", ["functionName" as ("functionName" extends keyof ReturnType<typeof C.filterKeyFn> ? string : void), 3, "functionName" as ("functionName" extends keyof ReturnType<typeof C.expressionKeyFn> ? string : void)]), "?"], (($0 : undefined, name : Ast.EntityValue|Ast.NumberValue, $1 : undefined, base : Ast.InvocationExpression, param : C.FilterSlot, $2 : undefined) : ([Ast.EntityValue|Ast.NumberValue, C.ParamSlot[]])|null => {
        const idArg = param.schema.getArgument('id');
        if (!idArg || !idArg.type.equals(name.getType()))
            return null;
        const questions = D.booleanQuestion(base, param);
        if (!questions)
            return null;
        return [name, questions];
    }), undefined, {  });
   $grammar.addRule("boolean_user_search_question_with_name", ["is", new $runtime.NonTerminal("constant_name", undefined), "a", new $runtime.NonTerminal("base_table", ["idType" as ("idType" extends keyof ReturnType<typeof C.expressionKeyFn> ? string : void), 1, "type" as ("type" extends keyof ReturnType<typeof C.entityOrNumberValueKeyFn> ? string : void)]), new $runtime.NonTerminal("preposition_filter", ["functionName" as ("functionName" extends keyof ReturnType<typeof C.filterKeyFn> ? string : void), 3, "functionName" as ("functionName" extends keyof ReturnType<typeof C.expressionKeyFn> ? string : void)]), "?"], (($0 : undefined, name : Ast.EntityValue|Ast.NumberValue, $1 : undefined, base : Ast.InvocationExpression, param : C.FilterSlot, $2 : undefined) : ([Ast.EntityValue|Ast.NumberValue, C.ParamSlot[]])|null => {
        const idArg = param.schema.getArgument('id');
        if (!idArg || !idArg.type.equals(name.getType()))
            return null;
        const questions = D.booleanQuestion(base, param);
        if (!questions)
            return null;
        return [name, questions];
    }), undefined, {  });
   $grammar.addRule("boolean_user_search_question_with_name", ["is", new $runtime.NonTerminal("constant_name", undefined), "a", new $runtime.NonTerminal("base_table", ["idType" as ("idType" extends keyof ReturnType<typeof C.expressionKeyFn> ? string : void), 1, "type" as ("type" extends keyof ReturnType<typeof C.entityOrNumberValueKeyFn> ? string : void)]), new $runtime.Choice(["that", "which"]), new $runtime.NonTerminal("avp_filter", ["functionName" as ("functionName" extends keyof ReturnType<typeof C.filterKeyFn> ? string : void), 3, "functionName" as ("functionName" extends keyof ReturnType<typeof C.expressionKeyFn> ? string : void)]), "?"], (($0 : undefined, name : Ast.EntityValue|Ast.NumberValue, $1 : undefined, base : Ast.InvocationExpression, $2 : undefined, param : C.FilterSlot, $3 : undefined) : ([Ast.EntityValue|Ast.NumberValue, C.ParamSlot[]])|null => {
        const idArg = param.schema.getArgument('id');
        if (!idArg || !idArg.type.equals(name.getType()))
            return null;
        const questions = D.booleanQuestion(base, param);
        if (!questions)
            return null;
        return [name, questions];
    }), undefined, {  });
   $grammar.addRule("boolean_user_search_question_with_name", ["is", new $runtime.NonTerminal("constant_name", undefined), "a", new $runtime.NonTerminal("base_table", ["idType" as ("idType" extends keyof ReturnType<typeof C.expressionKeyFn> ? string : void), 1, "type" as ("type" extends keyof ReturnType<typeof C.entityOrNumberValueKeyFn> ? string : void)]), new $runtime.Choice(["that has", "with"]), new $runtime.NonTerminal("npp_filter", ["functionName" as ("functionName" extends keyof ReturnType<typeof C.filterKeyFn> ? string : void), 3, "functionName" as ("functionName" extends keyof ReturnType<typeof C.expressionKeyFn> ? string : void)]), "?"], (($0 : undefined, name : Ast.EntityValue|Ast.NumberValue, $1 : undefined, base : Ast.InvocationExpression, $2 : undefined, param : C.FilterSlot, $3 : undefined) : ([Ast.EntityValue|Ast.NumberValue, C.ParamSlot[]])|null => {
        const idArg = param.schema.getArgument('id');
        if (!idArg || !idArg.type.equals(name.getType()))
            return null;
        const questions = D.booleanQuestion(base, param);
        if (!questions)
            return null;
        return [name, questions];
    }), undefined, {  });
   $grammar.declareSymbol("user_search_question");
   $grammar.addRule("user_search_question", [new $runtime.Choice(["no ,", ""]), new $runtime.NonTerminal("simple_user_search_question", undefined)], (($0 : undefined, questions : C.ParamSlot[]) : (C.ParamSlot[])|null => questions), C.paramArrayKeyFn, {  });
   $grammar.addRule("user_search_question", [new $runtime.Choice(["no ,", ""]), new $runtime.NonTerminal("thingpedia_user_question", undefined), "?"], (($0 : undefined, questions : C.ParamSlot[], $1 : undefined) : (C.ParamSlot[])|null => questions), C.paramArrayKeyFn, { weight: (0.3) });
   $grammar.addRule("user_search_question", [new $runtime.NonTerminal("boolean_user_search_question", undefined)], ((questions : C.ParamSlot[]) : (C.ParamSlot[])|null => questions), C.paramArrayKeyFn, {  });
   $grammar.declareSymbol("user_search_question_for_list");
   $grammar.addRule("user_search_question_for_list", [new $runtime.NonTerminal("simple_user_search_question", undefined)], ((questions : C.ParamSlot[]) : (D.UserSearchQuestionForList)|null => [null, questions]), D.userSearchQuestionForListKeyFn, {  });
   $grammar.addRule("user_search_question_for_list", [new $runtime.NonTerminal("simple_user_search_question_for_list", undefined)], ((questions : C.ParamSlot[]) : (D.UserSearchQuestionForList)|null => [null, questions]), D.userSearchQuestionForListKeyFn, {  });
   $grammar.addRule("user_search_question_for_list", [new $runtime.NonTerminal("boolean_user_search_question_for_list", undefined)], ((questions : C.ParamSlot[]) : (D.UserSearchQuestionForList)|null => [null, questions]), D.userSearchQuestionForListKeyFn, {  });
   $grammar.addRule("user_search_question_for_list", [new $runtime.NonTerminal("user_search_question_with_name", undefined)], (($0 : [Ast.EntityValue|Ast.NumberValue, C.ParamSlot[]]) : (D.UserSearchQuestionForList)|null => { return $0; }), D.userSearchQuestionForListKeyFn, { identity: (true) });
   $grammar.addRule("user_search_question_for_list", [new $runtime.NonTerminal("boolean_user_search_question_with_name", undefined)], (($0 : [Ast.EntityValue|Ast.NumberValue, C.ParamSlot[]]) : (D.UserSearchQuestionForList)|null => { return $0; }), D.userSearchQuestionForListKeyFn, { identity: (true) });
};
