// -*- mode: js; indent-tabs-mode: nil; js-basic-offset: 4 -*-
//
// This file is part of Genie
//
// Copyright 2020 The Board of Trustees of the Leland Stanford Junior University
//
// Licensed under the Apache License, Version 2.0 (the "License");
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
//
//    http://www.apache.org/licenses/LICENSE-2.0
//
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// See the License for the specific language governing permissions and
// limitations under the License.
//
// Author: Giovanni Campagna <gcampagn@cs.stanford.edu>

// Shared templates and utilities, mostly collecting actual phrases or groups of phrases
// rather than complex logic

import { Ast, } from 'thingtalk';
import * as C from '../../ast_manip';
import ThingpediaLoader from '../../load-thingpedia';
import type * as $Genie from "genie-toolkit";
export default async function($runtime : typeof $Genie.SentenceGeneratorRuntime, $ttUtils : typeof $Genie.ThingTalkUtils, $options : $Genie.SentenceGeneratorTypes.GrammarOptions, $locale : $Genie.I18n.LanguagePack, $grammar : $Genie.SentenceGenerator<any, any>, $loader : ThingpediaLoader) : Promise<void> {
   $grammar.declareSymbol("counterproposal_preamble");
   $grammar.addRule("counterproposal_preamble", ["i was thinking more of"], (($0 : undefined) : (string)|null => "i was thinking more of"), undefined, { identity: (true) });
   $grammar.addRule("counterproposal_preamble", ["i would like"], (($0 : undefined) : (string)|null => "i would like"), undefined, { identity: (true) });
   $grammar.addRule("counterproposal_preamble", ["i am looking for"], (($0 : undefined) : (string)|null => "i am looking for"), undefined, { identity: (true) });
   $grammar.addRule("counterproposal_preamble", ["do you have"], (($0 : undefined) : (string)|null => "do you have"), undefined, { identity: (true) });
   $grammar.addRule("counterproposal_preamble", ["can you find"], (($0 : undefined) : (string)|null => "can you find"), undefined, { identity: (true) });
   $grammar.addRule("counterproposal_preamble", ["is there"], (($0 : undefined) : (string)|null => "is there"), undefined, { identity: (true) });
   $grammar.addRule("counterproposal_preamble", ["that does not sound great , do you have"], (($0 : undefined) : (string)|null => "that does not sound great , do you have"), undefined, { identity: (true) });
   $grammar.declareSymbol("the_base_noun_phrase");
   $grammar.addRule("the_base_noun_phrase", ["the", new $runtime.NonTerminal("base_noun_phrase", undefined)], (($0 : undefined, noun : Ast.FunctionDef) : (Ast.FunctionDef)|null => noun), C.functionDefKeyFn, {  });
   $grammar.addRule("the_base_noun_phrase", ["that", new $runtime.NonTerminal("base_noun_phrase", undefined)], (($0 : undefined, noun : Ast.FunctionDef) : (Ast.FunctionDef)|null => noun), C.functionDefKeyFn, {  });
   $grammar.declareSymbol("generic_excitement_phrase");
   $grammar.addRule("generic_excitement_phrase", [""], (($0 : undefined) : (string)|null => ""), undefined, { priority: (0.1), identity: (true) });
   $grammar.addRule("generic_excitement_phrase", ["alright ,"], (($0 : undefined) : (string)|null => "alright ,"), undefined, { identity: (true) });
   $grammar.addRule("generic_excitement_phrase", ["great !"], (($0 : undefined) : (string)|null => "great !"), undefined, { identity: (true) });
   $grammar.addRule("generic_excitement_phrase", ["awesome !"], (($0 : undefined) : (string)|null => "awesome !"), undefined, { identity: (true) });
   $grammar.addRule("generic_excitement_phrase", ["sounds good !"], (($0 : undefined) : (string)|null => "sounds good !"), undefined, { identity: (true) });
   $grammar.addRule("generic_excitement_phrase", ["sounds good ,"], (($0 : undefined) : (string)|null => "sounds good ,"), undefined, { identity: (true) });
   $grammar.addRule("generic_excitement_phrase", ["perfect !"], (($0 : undefined) : (string)|null => "perfect !"), undefined, { identity: (true) });
   $grammar.declareSymbol("greeting");
   $grammar.addRule("greeting", ["hello !"], (($0 : undefined) : (string)|null => "hello !"), undefined, { identity: (true) });
   $grammar.addRule("greeting", ["hi !"], (($0 : undefined) : (string)|null => "hi !"), undefined, { identity: (true) });
   $grammar.declareSymbol("generic_negative_proposal_reply");
   $grammar.addRule("generic_negative_proposal_reply", ["i do not like that"], (($0 : undefined) : (string)|null => "i do not like that"), undefined, { identity: (true) });
   $grammar.addRule("generic_negative_proposal_reply", ["i do not like that , do you have any other option ?"], (($0 : undefined) : (string)|null => "i do not like that , do you have any other option ?"), undefined, { identity: (true) });
   $grammar.declareSymbol("accept_phrase");
   $grammar.addRule("accept_phrase", ["i like that"], (($0 : undefined) : (string)|null => "i like that"), undefined, { identity: (true) });
   $grammar.addRule("accept_phrase", ["yes i like that"], (($0 : undefined) : (string)|null => "yes i like that"), undefined, { identity: (true) });
   $grammar.addRule("accept_phrase", ["sure i like that"], (($0 : undefined) : (string)|null => "sure i like that"), undefined, { identity: (true) });
   $grammar.addRule("accept_phrase", ["sounds good"], (($0 : undefined) : (string)|null => "sounds good"), undefined, { identity: (true) });
   $grammar.declareSymbol("reject_phrase");
   $grammar.addRule("reject_phrase", ["no"], (($0 : undefined) : (string)|null => "no"), undefined, { identity: (true) });
   $grammar.addRule("reject_phrase", ["actually , no"], (($0 : undefined) : (string)|null => "actually , no"), undefined, { identity: (true) });
   $grammar.addRule("reject_phrase", ["that will not work"], (($0 : undefined) : (string)|null => "that will not work"), undefined, { identity: (true) });
   $grammar.addRule("reject_phrase", ["no , thank you"], (($0 : undefined) : (string)|null => "no , thank you"), undefined, { identity: (true) });
   $grammar.addRule("reject_phrase", ["not yet"], (($0 : undefined) : (string)|null => "not yet"), undefined, { identity: (true) });
   $grammar.addRule("reject_phrase", ["actually , not yet"], (($0 : undefined) : (string)|null => "actually , not yet"), undefined, { identity: (true) });
   $grammar.declareSymbol("tell_me_more_phrase");
   $grammar.addRule("tell_me_more_phrase", ["can you tell me more ?"], (($0 : undefined) : (string)|null => "can you tell me more ?"), undefined, { identity: (true) });
   $grammar.addRule("tell_me_more_phrase", ["can you tell me more about it ?"], (($0 : undefined) : (string)|null => "can you tell me more about it ?"), undefined, { identity: (true) });
   $grammar.addRule("tell_me_more_phrase", ["can you give me more info ?"], (($0 : undefined) : (string)|null => "can you give me more info ?"), undefined, { identity: (true) });
   $grammar.addRule("tell_me_more_phrase", ["give me more info !"], (($0 : undefined) : (string)|null => "give me more info !"), undefined, { identity: (true) });
   $grammar.addRule("tell_me_more_phrase", ["give me more information ."], (($0 : undefined) : (string)|null => "give me more information ."), undefined, { identity: (true) });
   $grammar.addRule("tell_me_more_phrase", ["tell me more ."], (($0 : undefined) : (string)|null => "tell me more ."), undefined, { identity: (true) });
   $grammar.declareSymbol("learn_more_offer");
   $grammar.addRule("learn_more_offer", ["would you like to learn more ?"], (($0 : undefined) : (string)|null => "would you like to learn more ?"), undefined, { priority: (0.1), identity: (true) });
   $grammar.addRule("learn_more_offer", ["would you like to know more about it ?"], (($0 : undefined) : (string)|null => "would you like to know more about it ?"), undefined, { priority: (0.1), identity: (true) });
   $grammar.addRule("learn_more_offer", ["would you like to hear more about it ?"], (($0 : undefined) : (string)|null => "would you like to hear more about it ?"), undefined, { priority: (0.1), identity: (true) });
   $grammar.addRule("learn_more_offer", ["are you interested to hear more about it ?"], (($0 : undefined) : (string)|null => "are you interested to hear more about it ?"), undefined, { priority: (0.1), identity: (true) });
   $grammar.declareSymbol("yes_please_phrase");
   $grammar.addRule("yes_please_phrase", ["yes"], (($0 : undefined) : (string)|null => "yes"), undefined, { identity: (true) });
   $grammar.addRule("yes_please_phrase", ["yes , please"], (($0 : undefined) : (string)|null => "yes , please"), undefined, { identity: (true) });
   $grammar.addRule("yes_please_phrase", ["yeah"], (($0 : undefined) : (string)|null => "yeah"), undefined, { identity: (true) });
   $grammar.declareSymbol("ok_thanks_phrase");
   $grammar.addRule("ok_thanks_phrase", ["ok thank you that is all i need ."], (($0 : undefined) : (string)|null => "ok thank you that is all i need ."), undefined, { identity: (true) });
   $grammar.addRule("ok_thanks_phrase", ["ok thanks that is all i need ."], (($0 : undefined) : (string)|null => "ok thanks that is all i need ."), undefined, { identity: (true) });
   $grammar.declareSymbol("thanks_phrase");
   $grammar.addRule("thanks_phrase", ["thank you , this is good for now ."], (($0 : undefined) : (string)|null => "thank you , this is good for now ."), undefined, { identity: (true) });
   $grammar.addRule("thanks_phrase", ["thank you , that will be all ."], (($0 : undefined) : (string)|null => "thank you , that will be all ."), undefined, { identity: (true) });
   $grammar.addRule("thanks_phrase", ["thank you , that will be all for now ."], (($0 : undefined) : (string)|null => "thank you , that will be all for now ."), undefined, { identity: (true) });
   $grammar.addRule("thanks_phrase", ["thank you this is what i need ."], (($0 : undefined) : (string)|null => "thank you this is what i need ."), undefined, { identity: (true) });
   $grammar.addRule("thanks_phrase", ["thanks , this is good for now ."], (($0 : undefined) : (string)|null => "thanks , this is good for now ."), undefined, { identity: (true) });
   $grammar.addRule("thanks_phrase", ["thanks this is what i need ."], (($0 : undefined) : (string)|null => "thanks this is what i need ."), undefined, { identity: (true) });
   $grammar.addRule("thanks_phrase", ["thank you for your help !"], (($0 : undefined) : (string)|null => "thank you for your help !"), undefined, { identity: (true) });
   $grammar.addRule("thanks_phrase", ["thank you for your help , this is what i need ."], (($0 : undefined) : (string)|null => "thank you for your help , this is what i need ."), undefined, { identity: (true) });
   $grammar.declareSymbol("no_thanks_phrase");
   $grammar.addRule("no_thanks_phrase", ["no thanks ."], (($0 : undefined) : (string)|null => "no thanks ."), undefined, { identity: (true) });
   $grammar.addRule("no_thanks_phrase", ["no , thanks ."], (($0 : undefined) : (string)|null => "no , thanks ."), undefined, { identity: (true) });
   $grammar.addRule("no_thanks_phrase", ["no i am good thanks ."], (($0 : undefined) : (string)|null => "no i am good thanks ."), undefined, { identity: (true) });
   $grammar.addRule("no_thanks_phrase", ["no need thanks ."], (($0 : undefined) : (string)|null => "no need thanks ."), undefined, { identity: (true) });
   $grammar.addRule("no_thanks_phrase", ["no thank you , this is good for now ."], (($0 : undefined) : (string)|null => "no thank you , this is good for now ."), undefined, { identity: (true) });
   $grammar.addRule("no_thanks_phrase", ["no thank you this is what i need ."], (($0 : undefined) : (string)|null => "no thank you this is what i need ."), undefined, { identity: (true) });
   $grammar.addRule("no_thanks_phrase", ["no thanks , this is good for now ."], (($0 : undefined) : (string)|null => "no thanks , this is good for now ."), undefined, { identity: (true) });
   $grammar.addRule("no_thanks_phrase", ["no thanks this is what i need ."], (($0 : undefined) : (string)|null => "no thanks this is what i need ."), undefined, { identity: (true) });
   $grammar.addRule("no_thanks_phrase", ["no thanks that will be it ."], (($0 : undefined) : (string)|null => "no thanks that will be it ."), undefined, { identity: (true) });
   $grammar.addRule("no_thanks_phrase", ["no thanks this will do ."], (($0 : undefined) : (string)|null => "no thanks this will do ."), undefined, { identity: (true) });
   $grammar.addRule("no_thanks_phrase", ["no thank you that will be it for now ."], (($0 : undefined) : (string)|null => "no thank you that will be it for now ."), undefined, { identity: (true) });
   $grammar.addRule("no_thanks_phrase", ["no thanks this will do for now ."], (($0 : undefined) : (string)|null => "no thanks this will do for now ."), undefined, { identity: (true) });
   $grammar.addRule("no_thanks_phrase", ["no thank you that will be all ."], (($0 : undefined) : (string)|null => "no thank you that will be all ."), undefined, { identity: (true) });
   $grammar.declareSymbol("anything_else_phrase");
   $grammar.addRule("anything_else_phrase", ["is there anything else i can help you with ?"], (($0 : undefined) : (string)|null => "is there anything else i can help you with ?"), undefined, { identity: (true) });
   $grammar.addRule("anything_else_phrase", ["can i help you with anything else ?"], (($0 : undefined) : (string)|null => "can i help you with anything else ?"), undefined, { identity: (true) });
   $grammar.addRule("anything_else_phrase", ["do you need anything else ?"], (($0 : undefined) : (string)|null => "do you need anything else ?"), undefined, { identity: (true) });
   $grammar.addRule("anything_else_phrase", ["anything else for you today ?"], (($0 : undefined) : (string)|null => "anything else for you today ?"), undefined, { identity: (true) });
   $grammar.declareSymbol("sorry_preamble");
   $grammar.addRule("sorry_preamble", ["sorry ,"], (($0 : undefined) : (string)|null => "sorry ,"), undefined, { identity: (true) });
   $grammar.addRule("sorry_preamble", ["i am sorry ,"], (($0 : undefined) : (string)|null => "i am sorry ,"), undefined, { identity: (true) });
   $grammar.addRule("sorry_preamble", [""], (($0 : undefined) : (string)|null => ""), undefined, { identity: (true) });
   $grammar.declareSymbol("nevermind_phrase");
   $grammar.addRule("nevermind_phrase", ["ok , never mind"], (($0 : undefined) : (string)|null => "ok , never mind"), undefined, { identity: (true) });
   $grammar.addRule("nevermind_phrase", ["never mind then"], (($0 : undefined) : (string)|null => "never mind then"), undefined, { identity: (true) });
   $grammar.addRule("nevermind_phrase", ["alright , never mind"], (($0 : undefined) : (string)|null => "alright , never mind"), undefined, { identity: (true) });
   $grammar.addRule("nevermind_phrase", ["too bad , alright"], (($0 : undefined) : (string)|null => "too bad , alright"), undefined, { identity: (true) });
   $grammar.declareSymbol("ask_recommend_phrase");
   $grammar.addRule("ask_recommend_phrase", ["which one would you recommend ?"], (($0 : undefined) : (string)|null => "which one would you recommend ?"), undefined, { identity: (true) });
   $grammar.addRule("ask_recommend_phrase", ["which 1 would you recommend ?"], (($0 : undefined) : (string)|null => "which 1 would you recommend ?"), undefined, { identity: (true) });
   $grammar.addRule("ask_recommend_phrase", ["what is your favorite ?"], (($0 : undefined) : (string)|null => "what is your favorite ?"), undefined, { identity: (true) });
   $grammar.addRule("ask_recommend_phrase", ["surprise me ! just pick one please ."], (($0 : undefined) : (string)|null => "surprise me ! just pick one please ."), undefined, { identity: (true) });
   $grammar.declareSymbol("generic_dontcare_phrase");
   $grammar.addRule("generic_dontcare_phrase", ["it does not matter ."], (($0 : undefined) : (string)|null => "it does not matter ."), undefined, { identity: (true) });
   $grammar.addRule("generic_dontcare_phrase", ["it does not matter . can you suggest one ?"], (($0 : undefined) : (string)|null => "it does not matter . can you suggest one ?"), undefined, { identity: (true) });
   $grammar.addRule("generic_dontcare_phrase", ["i", new $runtime.Choice(["do not have a", "have no"]), "preference ."], (($0 : undefined, $1 : undefined, $2 : undefined) : (string)|null => ''), undefined, {  });
   $grammar.addRule("generic_dontcare_phrase", ["i", new $runtime.Choice(["do not have a", "have no"]), "preference . what do you recommend ?"], (($0 : undefined, $1 : undefined, $2 : undefined) : (string)|null => ''), undefined, {  });
   $grammar.addRule("generic_dontcare_phrase", ["i have no preference . please make a recommendation ."], (($0 : undefined) : (string)|null => ''), undefined, {  });
   $grammar.addRule("generic_dontcare_phrase", ["i have no preference . which", new $runtime.Choice(["one", "1"]), "is best ?"], (($0 : undefined, $1 : undefined, $2 : undefined) : (string)|null => ''), undefined, {  });
   $grammar.declareSymbol("no_dontcare_phrase");
   $grammar.addRule("no_dontcare_phrase", ["no", new $runtime.Choice([".", ","]), "can you suggest one ?"], (($0 : undefined, $1 : undefined, $2 : undefined) : (string)|null => ''), undefined, {  });
   $grammar.addRule("no_dontcare_phrase", ["no", new $runtime.Choice([".", ","]), "please make a recommendation ."], (($0 : undefined, $1 : undefined, $2 : undefined) : (string)|null => ''), undefined, {  });
   $grammar.addRule("no_dontcare_phrase", ["no", new $runtime.Choice([".", ","]), "which", new $runtime.Choice(["one", "1"]), "is best ?"], (($0 : undefined, $1 : undefined, $2 : undefined, $3 : undefined, $4 : undefined) : (string)|null => ''), undefined, {  });
   $grammar.addRule("no_dontcare_phrase", ["no", new $runtime.Choice([".", ","]), "you choose for me !"], (($0 : undefined, $1 : undefined, $2 : undefined) : (string)|null => ''), undefined, {  });
   $grammar.addRule("no_dontcare_phrase", ["no", new $runtime.Choice([".", ","]), "surprise me !"], (($0 : undefined, $1 : undefined, $2 : undefined) : (string)|null => ''), undefined, {  });
   $grammar.addRule("no_dontcare_phrase", ["i have no preference . please make a recommendation ."], (($0 : undefined) : (string)|null => ''), undefined, {  });
   $grammar.declareSymbol("prefer_question_phrase");
   $grammar.addRule("prefer_question_phrase", ["do you want"], (($0 : undefined) : (string)|null => "do you want"), undefined, { identity: (true) });
   $grammar.addRule("prefer_question_phrase", ["would you prefer"], (($0 : undefined) : (string)|null => "would you prefer"), undefined, { identity: (true) });
   $grammar.addRule("prefer_question_phrase", ["would you like"], (($0 : undefined) : (string)|null => "would you like"), undefined, { identity: (true) });
   $grammar.addRule("prefer_question_phrase", ["are you interested in"], (($0 : undefined) : (string)|null => "are you interested in"), undefined, { identity: (true) });
   $grammar.addRule("prefer_question_phrase", ["are you looking for"], (($0 : undefined) : (string)|null => "are you looking for"), undefined, { identity: (true) });
   $grammar.declareSymbol("system_learn_more");
   $grammar.addRule("system_learn_more", ["what would you like to know ?"], (($0 : undefined) : (string)|null => "what would you like to know ?"), undefined, { identity: (true) });
   $grammar.addRule("system_learn_more", ["what would you like to hear ?"], (($0 : undefined) : (string)|null => "what would you like to hear ?"), undefined, { identity: (true) });
   $grammar.declareSymbol("repeat_command_phrase");
   $grammar.addRule("repeat_command_phrase", ["again !"], (($0 : undefined) : (string)|null => "again !"), undefined, { identity: (true) });
   $grammar.addRule("repeat_command_phrase", ["another one"], (($0 : undefined) : (string)|null => "another one"), undefined, { identity: (true) });
   $grammar.addRule("repeat_command_phrase", ["do this again"], (($0 : undefined) : (string)|null => "do this again"), undefined, { identity: (true) });
   $grammar.addRule("repeat_command_phrase", ["do it again"], (($0 : undefined) : (string)|null => "do it again"), undefined, { identity: (true) });
};
