// -*- mode: js; indent-tabs-mode: nil; js-basic-offset: 4 -*-
//
// This file is part of Genie
//
// Copyright 2019-2020 The Board of Trustees of the Leland Stanford Junior University
//
// Licensed under the Apache License, Version 2.0 (the "License");
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
//
//    http://www.apache.org/licenses/LICENSE-2.0
//
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// See the License for the specific language governing permissions and
// limitations under the License.
//
// Author: Silei Xu <silei@cs.stanford.edu>
//         Giovanni Campagna <gcampagn@cs.stanford.edu>

import assert from 'assert';
import { Ast, Type } from 'thingtalk';
import * as C from '../ast_manip';
import ThingpediaLoader from '../load-thingpedia';
import type * as $Genie from "genie-toolkit";
export default async function($runtime : typeof $Genie.SentenceGeneratorRuntime, $ttUtils : typeof $Genie.ThingTalkUtils, $options : $Genie.SentenceGeneratorTypes.GrammarOptions, $locale : $Genie.I18n.LanguagePack, $grammar : $Genie.SentenceGenerator<any, any>, $loader : ThingpediaLoader) : Promise<void> {

const HERE = new Ast.Value.Location(new Ast.Location.Relative('current_location'));
   $grammar.declareSymbol("get_predicate_filter");
   $grammar.addRule("get_predicate_filter", ["before", new $runtime.NonTerminal("constant_Time", undefined)], (($0 : undefined, t1 : Ast.Value) : (C.DomainIndependentFilterSlot)|null => C.timeGetPredicate($loader, null, t1)), undefined, {  });
   $grammar.addRule("get_predicate_filter", ["after", new $runtime.NonTerminal("constant_Time", undefined)], (($0 : undefined, t2 : Ast.Value) : (C.DomainIndependentFilterSlot)|null => C.timeGetPredicate($loader, t2, null)), undefined, {  });
   $grammar.addRule("get_predicate_filter", ["between", new $runtime.NonTerminal("constant_Time", undefined), "and", new $runtime.NonTerminal("constant_Time", undefined)], (($0 : undefined, t1 : Ast.Value, $1 : undefined, t2 : Ast.Value) : (C.DomainIndependentFilterSlot)|null => C.timeGetPredicate($loader, t1, t2)), undefined, {  });
   $grammar.addRule("get_predicate_filter", ["my location is", new $runtime.NonTerminal("constant_Location", undefined)], (($0 : undefined, loc : Ast.LocationValue) : (C.DomainIndependentFilterSlot)|null => C.locationGetPredicate($loader, loc)), undefined, {  });
   $grammar.addRule("get_predicate_filter", ["my location is not", new $runtime.NonTerminal("constant_Location", undefined)], (($0 : undefined, loc : Ast.LocationValue) : (C.DomainIndependentFilterSlot)|null => C.locationGetPredicate($loader, loc, true)), undefined, {  });
   $grammar.addRule("get_predicate_filter", ["i am at", new $runtime.NonTerminal("constant_Location", undefined)], (($0 : undefined, loc : Ast.LocationValue) : (C.DomainIndependentFilterSlot)|null => C.locationGetPredicate($loader, loc)), undefined, {  });
   $grammar.addRule("get_predicate_filter", ["i am not at", new $runtime.NonTerminal("constant_Location", undefined)], (($0 : undefined, loc : Ast.LocationValue) : (C.DomainIndependentFilterSlot)|null => C.locationGetPredicate($loader, loc, true)), undefined, {  });
   $grammar.addRule("get_predicate_filter", [new $runtime.NonTerminal("projection_Any", undefined), new $runtime.Choice(["is", "is exactly", "is equal to"]), new $runtime.NonTerminal("constant_Any", undefined)], ((proj : Ast.Expression, $0 : undefined, x : Ast.Value) : (C.DomainIndependentFilterSlot)|null => C.makeGetPredicate(proj, '==', x)), undefined, {  });
   $grammar.addRule("get_predicate_filter", [new $runtime.NonTerminal("projection_Any", undefined), new $runtime.Choice(["is not", "is n't", "is different than"]), new $runtime.NonTerminal("constant_Any", undefined)], ((proj : Ast.Expression, $0 : undefined, x : Ast.Value) : (C.DomainIndependentFilterSlot)|null => C.makeGetPredicate(proj, '==', x, true)), undefined, {  });
   $grammar.declareSymbol("if_filter");
   if ($options.flags.get_predicate) {
       $grammar.addRule("if_filter", [new $runtime.NonTerminal("get_predicate_filter", undefined)], (($0 : C.DomainIndependentFilterSlot) : (C.FilterSlot|C.DomainIndependentFilterSlot)|null => { return $0; }), C.filterKeyFn, { identity: (true) });
   }
   $grammar.addRule("if_filter", ["the", new $runtime.NonTerminal("out_param_Any", undefined), new $runtime.Choice(["is", "is exactly", "is equal to"]), new $runtime.NonTerminal("constant_Any", undefined)], (($0 : undefined, p : C.ParamSlot, $1 : undefined, x : Ast.Value) : (C.FilterSlot|C.DomainIndependentFilterSlot)|null => C.makeFilter($loader, p, '==', x)), C.filterKeyFn, {  });
   $grammar.addRule("if_filter", ["the", new $runtime.NonTerminal("out_param_Any", undefined), new $runtime.Choice(["is", "is exactly", "is equal to"]), new $runtime.NonTerminal("constant_Any", undefined)], (($0 : undefined, p : C.ParamSlot, $1 : undefined, x : Ast.Value) : (C.FilterSlot|C.DomainIndependentFilterSlot)|null => C.makeFilter($loader, p, '==', x)), C.filterKeyFn, {  });
   $grammar.addRule("if_filter", ["the", new $runtime.NonTerminal("out_param_Any", undefined), new $runtime.Choice(["is not", "is n't", "is different than"]), new $runtime.NonTerminal("constant_Any", ["type" as ("type" extends keyof ReturnType<typeof C.valueKeyFn> ? string : void), 1, "type" as ("type" extends keyof ReturnType<typeof C.paramKeyFn> ? string : void)])], (($0 : undefined, p : C.ParamSlot, $1 : undefined, x : Ast.Value) : (C.FilterSlot|C.DomainIndependentFilterSlot)|null => C.makeFilter($loader, p, '==', x, true)), C.filterKeyFn, {  });
   $grammar.addRule("if_filter", [new $runtime.NonTerminal("out_param_Any", ["type" as ("type" extends keyof ReturnType<typeof C.paramKeyFn> ? string : void), ((Type.Number)) as ReturnType<typeof C.paramKeyFn>["type"]]), "is", new $runtime.Choice(["greater than", "higher than", "larger than", "more than", "at least", "not less than"]), new $runtime.NonTerminal("constant_Number", undefined)], ((p : C.ParamSlot, $0 : undefined, $1 : undefined, x : Ast.NumberValue) : (C.FilterSlot|C.DomainIndependentFilterSlot)|null => C.makeFilter($loader, p, '>=', x)), C.filterKeyFn, {  });
   $grammar.addRule("if_filter", [new $runtime.NonTerminal("out_param_Any", ["type" as ("type" extends keyof ReturnType<typeof C.paramKeyFn> ? string : void), ((Type.Number)) as ReturnType<typeof C.paramKeyFn>["type"]]), "is", new $runtime.Choice(["smaller than", "lower than", "less than", "at most", "not more than"]), new $runtime.NonTerminal("constant_Number", undefined)], ((p : C.ParamSlot, $0 : undefined, $1 : undefined, x : Ast.NumberValue) : (C.FilterSlot|C.DomainIndependentFilterSlot)|null => C.makeFilter($loader, p, '<=', x)), C.filterKeyFn, {  });
   $grammar.addRule("if_filter", [new $runtime.NonTerminal("out_param_Any", ["type" as ("type" extends keyof ReturnType<typeof C.paramKeyFn> ? string : void), ((Type.Currency)) as ReturnType<typeof C.paramKeyFn>["type"]]), "is", new $runtime.Choice(["more expensive", "more costly", "more"]), "than", new $runtime.NonTerminal("constant_Currency", undefined)], ((p : C.ParamSlot, $0 : undefined, $1 : undefined, $2 : undefined, x : Ast.Value) : (C.FilterSlot|C.DomainIndependentFilterSlot)|null => C.makeFilter($loader, p, '>=', x)), C.filterKeyFn, {  });
   $grammar.addRule("if_filter", [new $runtime.NonTerminal("out_param_Any", ["type" as ("type" extends keyof ReturnType<typeof C.paramKeyFn> ? string : void), ((Type.Currency)) as ReturnType<typeof C.paramKeyFn>["type"]]), "is", new $runtime.Choice(["less expensive", "cheaper", "less"]), "than", new $runtime.NonTerminal("constant_Currency", undefined)], ((p : C.ParamSlot, $0 : undefined, $1 : undefined, $2 : undefined, x : Ast.Value) : (C.FilterSlot|C.DomainIndependentFilterSlot)|null => C.makeFilter($loader, p, '<=', x)), C.filterKeyFn, {  });
   $grammar.addRule("if_filter", [new $runtime.NonTerminal("out_param_Any", ["type" as ("type" extends keyof ReturnType<typeof C.paramKeyFn> ? string : void), ((new Type.Measure('ms'))) as ReturnType<typeof C.paramKeyFn>["type"]]), "is", new $runtime.Choice(["longer than", "no shorter than"]), new $runtime.NonTerminal("constant_Measure_ms", undefined)], ((p : C.ParamSlot, $0 : undefined, $1 : undefined, x : Ast.Value) : (C.FilterSlot|C.DomainIndependentFilterSlot)|null => C.makeFilter($loader, p, '>=', x)), C.filterKeyFn, {  });
   $grammar.addRule("if_filter", [new $runtime.NonTerminal("out_param_Any", ["type" as ("type" extends keyof ReturnType<typeof C.paramKeyFn> ? string : void), ((new Type.Measure('ms'))) as ReturnType<typeof C.paramKeyFn>["type"]]), "is", new $runtime.Choice(["no longer than", "shorter than"]), new $runtime.NonTerminal("constant_Measure_ms", undefined)], ((p : C.ParamSlot, $0 : undefined, $1 : undefined, x : Ast.Value) : (C.FilterSlot|C.DomainIndependentFilterSlot)|null => C.makeFilter($loader, p, '<=', x)), C.filterKeyFn, {  });
   $grammar.addRule("if_filter", [new $runtime.NonTerminal("out_param_Any", ["type" as ("type" extends keyof ReturnType<typeof C.paramKeyFn> ? string : void), ((new Type.Measure('byte'))) as ReturnType<typeof C.paramKeyFn>["type"]]), "is", new $runtime.Choice(["larger", "more", "no smaller", "no less"]), "than", new $runtime.NonTerminal("constant_Measure_byte", undefined)], ((p : C.ParamSlot, $0 : undefined, $1 : undefined, $2 : undefined, x : Ast.Value) : (C.FilterSlot|C.DomainIndependentFilterSlot)|null => C.makeFilter($loader, p, '>=', x)), C.filterKeyFn, {  });
   $grammar.addRule("if_filter", [new $runtime.NonTerminal("out_param_Any", ["type" as ("type" extends keyof ReturnType<typeof C.paramKeyFn> ? string : void), ((new Type.Measure('byte'))) as ReturnType<typeof C.paramKeyFn>["type"]]), "is", new $runtime.Choice(["smaller", "less", "no larger", "no more"]), "than", new $runtime.NonTerminal("constant_Measure_byte", undefined)], ((p : C.ParamSlot, $0 : undefined, $1 : undefined, $2 : undefined, x : Ast.Value) : (C.FilterSlot|C.DomainIndependentFilterSlot)|null => C.makeFilter($loader, p, '<=', x)), C.filterKeyFn, {  });
   $grammar.addRule("if_filter", [new $runtime.NonTerminal("out_param_Any", ["type" as ("type" extends keyof ReturnType<typeof C.paramKeyFn> ? string : void), ((new Type.Measure('kg'))) as ReturnType<typeof C.paramKeyFn>["type"]]), "is", new $runtime.Choice(["heavier", "larger"]), "than", new $runtime.NonTerminal("constant_Measure_kg", undefined)], ((p : C.ParamSlot, $0 : undefined, $1 : undefined, $2 : undefined, x : Ast.Value) : (C.FilterSlot|C.DomainIndependentFilterSlot)|null => C.makeFilter($loader, p, '>=', x)), C.filterKeyFn, {  });
   $grammar.addRule("if_filter", [new $runtime.NonTerminal("out_param_Any", ["type" as ("type" extends keyof ReturnType<typeof C.paramKeyFn> ? string : void), ((new Type.Measure('kg'))) as ReturnType<typeof C.paramKeyFn>["type"]]), "is", new $runtime.Choice(["lighter", "smaller"]), "than", new $runtime.NonTerminal("constant_Measure_kg", undefined)], ((p : C.ParamSlot, $0 : undefined, $1 : undefined, $2 : undefined, x : Ast.Value) : (C.FilterSlot|C.DomainIndependentFilterSlot)|null => C.makeFilter($loader, p, '<=', x)), C.filterKeyFn, {  });
   $grammar.addRule("if_filter", [new $runtime.NonTerminal("out_param_Any", ["type" as ("type" extends keyof ReturnType<typeof C.paramKeyFn> ? string : void), ((new Type.Measure('C'))) as ReturnType<typeof C.paramKeyFn>["type"]]), "is", new $runtime.Choice(["hotter", "higher"]), "than", new $runtime.NonTerminal("constant_Measure_C", undefined)], ((p : C.ParamSlot, $0 : undefined, $1 : undefined, $2 : undefined, x : Ast.Value) : (C.FilterSlot|C.DomainIndependentFilterSlot)|null => C.makeFilter($loader, p, '>=', x)), C.filterKeyFn, {  });
   $grammar.addRule("if_filter", [new $runtime.NonTerminal("out_param_Any", ["type" as ("type" extends keyof ReturnType<typeof C.paramKeyFn> ? string : void), ((new Type.Measure('C'))) as ReturnType<typeof C.paramKeyFn>["type"]]), "is", new $runtime.Choice(["cooler", "colder", "lower"]), "than", new $runtime.NonTerminal("constant_Measure_C", undefined)], ((p : C.ParamSlot, $0 : undefined, $1 : undefined, $2 : undefined, x : Ast.Value) : (C.FilterSlot|C.DomainIndependentFilterSlot)|null => C.makeFilter($loader, p, '<=', x)), C.filterKeyFn, {  });
   $grammar.addRule("if_filter", [new $runtime.NonTerminal("out_param_Any", ["type" as ("type" extends keyof ReturnType<typeof C.paramKeyFn> ? string : void), ((new Type.Measure('m'))) as ReturnType<typeof C.paramKeyFn>["type"]]), "is", new $runtime.Choice(["farther", "more distant", "longer"]), "than", new $runtime.NonTerminal("constant_Measure_m", undefined)], ((p : C.ParamSlot, $0 : undefined, $1 : undefined, $2 : undefined, x : Ast.Value) : (C.FilterSlot|C.DomainIndependentFilterSlot)|null => C.makeFilter($loader, p, '>=', x)), C.filterKeyFn, {  });
   $grammar.addRule("if_filter", [new $runtime.NonTerminal("out_param_Any", ["type" as ("type" extends keyof ReturnType<typeof C.paramKeyFn> ? string : void), ((new Type.Measure('m'))) as ReturnType<typeof C.paramKeyFn>["type"]]), "is", new $runtime.Choice(["nearer", "closer", "shorter", "no longer", "no farther"]), "than", new $runtime.NonTerminal("constant_Measure_m", undefined)], ((p : C.ParamSlot, $0 : undefined, $1 : undefined, $2 : undefined, x : Ast.Value) : (C.FilterSlot|C.DomainIndependentFilterSlot)|null => C.makeFilter($loader, p, '<=', x)), C.filterKeyFn, {  });
   $grammar.addRule("if_filter", [new $runtime.NonTerminal("out_param_Any", ["type" as ("type" extends keyof ReturnType<typeof C.paramKeyFn> ? string : void), ((new Type.Measure('m2'))) as ReturnType<typeof C.paramKeyFn>["type"]]), "is", new $runtime.Choice(["larger", "bigger", "no smaller"]), "than", new $runtime.NonTerminal("constant_Measure_m2", undefined)], ((p : C.ParamSlot, $0 : undefined, $1 : undefined, $2 : undefined, x : Ast.Value) : (C.FilterSlot|C.DomainIndependentFilterSlot)|null => C.makeFilter($loader, p, '>=', x)), C.filterKeyFn, {  });
   $grammar.addRule("if_filter", [new $runtime.NonTerminal("out_param_Any", ["type" as ("type" extends keyof ReturnType<typeof C.paramKeyFn> ? string : void), ((new Type.Measure('m2'))) as ReturnType<typeof C.paramKeyFn>["type"]]), "is", new $runtime.Choice(["smaller", "no larger", "no bigger"]), "than", new $runtime.NonTerminal("constant_Measure_m2", undefined)], ((p : C.ParamSlot, $0 : undefined, $1 : undefined, $2 : undefined, x : Ast.Value) : (C.FilterSlot|C.DomainIndependentFilterSlot)|null => C.makeFilter($loader, p, '<=', x)), C.filterKeyFn, {  });
   $grammar.addRule("if_filter", [new $runtime.NonTerminal("out_param_Any", ["type" as ("type" extends keyof ReturnType<typeof C.paramKeyFn> ? string : void), ((new Type.Measure('m3'))) as ReturnType<typeof C.paramKeyFn>["type"]]), "is", new $runtime.Choice(["larger", "bigger", "no smaller"]), "than", new $runtime.NonTerminal("constant_Measure_m3", undefined)], ((p : C.ParamSlot, $0 : undefined, $1 : undefined, $2 : undefined, x : Ast.Value) : (C.FilterSlot|C.DomainIndependentFilterSlot)|null => C.makeFilter($loader, p, '>=', x)), C.filterKeyFn, {  });
   $grammar.addRule("if_filter", [new $runtime.NonTerminal("out_param_Any", ["type" as ("type" extends keyof ReturnType<typeof C.paramKeyFn> ? string : void), ((new Type.Measure('m3'))) as ReturnType<typeof C.paramKeyFn>["type"]]), "is", new $runtime.Choice(["smaller", "no larger", "no bigger"]), "than", new $runtime.NonTerminal("constant_Measure_m3", undefined)], ((p : C.ParamSlot, $0 : undefined, $1 : undefined, $2 : undefined, x : Ast.Value) : (C.FilterSlot|C.DomainIndependentFilterSlot)|null => C.makeFilter($loader, p, '<=', x)), C.filterKeyFn, {  });
   $grammar.addRule("if_filter", [new $runtime.NonTerminal("out_param_Any", ["type" as ("type" extends keyof ReturnType<typeof C.paramKeyFn> ? string : void), ((new Type.Measure('mps'))) as ReturnType<typeof C.paramKeyFn>["type"]]), "is", new $runtime.Choice(["faster", "quicker", "speedier"]), "than", new $runtime.NonTerminal("constant_Measure_mps", undefined)], ((p : C.ParamSlot, $0 : undefined, $1 : undefined, $2 : undefined, x : Ast.Value) : (C.FilterSlot|C.DomainIndependentFilterSlot)|null => C.makeFilter($loader, p, '>=', x)), C.filterKeyFn, {  });
   $grammar.addRule("if_filter", [new $runtime.NonTerminal("out_param_Any", ["type" as ("type" extends keyof ReturnType<typeof C.paramKeyFn> ? string : void), ((new Type.Measure('mps'))) as ReturnType<typeof C.paramKeyFn>["type"]]), "is", new $runtime.Choice(["slower", "more slowly"]), "than", new $runtime.NonTerminal("constant_Measure_mps", undefined)], ((p : C.ParamSlot, $0 : undefined, $1 : undefined, $2 : undefined, x : Ast.Value) : (C.FilterSlot|C.DomainIndependentFilterSlot)|null => C.makeFilter($loader, p, '<=', x)), C.filterKeyFn, {  });
   if (!$options.flags.turking) {
       $grammar.addRule("if_filter", [new $runtime.NonTerminal("out_param_Any", ["type" as ("type" extends keyof ReturnType<typeof C.paramKeyFn> ? string : void), ((Type.Date)) as ReturnType<typeof C.paramKeyFn>["type"]]), "is", new $runtime.Choice(["after", "later than"]), new $runtime.NonTerminal("constant_Date", undefined)], ((p : C.ParamSlot, $0 : undefined, $1 : undefined, x : Ast.Value) : (C.FilterSlot|C.DomainIndependentFilterSlot)|null => C.makeFilter($loader, p, '>=', x)), C.filterKeyFn, {  });
       $grammar.addRule("if_filter", [new $runtime.NonTerminal("out_param_Any", ["type" as ("type" extends keyof ReturnType<typeof C.paramKeyFn> ? string : void), ((Type.Date)) as ReturnType<typeof C.paramKeyFn>["type"]]), "is", new $runtime.Choice(["before", "earlier than", "sooner than"]), new $runtime.NonTerminal("constant_Date", undefined)], ((p : C.ParamSlot, $0 : undefined, $1 : undefined, x : Ast.Value) : (C.FilterSlot|C.DomainIndependentFilterSlot)|null => C.makeFilter($loader, p, '<=', x)), C.filterKeyFn, {  });
       $grammar.addRule("if_filter", [new $runtime.NonTerminal("out_param_Any", ["type" as ("type" extends keyof ReturnType<typeof C.paramKeyFn> ? string : void), ((Type.Time)) as ReturnType<typeof C.paramKeyFn>["type"]]), "is", new $runtime.Choice(["after", "later than"]), new $runtime.NonTerminal("constant_Time", undefined)], ((p : C.ParamSlot, $0 : undefined, $1 : undefined, x : Ast.Value) : (C.FilterSlot|C.DomainIndependentFilterSlot)|null => C.makeFilter($loader, p, '>=', x)), C.filterKeyFn, {  });
       $grammar.addRule("if_filter", [new $runtime.NonTerminal("out_param_Any", ["type" as ("type" extends keyof ReturnType<typeof C.paramKeyFn> ? string : void), ((Type.Time)) as ReturnType<typeof C.paramKeyFn>["type"]]), "is", new $runtime.Choice(["before", "earlier than", "sooner than"]), new $runtime.NonTerminal("constant_Time", undefined)], ((p : C.ParamSlot, $0 : undefined, $1 : undefined, x : Ast.Value) : (C.FilterSlot|C.DomainIndependentFilterSlot)|null => C.makeFilter($loader, p, '<=', x)), C.filterKeyFn, {  });
   }
   $grammar.addRule("if_filter", ["the", new $runtime.NonTerminal("out_param_Array__Any", undefined), new $runtime.Choice(["contain", "include"]), new $runtime.NonTerminal("constant_Any", undefined)], (($0 : undefined, p : C.ParamSlot, $1 : undefined, x : Ast.Value) : (C.FilterSlot|C.DomainIndependentFilterSlot)|null => C.makeFilter($loader, p, 'contains', x)), C.filterKeyFn, {  });
   $grammar.addRule("if_filter", ["the", new $runtime.NonTerminal("out_param_Array__Any", undefined), "do not", new $runtime.Choice(["contain", "include"]), new $runtime.NonTerminal("constant_Any", undefined)], (($0 : undefined, p : C.ParamSlot, $1 : undefined, $2 : undefined, x : Ast.Value) : (C.FilterSlot|C.DomainIndependentFilterSlot)|null => C.makeFilter($loader, p, 'contains', x, true)), C.filterKeyFn, {  });
   $grammar.addRule("if_filter", ["the", new $runtime.NonTerminal("out_param_Any", ["type" as ("type" extends keyof ReturnType<typeof C.paramKeyFn> ? string : void), ((Type.String)) as ReturnType<typeof C.paramKeyFn>["type"]]), new $runtime.Choice(["contains", "includes"]), new $runtime.NonTerminal("constant_String", undefined)], (($0 : undefined, p : C.ParamSlot, $1 : undefined, x : Ast.Value) : (C.FilterSlot|C.DomainIndependentFilterSlot)|null => C.makeFilter($loader, p, '=~', x)), C.filterKeyFn, {  });
   $grammar.addRule("if_filter", ["the", new $runtime.NonTerminal("out_param_Any", ["type" as ("type" extends keyof ReturnType<typeof C.paramKeyFn> ? string : void), ((Type.String)) as ReturnType<typeof C.paramKeyFn>["type"]]), "does not", new $runtime.Choice(["contain", "include"]), new $runtime.NonTerminal("constant_String", undefined)], (($0 : undefined, p : C.ParamSlot, $1 : undefined, $2 : undefined, x : Ast.Value) : (C.FilterSlot|C.DomainIndependentFilterSlot)|null => C.makeFilter($loader, p, '=~', x, true)), C.filterKeyFn, {  });
   $grammar.addRule("if_filter", ["the", new $runtime.NonTerminal("out_param_Any", ["type" as ("type" extends keyof ReturnType<typeof C.paramKeyFn> ? string : void), ((Type.RecurrentTimeSpecification)) as ReturnType<typeof C.paramKeyFn>["type"]]), new $runtime.Choice(["contains", "includes"]), new $runtime.NonTerminal("constant_Date", undefined)], (($0 : undefined, p : C.ParamSlot, $1 : undefined, x : Ast.Value) : (C.FilterSlot|C.DomainIndependentFilterSlot)|null => C.makeFilter($loader, p, '=~', x)), C.filterKeyFn, {  });
   $grammar.addRule("if_filter", ["the", new $runtime.NonTerminal("out_param_Any", ["type" as ("type" extends keyof ReturnType<typeof C.paramKeyFn> ? string : void), ((Type.RecurrentTimeSpecification)) as ReturnType<typeof C.paramKeyFn>["type"]]), "does not", new $runtime.Choice(["contain", "include"]), new $runtime.NonTerminal("constant_Date", undefined)], (($0 : undefined, p : C.ParamSlot, $1 : undefined, $2 : undefined, x : Ast.Value) : (C.FilterSlot|C.DomainIndependentFilterSlot)|null => C.makeFilter($loader, p, '=~', x, true)), C.filterKeyFn, {  });
   $grammar.addRule("if_filter", ["the", new $runtime.NonTerminal("out_param_Any", ["type" as ("type" extends keyof ReturnType<typeof C.paramKeyFn> ? string : void), ((Type.RecurrentTimeSpecification)) as ReturnType<typeof C.paramKeyFn>["type"]]), new $runtime.Choice(["contains", "includes"]), new $runtime.NonTerminal("constant_Time", undefined)], (($0 : undefined, p : C.ParamSlot, $1 : undefined, x : Ast.Value) : (C.FilterSlot|C.DomainIndependentFilterSlot)|null => C.makeFilter($loader, p, '=~', x)), C.filterKeyFn, {  });
   $grammar.addRule("if_filter", ["the", new $runtime.NonTerminal("out_param_Any", ["type" as ("type" extends keyof ReturnType<typeof C.paramKeyFn> ? string : void), ((Type.RecurrentTimeSpecification)) as ReturnType<typeof C.paramKeyFn>["type"]]), "does not", new $runtime.Choice(["contain", "include"]), new $runtime.NonTerminal("constant_Time", undefined)], (($0 : undefined, p : C.ParamSlot, $1 : undefined, $2 : undefined, x : Ast.Value) : (C.FilterSlot|C.DomainIndependentFilterSlot)|null => C.makeFilter($loader, p, '=~', x, true)), C.filterKeyFn, {  });
   $grammar.addRule("if_filter", [new $runtime.NonTerminal("constant_String", undefined), "is in", new $runtime.NonTerminal("the_out_param_Any", ["type" as ("type" extends keyof ReturnType<typeof C.paramKeyFn> ? string : void), ((Type.String)) as ReturnType<typeof C.paramKeyFn>["type"]])], ((x : Ast.Value, $0 : undefined, p : C.ParamSlot) : (C.FilterSlot|C.DomainIndependentFilterSlot)|null => C.makeFilter($loader, p, '=~', x)), C.filterKeyFn, {  });
   if (!$options.flags.turking) {
       if ($options.flags.range_filters) {
           $grammar.addRule("if_filter", [new $runtime.NonTerminal("range_filter", undefined)], (($0 : C.FilterSlot) : (C.FilterSlot|C.DomainIndependentFilterSlot)|null => { return $0; }), C.filterKeyFn, { identity: (true) });
       }
   }
   $grammar.declareSymbol("edge_filter");
   if (!$options.flags.turking) {
       $grammar.addRule("edge_filter", ["the", new $runtime.NonTerminal("out_param_Any", undefined), new $runtime.Choice(["becomes", "becomes equal to"]), new $runtime.NonTerminal("constant_Any", undefined)], (($0 : undefined, p : C.ParamSlot, $1 : undefined, x : Ast.Value) : (C.FilterSlot)|null => C.makeFilter($loader, p, '==', x)), C.filterKeyFn, {  });
   }
   $grammar.addRule("edge_filter", [new $runtime.NonTerminal("out_param_Any", ["type" as ("type" extends keyof ReturnType<typeof C.paramKeyFn> ? string : void), ((Type.Number)) as ReturnType<typeof C.paramKeyFn>["type"]]), new $runtime.Choice(["is now", "becomes", "goes"]), new $runtime.Choice(["greater than", "higher than", "larger than", "more than", "at least", "not less than"]), new $runtime.NonTerminal("constant_Number", undefined)], ((p : C.ParamSlot, $0 : undefined, $1 : undefined, x : Ast.NumberValue) : (C.FilterSlot)|null => C.makeFilter($loader, p, '>=', x)), C.filterKeyFn, {  });
   $grammar.addRule("edge_filter", [new $runtime.NonTerminal("out_param_Any", ["type" as ("type" extends keyof ReturnType<typeof C.paramKeyFn> ? string : void), ((Type.Number)) as ReturnType<typeof C.paramKeyFn>["type"]]), new $runtime.Choice(["is now", "becomes", "goes"]), new $runtime.Choice(["smaller than", "lower than", "less than", "at most", "not more than"]), new $runtime.NonTerminal("constant_Number", undefined)], ((p : C.ParamSlot, $0 : undefined, $1 : undefined, x : Ast.NumberValue) : (C.FilterSlot)|null => C.makeFilter($loader, p, '<=', x)), C.filterKeyFn, {  });
   $grammar.addRule("edge_filter", [new $runtime.NonTerminal("out_param_Any", ["type" as ("type" extends keyof ReturnType<typeof C.paramKeyFn> ? string : void), ((Type.Currency)) as ReturnType<typeof C.paramKeyFn>["type"]]), new $runtime.Choice(["is now", "becomes", "goes"]), new $runtime.Choice(["more expensive", "more costly", "more"]), "than", new $runtime.NonTerminal("constant_Currency", undefined)], ((p : C.ParamSlot, $0 : undefined, $1 : undefined, $2 : undefined, x : Ast.Value) : (C.FilterSlot)|null => C.makeFilter($loader, p, '>=', x)), C.filterKeyFn, {  });
   $grammar.addRule("edge_filter", [new $runtime.NonTerminal("out_param_Any", ["type" as ("type" extends keyof ReturnType<typeof C.paramKeyFn> ? string : void), ((Type.Currency)) as ReturnType<typeof C.paramKeyFn>["type"]]), new $runtime.Choice(["is now", "becomes", "goes"]), new $runtime.Choice(["less expensive", "cheaper", "less"]), "than", new $runtime.NonTerminal("constant_Currency", undefined)], ((p : C.ParamSlot, $0 : undefined, $1 : undefined, $2 : undefined, x : Ast.Value) : (C.FilterSlot)|null => C.makeFilter($loader, p, '<=', x)), C.filterKeyFn, {  });
   $grammar.addRule("edge_filter", [new $runtime.NonTerminal("out_param_Any", ["type" as ("type" extends keyof ReturnType<typeof C.paramKeyFn> ? string : void), ((new Type.Measure('ms'))) as ReturnType<typeof C.paramKeyFn>["type"]]), new $runtime.Choice(["is now", "becomes", "goes"]), "longer than", new $runtime.NonTerminal("constant_Measure_ms", undefined)], ((p : C.ParamSlot, $0 : undefined, $1 : undefined, x : Ast.Value) : (C.FilterSlot)|null => C.makeFilter($loader, p, '>=', x)), C.filterKeyFn, {  });
   $grammar.addRule("edge_filter", [new $runtime.NonTerminal("out_param_Any", ["type" as ("type" extends keyof ReturnType<typeof C.paramKeyFn> ? string : void), ((new Type.Measure('ms'))) as ReturnType<typeof C.paramKeyFn>["type"]]), new $runtime.Choice(["is now", "becomes", "goes"]), "shorter than", new $runtime.NonTerminal("constant_Measure_ms", undefined)], ((p : C.ParamSlot, $0 : undefined, $1 : undefined, x : Ast.Value) : (C.FilterSlot)|null => C.makeFilter($loader, p, '<=', x)), C.filterKeyFn, {  });
   $grammar.addRule("edge_filter", [new $runtime.NonTerminal("out_param_Any", ["type" as ("type" extends keyof ReturnType<typeof C.paramKeyFn> ? string : void), ((new Type.Measure('byte'))) as ReturnType<typeof C.paramKeyFn>["type"]]), new $runtime.Choice(["is now", "becomes", "goes"]), new $runtime.Choice(["larger", "more"]), "than", new $runtime.NonTerminal("constant_Measure_byte", undefined)], ((p : C.ParamSlot, $0 : undefined, $1 : undefined, $2 : undefined, x : Ast.Value) : (C.FilterSlot)|null => C.makeFilter($loader, p, '>=', x)), C.filterKeyFn, {  });
   $grammar.addRule("edge_filter", [new $runtime.NonTerminal("out_param_Any", ["type" as ("type" extends keyof ReturnType<typeof C.paramKeyFn> ? string : void), ((new Type.Measure('byte'))) as ReturnType<typeof C.paramKeyFn>["type"]]), new $runtime.Choice(["is now", "becomes", "goes"]), new $runtime.Choice(["smaller", "less"]), "than", new $runtime.NonTerminal("constant_Measure_byte", undefined)], ((p : C.ParamSlot, $0 : undefined, $1 : undefined, $2 : undefined, x : Ast.Value) : (C.FilterSlot)|null => C.makeFilter($loader, p, '<=', x)), C.filterKeyFn, {  });
   $grammar.addRule("edge_filter", [new $runtime.NonTerminal("out_param_Any", ["type" as ("type" extends keyof ReturnType<typeof C.paramKeyFn> ? string : void), ((new Type.Measure('kg'))) as ReturnType<typeof C.paramKeyFn>["type"]]), new $runtime.Choice(["is now", "becomes", "goes"]), new $runtime.Choice(["heavier", "larger"]), "than", new $runtime.NonTerminal("constant_Measure_kg", undefined)], ((p : C.ParamSlot, $0 : undefined, $1 : undefined, $2 : undefined, x : Ast.Value) : (C.FilterSlot)|null => C.makeFilter($loader, p, '>=', x)), C.filterKeyFn, {  });
   $grammar.addRule("edge_filter", [new $runtime.NonTerminal("out_param_Any", ["type" as ("type" extends keyof ReturnType<typeof C.paramKeyFn> ? string : void), ((new Type.Measure('kg'))) as ReturnType<typeof C.paramKeyFn>["type"]]), new $runtime.Choice(["is now", "becomes", "goes"]), new $runtime.Choice(["lighter", "smaller"]), "than", new $runtime.NonTerminal("constant_Measure_kg", undefined)], ((p : C.ParamSlot, $0 : undefined, $1 : undefined, $2 : undefined, x : Ast.Value) : (C.FilterSlot)|null => C.makeFilter($loader, p, '<=', x)), C.filterKeyFn, {  });
   $grammar.addRule("edge_filter", [new $runtime.NonTerminal("out_param_Any", ["type" as ("type" extends keyof ReturnType<typeof C.paramKeyFn> ? string : void), ((new Type.Measure('C'))) as ReturnType<typeof C.paramKeyFn>["type"]]), new $runtime.Choice(["is now", "becomes", "goes"]), new $runtime.Choice(["hotter", "higher"]), "than", new $runtime.NonTerminal("constant_Measure_C", undefined)], ((p : C.ParamSlot, $0 : undefined, $1 : undefined, $2 : undefined, x : Ast.Value) : (C.FilterSlot)|null => C.makeFilter($loader, p, '>=', x)), C.filterKeyFn, {  });
   $grammar.addRule("edge_filter", [new $runtime.NonTerminal("out_param_Any", ["type" as ("type" extends keyof ReturnType<typeof C.paramKeyFn> ? string : void), ((new Type.Measure('C'))) as ReturnType<typeof C.paramKeyFn>["type"]]), new $runtime.Choice(["is now", "becomes", "goes"]), new $runtime.Choice(["cooler", "colder", "lower"]), "than", new $runtime.NonTerminal("constant_Measure_C", undefined)], ((p : C.ParamSlot, $0 : undefined, $1 : undefined, $2 : undefined, x : Ast.Value) : (C.FilterSlot)|null => C.makeFilter($loader, p, '<=', x)), C.filterKeyFn, {  });
   $grammar.addRule("edge_filter", [new $runtime.NonTerminal("out_param_Any", ["type" as ("type" extends keyof ReturnType<typeof C.paramKeyFn> ? string : void), ((new Type.Measure('m'))) as ReturnType<typeof C.paramKeyFn>["type"]]), new $runtime.Choice(["is now", "becomes", "goes"]), new $runtime.Choice(["farther", "more distant", "longer"]), "than", new $runtime.NonTerminal("constant_Measure_m", undefined)], ((p : C.ParamSlot, $0 : undefined, $1 : undefined, $2 : undefined, x : Ast.Value) : (C.FilterSlot)|null => C.makeFilter($loader, p, '>=', x)), C.filterKeyFn, {  });
   $grammar.addRule("edge_filter", [new $runtime.NonTerminal("out_param_Any", ["type" as ("type" extends keyof ReturnType<typeof C.paramKeyFn> ? string : void), ((new Type.Measure('m'))) as ReturnType<typeof C.paramKeyFn>["type"]]), new $runtime.Choice(["is now", "becomes", "goes"]), new $runtime.Choice(["nearer", "closer", "shorter"]), "than", new $runtime.NonTerminal("constant_Measure_m", undefined)], ((p : C.ParamSlot, $0 : undefined, $1 : undefined, $2 : undefined, x : Ast.Value) : (C.FilterSlot)|null => C.makeFilter($loader, p, '<=', x)), C.filterKeyFn, {  });
   $grammar.addRule("edge_filter", [new $runtime.NonTerminal("out_param_Any", ["type" as ("type" extends keyof ReturnType<typeof C.paramKeyFn> ? string : void), ((new Type.Measure('m2'))) as ReturnType<typeof C.paramKeyFn>["type"]]), new $runtime.Choice(["is now", "becomes", "goes"]), new $runtime.Choice(["bigger", "larger"]), "than", new $runtime.NonTerminal("constant_Measure_m2", undefined)], ((p : C.ParamSlot, $0 : undefined, $1 : undefined, $2 : undefined, x : Ast.Value) : (C.FilterSlot)|null => C.makeFilter($loader, p, '>=', x)), C.filterKeyFn, {  });
   $grammar.addRule("edge_filter", [new $runtime.NonTerminal("out_param_Any", ["type" as ("type" extends keyof ReturnType<typeof C.paramKeyFn> ? string : void), ((new Type.Measure('m2'))) as ReturnType<typeof C.paramKeyFn>["type"]]), new $runtime.Choice(["is now", "becomes", "goes"]), "smaller", "than", new $runtime.NonTerminal("constant_Measure_m2", undefined)], ((p : C.ParamSlot, $0 : undefined, $1 : undefined, $2 : undefined, x : Ast.Value) : (C.FilterSlot)|null => C.makeFilter($loader, p, '<=', x)), C.filterKeyFn, {  });
   $grammar.addRule("edge_filter", [new $runtime.NonTerminal("out_param_Any", ["type" as ("type" extends keyof ReturnType<typeof C.paramKeyFn> ? string : void), ((new Type.Measure('m3'))) as ReturnType<typeof C.paramKeyFn>["type"]]), new $runtime.Choice(["is now", "becomes", "goes"]), new $runtime.Choice(["bigger", "larger"]), "than", new $runtime.NonTerminal("constant_Measure_m3", undefined)], ((p : C.ParamSlot, $0 : undefined, $1 : undefined, $2 : undefined, x : Ast.Value) : (C.FilterSlot)|null => C.makeFilter($loader, p, '>=', x)), C.filterKeyFn, {  });
   $grammar.addRule("edge_filter", [new $runtime.NonTerminal("out_param_Any", ["type" as ("type" extends keyof ReturnType<typeof C.paramKeyFn> ? string : void), ((new Type.Measure('m3'))) as ReturnType<typeof C.paramKeyFn>["type"]]), new $runtime.Choice(["is now", "becomes", "goes"]), "smaller", "than", new $runtime.NonTerminal("constant_Measure_m3", undefined)], ((p : C.ParamSlot, $0 : undefined, $1 : undefined, $2 : undefined, x : Ast.Value) : (C.FilterSlot)|null => C.makeFilter($loader, p, '<=', x)), C.filterKeyFn, {  });
   $grammar.addRule("edge_filter", [new $runtime.NonTerminal("out_param_Any", ["type" as ("type" extends keyof ReturnType<typeof C.paramKeyFn> ? string : void), ((new Type.Measure('mps'))) as ReturnType<typeof C.paramKeyFn>["type"]]), new $runtime.Choice(["is now", "becomes", "goes"]), new $runtime.Choice(["faster", "quicker", "speedier"]), "than", new $runtime.NonTerminal("constant_Measure_mps", undefined)], ((p : C.ParamSlot, $0 : undefined, $1 : undefined, $2 : undefined, x : Ast.Value) : (C.FilterSlot)|null => C.makeFilter($loader, p, '>=', x)), C.filterKeyFn, {  });
   $grammar.addRule("edge_filter", [new $runtime.NonTerminal("out_param_Any", ["type" as ("type" extends keyof ReturnType<typeof C.paramKeyFn> ? string : void), ((new Type.Measure('mps'))) as ReturnType<typeof C.paramKeyFn>["type"]]), new $runtime.Choice(["is now", "becomes", "goes"]), new $runtime.Choice(["slower", "more slowly"]), "than", new $runtime.NonTerminal("constant_Measure_mps", undefined)], ((p : C.ParamSlot, $0 : undefined, $1 : undefined, $2 : undefined, x : Ast.Value) : (C.FilterSlot)|null => C.makeFilter($loader, p, '<=', x)), C.filterKeyFn, {  });
   $grammar.addRule("edge_filter", [new $runtime.NonTerminal("out_param_Any", ["type" as ("type" extends keyof ReturnType<typeof C.paramKeyFn> ? string : void), ((Type.Date)) as ReturnType<typeof C.paramKeyFn>["type"]]), new $runtime.Choice(["is now", "becomes", "goes"]), new $runtime.Choice(["after", "later than"]), new $runtime.NonTerminal("constant_Date", undefined)], ((p : C.ParamSlot, $0 : undefined, $1 : undefined, x : Ast.Value) : (C.FilterSlot)|null => C.makeFilter($loader, p, '>=', x)), C.filterKeyFn, {  });
   $grammar.addRule("edge_filter", [new $runtime.NonTerminal("out_param_Any", ["type" as ("type" extends keyof ReturnType<typeof C.paramKeyFn> ? string : void), ((Type.Date)) as ReturnType<typeof C.paramKeyFn>["type"]]), new $runtime.Choice(["is now", "becomes", "goes"]), new $runtime.Choice(["before", "earlier than", "sooner than"]), new $runtime.NonTerminal("constant_Date", undefined)], ((p : C.ParamSlot, $0 : undefined, $1 : undefined, x : Ast.Value) : (C.FilterSlot)|null => C.makeFilter($loader, p, '<=', x)), C.filterKeyFn, {  });
   $grammar.addRule("edge_filter", [new $runtime.NonTerminal("out_param_Any", ["type" as ("type" extends keyof ReturnType<typeof C.paramKeyFn> ? string : void), ((Type.Time)) as ReturnType<typeof C.paramKeyFn>["type"]]), new $runtime.Choice(["is now", "becomes", "goes"]), new $runtime.Choice(["after", "later than"]), new $runtime.NonTerminal("constant_Time", undefined)], ((p : C.ParamSlot, $0 : undefined, $1 : undefined, x : Ast.Value) : (C.FilterSlot)|null => C.makeFilter($loader, p, '>=', x)), C.filterKeyFn, {  });
   $grammar.addRule("edge_filter", [new $runtime.NonTerminal("out_param_Any", ["type" as ("type" extends keyof ReturnType<typeof C.paramKeyFn> ? string : void), ((Type.Time)) as ReturnType<typeof C.paramKeyFn>["type"]]), new $runtime.Choice(["is now", "becomes", "goes"]), new $runtime.Choice(["before", "earlier than", "sooner than"]), new $runtime.NonTerminal("constant_Time", undefined)], ((p : C.ParamSlot, $0 : undefined, $1 : undefined, x : Ast.Value) : (C.FilterSlot)|null => C.makeFilter($loader, p, '<=', x)), C.filterKeyFn, {  });
   $grammar.declareSymbol("either_filter");
   $grammar.addRule("either_filter", ["the", new $runtime.NonTerminal("out_param_Any", undefined), new $runtime.Choice(["is", "is equal to", "is one of", "is either"]), new $runtime.NonTerminal("constant_or_pairs", ["type" as ("type" extends keyof ReturnType<typeof C.valueArrayKeyFn> ? string : void), 1, "type" as ("type" extends keyof ReturnType<typeof C.paramKeyFn> ? string : void)])], (($0 : undefined, param : C.ParamSlot, $1 : undefined, pair : [Ast.Value, Ast.Value]) : (C.FilterSlot)|null => {
        // param is a Value.VarRef
        //console.log('param: ' + param.name);
        const [v1, v2] = pair;
        let vtype = v1.getType();
        if (vtype.isBoolean) // "is equal to true or false" does not make sense
            return null;
        return C.makeFilter($loader, param, 'in_array', new Ast.Value.Array([v1, v2]));
    }), C.filterKeyFn, {  });
   $grammar.addRule("either_filter", ["the", new $runtime.NonTerminal("out_param_Any", undefined), new $runtime.Choice(["is", "is equal to", "is one of", "is either"]), new $runtime.NonTerminal("constant_or_pairs", ["type" as ("type" extends keyof ReturnType<typeof C.valueArrayKeyFn> ? string : void), ((Type.String)) as ReturnType<typeof C.valueArrayKeyFn>["type"]])], (($0 : undefined, param : C.ParamSlot, $1 : undefined, pair : [Ast.Value, Ast.Value]) : (C.FilterSlot)|null => {
        // param is a Value.VarRef
        //console.log('param: ' + param.name);
        const [v1, v2] = pair;
        let vtype = v1.getType();
        if (vtype.isBoolean) // "is equal to true or false" does not make sense
            return null;
        return C.makeFilter($loader, param, 'in_array', new Ast.Value.Array([v1, v2]));
    }), C.filterKeyFn, {  });
   $grammar.addRule("either_filter", ["the", new $runtime.NonTerminal("out_param_Any", undefined), "is", new $runtime.Choice(["not", "neither"]), new $runtime.NonTerminal("constant_nor_pairs", ["type" as ("type" extends keyof ReturnType<typeof C.valueArrayKeyFn> ? string : void), 1, "type" as ("type" extends keyof ReturnType<typeof C.paramKeyFn> ? string : void)])], (($0 : undefined, param : C.ParamSlot, $1 : undefined, $2 : undefined, pair : [Ast.Value, Ast.Value]) : (C.FilterSlot)|null => {
        // param is a Value.VarRef
        //console.log('param: ' + param.name);
        const [v1, v2] = pair;
        let vtype = v1.getType();
        if (vtype.isBoolean) // "is neither true nor false" does not make sense
            return null;
        return C.makeFilter($loader, param, 'in_array', new Ast.Value.Array([v1, v2]), true);
    }), C.filterKeyFn, {  });
   $grammar.declareSymbol("range");
   $grammar.addRule("range", ["between", new $runtime.NonTerminal("constant_Numeric", undefined), "and", new $runtime.NonTerminal("constant_Numeric", ["type" as ("type" extends keyof ReturnType<typeof C.valueKeyFn> ? string : void), 1, "type" as ("type" extends keyof ReturnType<typeof C.valueKeyFn> ? string : void)])], (($0 : undefined, v1 : Ast.Value, $1 : undefined, v2 : Ast.Value) : ([Ast.Value, Ast.Value])|null => {
        assert(v1.getType().equals(v2.getType()));
        if (v1.equals(v2))
            return null;
        return [v1, v2];
    }), C.valueArrayKeyFn, {  });
   $grammar.addRule("range", ["in the range from", new $runtime.NonTerminal("constant_Numeric", undefined), "to", new $runtime.NonTerminal("constant_Numeric", ["type" as ("type" extends keyof ReturnType<typeof C.valueKeyFn> ? string : void), 1, "type" as ("type" extends keyof ReturnType<typeof C.valueKeyFn> ? string : void)])], (($0 : undefined, v1 : Ast.Value, $1 : undefined, v2 : Ast.Value) : ([Ast.Value, Ast.Value])|null => {
        assert(v1.getType().equals(v2.getType()));
        if (v1.equals(v2))
            return null;
        return [v1, v2];
    }), C.valueArrayKeyFn, {  });
   $grammar.declareSymbol("range_filter");
   $grammar.addRule("range_filter", ["the", new $runtime.NonTerminal("out_param_Any", ["is_numeric" as ("is_numeric" extends keyof ReturnType<typeof C.paramKeyFn> ? string : void), (true) as ReturnType<typeof C.paramKeyFn>["is_numeric"]]), "is", new $runtime.NonTerminal("range", ["type" as ("type" extends keyof ReturnType<typeof C.valueArrayKeyFn> ? string : void), 1, "type" as ("type" extends keyof ReturnType<typeof C.paramKeyFn> ? string : void)])], (($0 : undefined, param : C.ParamSlot, $1 : undefined, range : [Ast.Value, Ast.Value]) : (C.FilterSlot)|null => {
        const [v1, v2] = range;
        const f1 = C.makeFilter($loader, param, '>=', v1);
        const f2 = C.makeFilter($loader, param, '<=', v2);
        if (!f1 || !f2)
            return null;
        return { schema: f1.schema, ptype: f1.ptype,
            ast: new Ast.BooleanExpression.And(null, [f1.ast, f2.ast]) };
    }), C.filterKeyFn, {  });
   $grammar.declareSymbol("with_filter");
   $grammar.addRule("with_filter", ["the", new $runtime.NonTerminal("out_param_Any", undefined), "equal to", new $runtime.NonTerminal("constant_Any", undefined)], (($0 : undefined, p : C.ParamSlot, $1 : undefined, x : Ast.Value) : (C.FilterSlot)|null => C.makeFilter($loader, p, '==', x)), C.filterKeyFn, {  });
   $grammar.addRule("with_filter", ["the", new $runtime.NonTerminal("out_param_Any", undefined), "equal to", new $runtime.NonTerminal("constant_Any", undefined)], (($0 : undefined, p : C.ParamSlot, $1 : undefined, x : Ast.Value) : (C.FilterSlot)|null => C.makeFilter($loader, p, '==', x)), C.filterKeyFn, {  });
   if (!$options.flags.turking) {
       $grammar.addRule("with_filter", ["the", new $runtime.NonTerminal("out_param_Any", ["type" as ("type" extends keyof ReturnType<typeof C.paramKeyFn> ? string : void), ((Type.String)) as ReturnType<typeof C.paramKeyFn>["type"]]), new $runtime.Choice(["containing", "including"]), new $runtime.NonTerminal("constant_String", undefined)], (($0 : undefined, p : C.ParamSlot, $1 : undefined, x : Ast.Value) : (C.FilterSlot)|null => C.makeFilter($loader, p, '=~', x)), C.filterKeyFn, {  });
   }
   if (!$options.flags.turking) {
       $grammar.addRule("with_filter", [new $runtime.NonTerminal("constant_String", undefined), new $runtime.Choice(["in the", "in its", "in their"]), new $runtime.NonTerminal("out_param_Any", ["type" as ("type" extends keyof ReturnType<typeof C.paramKeyFn> ? string : void), ((Type.String)) as ReturnType<typeof C.paramKeyFn>["type"]])], ((x : Ast.Value, $0 : undefined, p : C.ParamSlot) : (C.FilterSlot)|null => C.makeFilter($loader, p, '=~', x)), C.filterKeyFn, {  });
   }
   if ($options.flags.schema_org) {
       $grammar.addRule("with_filter", ["the", new $runtime.NonTerminal("out_param_Any", ["type" as ("type" extends keyof ReturnType<typeof C.paramKeyFn> ? string : void), ((Type.String)) as ReturnType<typeof C.paramKeyFn>["type"]]), new $runtime.Choice(["being", "containing"]), new $runtime.NonTerminal("constant_String", undefined)], (($0 : undefined, p : C.ParamSlot, $1 : undefined, x : Ast.Value) : (C.FilterSlot)|null => C.makeFilter($loader, p, '=~', x)), C.filterKeyFn, {  });
   }
   if ($options.flags.schema_org) {
       $grammar.addRule("with_filter", [new $runtime.NonTerminal("constant_String", undefined), new $runtime.Choice(["as", "in the"]), new $runtime.NonTerminal("out_param_Any", ["type" as ("type" extends keyof ReturnType<typeof C.paramKeyFn> ? string : void), ((Type.String)) as ReturnType<typeof C.paramKeyFn>["type"]])], ((x : Ast.Value, $0 : undefined, p : C.ParamSlot) : (C.FilterSlot)|null => C.makeFilter($loader, p, '=~', x)), C.filterKeyFn, {  });
   }
   $grammar.addRule("with_filter", ["the", new $runtime.NonTerminal("out_param_Any", ["type" as ("type" extends keyof ReturnType<typeof C.paramKeyFn> ? string : void), ((Type.String)) as ReturnType<typeof C.paramKeyFn>["type"]]), "equal to", new $runtime.NonTerminal("constant_String_ContextRef", undefined)], (($0 : undefined, p : C.ParamSlot, $1 : undefined, x : Ast.ContextRefValue) : (C.FilterSlot)|null => C.makeFilter($loader, p, '=~', x)), C.filterKeyFn, {  });
   if (!$options.flags.turking) {
       $grammar.addRule("with_filter", ["the", new $runtime.NonTerminal("out_param_Any", ["type" as ("type" extends keyof ReturnType<typeof C.paramKeyFn> ? string : void), ((Type.String)) as ReturnType<typeof C.paramKeyFn>["type"]]), new $runtime.Choice(["containing", "including"]), new $runtime.NonTerminal("constant_String_ContextRef", undefined)], (($0 : undefined, p : C.ParamSlot, $1 : undefined, x : Ast.ContextRefValue) : (C.FilterSlot)|null => C.makeFilter($loader, p, '=~', x)), C.filterKeyFn, {  });
   }
   if (!$options.flags.turking) {
       $grammar.addRule("with_filter", [new $runtime.NonTerminal("constant_String_ContextRef", undefined), new $runtime.Choice(["in the", "in its", "in their"]), new $runtime.NonTerminal("out_param_Any", ["type" as ("type" extends keyof ReturnType<typeof C.paramKeyFn> ? string : void), ((Type.String)) as ReturnType<typeof C.paramKeyFn>["type"]])], ((x : Ast.ContextRefValue, $0 : undefined, p : C.ParamSlot) : (C.FilterSlot)|null => C.makeFilter($loader, p, '=~', x)), C.filterKeyFn, {  });
   }
   if ($options.flags.schema_org) {
       $grammar.addRule("with_filter", ["the", new $runtime.NonTerminal("out_param_Any", ["type" as ("type" extends keyof ReturnType<typeof C.paramKeyFn> ? string : void), ((Type.String)) as ReturnType<typeof C.paramKeyFn>["type"]]), new $runtime.Choice(["being", "containing"]), new $runtime.NonTerminal("constant_String_ContextRef", undefined)], (($0 : undefined, p : C.ParamSlot, $1 : undefined, x : Ast.ContextRefValue) : (C.FilterSlot)|null => C.makeFilter($loader, p, '=~', x)), C.filterKeyFn, {  });
   }
   if ($options.flags.schema_org) {
       $grammar.addRule("with_filter", [new $runtime.NonTerminal("constant_String_ContextRef", undefined), new $runtime.Choice(["as", "in the"]), new $runtime.NonTerminal("out_param_Any", ["type" as ("type" extends keyof ReturnType<typeof C.paramKeyFn> ? string : void), ((Type.String)) as ReturnType<typeof C.paramKeyFn>["type"]])], ((x : Ast.ContextRefValue, $0 : undefined, p : C.ParamSlot) : (C.FilterSlot)|null => C.makeFilter($loader, p, '=~', x)), C.filterKeyFn, {  });
   }
   if (!$options.flags.turking) {
       $grammar.addRule("with_filter", ["the", new $runtime.NonTerminal("out_param_Any", ["type" as ("type" extends keyof ReturnType<typeof C.paramKeyFn> ? string : void), ((Type.String)) as ReturnType<typeof C.paramKeyFn>["type"]]), "containing both", new $runtime.NonTerminal("constant_String", undefined), "and", new $runtime.NonTerminal("constant_String", undefined)], (($0 : undefined, p : C.ParamSlot, $1 : undefined, x : Ast.Value, $2 : undefined, y : Ast.Value) : (C.FilterSlot)|null => C.makeAndFilter($loader, p, '=~', [x, y])), C.filterKeyFn, {  });
   }
   if (!$options.flags.turking) {
       $grammar.addRule("with_filter", ["the", new $runtime.NonTerminal("out_param_Any", ["type" as ("type" extends keyof ReturnType<typeof C.paramKeyFn> ? string : void), ((Type.String)) as ReturnType<typeof C.paramKeyFn>["type"]]), "containing neither", new $runtime.NonTerminal("constant_String", undefined), "nor", new $runtime.NonTerminal("constant_String", undefined)], (($0 : undefined, p : C.ParamSlot, $1 : undefined, x : Ast.Value, $2 : undefined, y : Ast.Value) : (C.FilterSlot)|null => C.makeAndFilter($loader, p, '=~', [x, y], true)), C.filterKeyFn, {  });
   }
   if (!$options.flags.turking) {
       $grammar.addRule("with_filter", ["both", new $runtime.NonTerminal("constant_String", undefined), "and", new $runtime.NonTerminal("constant_String", undefined), "in", new $runtime.NonTerminal("the_out_param_Any", ["type" as ("type" extends keyof ReturnType<typeof C.paramKeyFn> ? string : void), ((Type.String)) as ReturnType<typeof C.paramKeyFn>["type"]])], (($0 : undefined, x : Ast.Value, $1 : undefined, y : Ast.Value, $2 : undefined, p : C.ParamSlot) : (C.FilterSlot)|null => C.makeAndFilter($loader, p, '=~', [x, y])), C.filterKeyFn, {  });
   }
   if (!$options.flags.turking) {
       $grammar.addRule("with_filter", ["the", new $runtime.NonTerminal("out_param_Any", ["type" as ("type" extends keyof ReturnType<typeof C.paramKeyFn> ? string : void), ((Type.String)) as ReturnType<typeof C.paramKeyFn>["type"]]), "containing either", new $runtime.NonTerminal("constant_String", undefined), "or", new $runtime.NonTerminal("constant_String", undefined)], (($0 : undefined, p : C.ParamSlot, $1 : undefined, x : Ast.Value, $2 : undefined, y : Ast.Value) : (C.FilterSlot)|null => C.makeOrFilter($loader, p, '=~', [x, y])), C.filterKeyFn, {  });
   }
   if (!$options.flags.turking) {
       $grammar.addRule("with_filter", ["either", new $runtime.NonTerminal("constant_String", undefined), "or", new $runtime.NonTerminal("constant_String", undefined), "in", new $runtime.NonTerminal("the_out_param_Any", ["type" as ("type" extends keyof ReturnType<typeof C.paramKeyFn> ? string : void), ((Type.String)) as ReturnType<typeof C.paramKeyFn>["type"]])], (($0 : undefined, x : Ast.Value, $1 : undefined, y : Ast.Value, $2 : undefined, p : C.ParamSlot) : (C.FilterSlot)|null => C.makeOrFilter($loader, p, '=~', [x, y])), C.filterKeyFn, {  });
   }
   if (!$options.flags.turking) {
       $grammar.addRule("with_filter", ["the", new $runtime.NonTerminal("out_param_Any", ["type" as ("type" extends keyof ReturnType<typeof C.paramKeyFn> ? string : void), ((Type.String)) as ReturnType<typeof C.paramKeyFn>["type"]]), new $runtime.Choice(["containing", "including"]), new $runtime.NonTerminal("constant_String", undefined), new $runtime.Choice(["but not", "without", "but no"]), new $runtime.NonTerminal("constant_String", undefined)], (($0 : undefined, p : C.ParamSlot, $1 : undefined, x : Ast.Value, $2 : undefined, y : Ast.Value) : (C.FilterSlot)|null => C.makeButFilter($loader, p, '=~', [x, y])), C.filterKeyFn, {  });
   }
   $grammar.addRule("with_filter", [new $runtime.NonTerminal("out_param_Any", ["type" as ("type" extends keyof ReturnType<typeof C.paramKeyFn> ? string : void), ((Type.Number)) as ReturnType<typeof C.paramKeyFn>["type"]]), new $runtime.Choice(["greater than", "higher than", "larger than", "more than", "at least", "not less than"]), new $runtime.NonTerminal("constant_Number", undefined)], ((p : C.ParamSlot, $0 : undefined, x : Ast.NumberValue) : (C.FilterSlot)|null => C.makeFilter($loader, p, '>=', x)), C.filterKeyFn, {  });
   $grammar.addRule("with_filter", [new $runtime.NonTerminal("out_param_Any", ["type" as ("type" extends keyof ReturnType<typeof C.paramKeyFn> ? string : void), ((Type.Number)) as ReturnType<typeof C.paramKeyFn>["type"]]), new $runtime.Choice(["smaller than", "lower than", "less than", "at most", "not more than"]), new $runtime.NonTerminal("constant_Number", undefined)], ((p : C.ParamSlot, $0 : undefined, x : Ast.NumberValue) : (C.FilterSlot)|null => C.makeFilter($loader, p, '<=', x)), C.filterKeyFn, {  });
   $grammar.addRule("with_filter", [new $runtime.NonTerminal("constant_Number", undefined), new $runtime.Choice(["or more", "or higher", "or greater", "or above"]), "the", new $runtime.NonTerminal("out_param_Any", ["type" as ("type" extends keyof ReturnType<typeof C.paramKeyFn> ? string : void), ((Type.Number)) as ReturnType<typeof C.paramKeyFn>["type"]])], ((x : Ast.NumberValue, $0 : undefined, $1 : undefined, p : C.ParamSlot) : (C.FilterSlot)|null => C.makeFilter($loader, p, '>=', x)), C.filterKeyFn, {  });
   $grammar.addRule("with_filter", [new $runtime.NonTerminal("constant_Number", undefined), new $runtime.Choice(["or less", "or lower", "or below"]), "the", new $runtime.NonTerminal("out_param_Any", ["type" as ("type" extends keyof ReturnType<typeof C.paramKeyFn> ? string : void), ((Type.Number)) as ReturnType<typeof C.paramKeyFn>["type"]])], ((x : Ast.NumberValue, $0 : undefined, $1 : undefined, p : C.ParamSlot) : (C.FilterSlot)|null => C.makeFilter($loader, p, '<=', x)), C.filterKeyFn, {  });
   if (!$options.flags.turking) {
       $grammar.addRule("with_filter", ["over", new $runtime.NonTerminal("constant_Number", undefined), new $runtime.NonTerminal("out_param_Any", ["type" as ("type" extends keyof ReturnType<typeof C.paramKeyFn> ? string : void), ((Type.Number)) as ReturnType<typeof C.paramKeyFn>["type"]])], (($0 : undefined, x : Ast.NumberValue, p : C.ParamSlot) : (C.FilterSlot)|null => C.makeFilter($loader, p, '>=', x)), C.filterKeyFn, {  });
   }
   $grammar.addRule("with_filter", [new $runtime.NonTerminal("out_param_Any", ["type" as ("type" extends keyof ReturnType<typeof C.paramKeyFn> ? string : void), ((Type.Currency)) as ReturnType<typeof C.paramKeyFn>["type"]]), new $runtime.Choice(["more expensive", "more costly", "more"]), "than", new $runtime.NonTerminal("constant_Currency", undefined)], ((p : C.ParamSlot, $0 : undefined, $1 : undefined, x : Ast.Value) : (C.FilterSlot)|null => C.makeFilter($loader, p, '>=', x)), C.filterKeyFn, {  });
   $grammar.addRule("with_filter", [new $runtime.NonTerminal("out_param_Any", ["type" as ("type" extends keyof ReturnType<typeof C.paramKeyFn> ? string : void), ((Type.Currency)) as ReturnType<typeof C.paramKeyFn>["type"]]), new $runtime.Choice(["less expensive", "cheaper", "less"]), "than", new $runtime.NonTerminal("constant_Currency", undefined)], ((p : C.ParamSlot, $0 : undefined, $1 : undefined, x : Ast.Value) : (C.FilterSlot)|null => C.makeFilter($loader, p, '<=', x)), C.filterKeyFn, {  });
   $grammar.addRule("with_filter", [new $runtime.NonTerminal("constant_Currency", undefined), new $runtime.Choice(["or more", "or above"]), "the", new $runtime.NonTerminal("out_param_Any", ["type" as ("type" extends keyof ReturnType<typeof C.paramKeyFn> ? string : void), ((Type.Currency)) as ReturnType<typeof C.paramKeyFn>["type"]])], ((x : Ast.Value, $0 : undefined, $1 : undefined, p : C.ParamSlot) : (C.FilterSlot)|null => C.makeFilter($loader, p, '>=', x)), C.filterKeyFn, {  });
   $grammar.addRule("with_filter", [new $runtime.NonTerminal("constant_Currency", undefined), new $runtime.Choice(["or less", "or cheaper", "or below"]), "the", new $runtime.NonTerminal("out_param_Any", ["type" as ("type" extends keyof ReturnType<typeof C.paramKeyFn> ? string : void), ((Type.Currency)) as ReturnType<typeof C.paramKeyFn>["type"]])], ((x : Ast.Value, $0 : undefined, $1 : undefined, p : C.ParamSlot) : (C.FilterSlot)|null => C.makeFilter($loader, p, '<=', x)), C.filterKeyFn, {  });
   $grammar.addRule("with_filter", [new $runtime.NonTerminal("out_param_Any", ["type" as ("type" extends keyof ReturnType<typeof C.paramKeyFn> ? string : void), ((new Type.Measure('ms'))) as ReturnType<typeof C.paramKeyFn>["type"]]), "longer than", new $runtime.NonTerminal("constant_Measure_ms", undefined)], ((p : C.ParamSlot, $0 : undefined, x : Ast.Value) : (C.FilterSlot)|null => C.makeFilter($loader, p, '>=', x)), C.filterKeyFn, {  });
   $grammar.addRule("with_filter", [new $runtime.NonTerminal("out_param_Any", ["type" as ("type" extends keyof ReturnType<typeof C.paramKeyFn> ? string : void), ((new Type.Measure('ms'))) as ReturnType<typeof C.paramKeyFn>["type"]]), "shorter than", new $runtime.NonTerminal("constant_Measure_ms", undefined)], ((p : C.ParamSlot, $0 : undefined, x : Ast.Value) : (C.FilterSlot)|null => C.makeFilter($loader, p, '<=', x)), C.filterKeyFn, {  });
   $grammar.addRule("with_filter", [new $runtime.NonTerminal("constant_Measure_ms", undefined), new $runtime.Choice(["or longer", "or more"]), "the", new $runtime.NonTerminal("out_param_Any", ["type" as ("type" extends keyof ReturnType<typeof C.paramKeyFn> ? string : void), ((new Type.Measure('ms'))) as ReturnType<typeof C.paramKeyFn>["type"]])], ((x : Ast.Value, $0 : undefined, $1 : undefined, p : C.ParamSlot) : (C.FilterSlot)|null => C.makeFilter($loader, p, '>=', x)), C.filterKeyFn, {  });
   $grammar.addRule("with_filter", [new $runtime.NonTerminal("constant_Measure_ms", undefined), new $runtime.Choice(["or shorter", "or less"]), "the", new $runtime.NonTerminal("out_param_Any", ["type" as ("type" extends keyof ReturnType<typeof C.paramKeyFn> ? string : void), ((new Type.Measure('ms'))) as ReturnType<typeof C.paramKeyFn>["type"]])], ((x : Ast.Value, $0 : undefined, $1 : undefined, p : C.ParamSlot) : (C.FilterSlot)|null => C.makeFilter($loader, p, '<=', x)), C.filterKeyFn, {  });
   $grammar.addRule("with_filter", [new $runtime.NonTerminal("out_param_Any", ["type" as ("type" extends keyof ReturnType<typeof C.paramKeyFn> ? string : void), ((new Type.Measure('byte'))) as ReturnType<typeof C.paramKeyFn>["type"]]), new $runtime.Choice(["larger", "more"]), "than", new $runtime.NonTerminal("constant_Measure_byte", undefined)], ((p : C.ParamSlot, $0 : undefined, $1 : undefined, x : Ast.Value) : (C.FilterSlot)|null => C.makeFilter($loader, p, '>=', x)), C.filterKeyFn, {  });
   $grammar.addRule("with_filter", [new $runtime.NonTerminal("out_param_Any", ["type" as ("type" extends keyof ReturnType<typeof C.paramKeyFn> ? string : void), ((new Type.Measure('byte'))) as ReturnType<typeof C.paramKeyFn>["type"]]), new $runtime.Choice(["smaller", "less"]), "than", new $runtime.NonTerminal("constant_Measure_byte", undefined)], ((p : C.ParamSlot, $0 : undefined, $1 : undefined, x : Ast.Value) : (C.FilterSlot)|null => C.makeFilter($loader, p, '<=', x)), C.filterKeyFn, {  });
   $grammar.addRule("with_filter", [new $runtime.NonTerminal("constant_Measure_byte", undefined), new $runtime.Choice(["or larger", "or more"]), "the", new $runtime.NonTerminal("out_param_Any", ["type" as ("type" extends keyof ReturnType<typeof C.paramKeyFn> ? string : void), ((new Type.Measure('byte'))) as ReturnType<typeof C.paramKeyFn>["type"]])], ((x : Ast.Value, $0 : undefined, $1 : undefined, p : C.ParamSlot) : (C.FilterSlot)|null => C.makeFilter($loader, p, '>=', x)), C.filterKeyFn, {  });
   $grammar.addRule("with_filter", [new $runtime.NonTerminal("constant_Measure_byte", undefined), new $runtime.Choice(["or smaller", "or less"]), "the", new $runtime.NonTerminal("out_param_Any", ["type" as ("type" extends keyof ReturnType<typeof C.paramKeyFn> ? string : void), ((new Type.Measure('byte'))) as ReturnType<typeof C.paramKeyFn>["type"]])], ((x : Ast.Value, $0 : undefined, $1 : undefined, p : C.ParamSlot) : (C.FilterSlot)|null => C.makeFilter($loader, p, '<=', x)), C.filterKeyFn, {  });
   $grammar.addRule("with_filter", [new $runtime.NonTerminal("out_param_Any", ["type" as ("type" extends keyof ReturnType<typeof C.paramKeyFn> ? string : void), ((new Type.Measure('kg'))) as ReturnType<typeof C.paramKeyFn>["type"]]), new $runtime.Choice(["heavier", "larger"]), "than", new $runtime.NonTerminal("constant_Measure_kg", undefined)], ((p : C.ParamSlot, $0 : undefined, $1 : undefined, x : Ast.Value) : (C.FilterSlot)|null => C.makeFilter($loader, p, '>=', x)), C.filterKeyFn, {  });
   $grammar.addRule("with_filter", [new $runtime.NonTerminal("out_param_Any", ["type" as ("type" extends keyof ReturnType<typeof C.paramKeyFn> ? string : void), ((new Type.Measure('kg'))) as ReturnType<typeof C.paramKeyFn>["type"]]), new $runtime.Choice(["lighter", "smaller"]), "than", new $runtime.NonTerminal("constant_Measure_kg", undefined)], ((p : C.ParamSlot, $0 : undefined, $1 : undefined, x : Ast.Value) : (C.FilterSlot)|null => C.makeFilter($loader, p, '<=', x)), C.filterKeyFn, {  });
   $grammar.addRule("with_filter", [new $runtime.NonTerminal("constant_Measure_kg", undefined), "or more", "the", new $runtime.NonTerminal("out_param_Any", ["type" as ("type" extends keyof ReturnType<typeof C.paramKeyFn> ? string : void), ((new Type.Measure('kg'))) as ReturnType<typeof C.paramKeyFn>["type"]])], ((x : Ast.Value, $0 : undefined, $1 : undefined, p : C.ParamSlot) : (C.FilterSlot)|null => C.makeFilter($loader, p, '>=', x)), C.filterKeyFn, {  });
   $grammar.addRule("with_filter", [new $runtime.NonTerminal("constant_Measure_kg", undefined), "or less", "the", new $runtime.NonTerminal("out_param_Any", ["type" as ("type" extends keyof ReturnType<typeof C.paramKeyFn> ? string : void), ((new Type.Measure('kg'))) as ReturnType<typeof C.paramKeyFn>["type"]])], ((x : Ast.Value, $0 : undefined, $1 : undefined, p : C.ParamSlot) : (C.FilterSlot)|null => C.makeFilter($loader, p, '<=', x)), C.filterKeyFn, {  });
   if (!$options.flags.turking) {
       $grammar.addRule("with_filter", ["over", new $runtime.NonTerminal("constant_Measure_kg", undefined), new $runtime.NonTerminal("out_param_Any", ["type" as ("type" extends keyof ReturnType<typeof C.paramKeyFn> ? string : void), ((new Type.Measure('kg'))) as ReturnType<typeof C.paramKeyFn>["type"]])], (($0 : undefined, x : Ast.Value, p : C.ParamSlot) : (C.FilterSlot)|null => C.makeFilter($loader, p, '>=', x)), C.filterKeyFn, {  });
   }
   $grammar.addRule("with_filter", [new $runtime.NonTerminal("out_param_Any", ["type" as ("type" extends keyof ReturnType<typeof C.paramKeyFn> ? string : void), ((new Type.Measure('C'))) as ReturnType<typeof C.paramKeyFn>["type"]]), new $runtime.Choice(["hotter", "higher"]), "than", new $runtime.NonTerminal("constant_Measure_C", undefined)], ((p : C.ParamSlot, $0 : undefined, $1 : undefined, x : Ast.Value) : (C.FilterSlot)|null => C.makeFilter($loader, p, '>=', x)), C.filterKeyFn, {  });
   $grammar.addRule("with_filter", [new $runtime.NonTerminal("out_param_Any", ["type" as ("type" extends keyof ReturnType<typeof C.paramKeyFn> ? string : void), ((new Type.Measure('C'))) as ReturnType<typeof C.paramKeyFn>["type"]]), new $runtime.Choice(["cooler", "colder", "lower"]), "than", new $runtime.NonTerminal("constant_Measure_C", undefined)], ((p : C.ParamSlot, $0 : undefined, $1 : undefined, x : Ast.Value) : (C.FilterSlot)|null => C.makeFilter($loader, p, '<=', x)), C.filterKeyFn, {  });
   $grammar.addRule("with_filter", [new $runtime.NonTerminal("out_param_Any", ["type" as ("type" extends keyof ReturnType<typeof C.paramKeyFn> ? string : void), ((new Type.Measure('m'))) as ReturnType<typeof C.paramKeyFn>["type"]]), new $runtime.Choice(["farther", "more distant", "longer"]), "than", new $runtime.NonTerminal("constant_Measure_m", undefined)], ((p : C.ParamSlot, $0 : undefined, $1 : undefined, x : Ast.Value) : (C.FilterSlot)|null => C.makeFilter($loader, p, '>=', x)), C.filterKeyFn, {  });
   $grammar.addRule("with_filter", [new $runtime.NonTerminal("out_param_Any", ["type" as ("type" extends keyof ReturnType<typeof C.paramKeyFn> ? string : void), ((new Type.Measure('m'))) as ReturnType<typeof C.paramKeyFn>["type"]]), new $runtime.Choice(["nearer", "closer", "shorter"]), "than", new $runtime.NonTerminal("constant_Measure_m", undefined)], ((p : C.ParamSlot, $0 : undefined, $1 : undefined, x : Ast.Value) : (C.FilterSlot)|null => C.makeFilter($loader, p, '<=', x)), C.filterKeyFn, {  });
   $grammar.addRule("with_filter", [new $runtime.NonTerminal("out_param_Any", ["type" as ("type" extends keyof ReturnType<typeof C.paramKeyFn> ? string : void), ((new Type.Measure('m2'))) as ReturnType<typeof C.paramKeyFn>["type"]]), new $runtime.Choice(["larger", "bigger"]), "than", new $runtime.NonTerminal("constant_Measure_m2", undefined)], ((p : C.ParamSlot, $0 : undefined, $1 : undefined, x : Ast.Value) : (C.FilterSlot)|null => C.makeFilter($loader, p, '>=', x)), C.filterKeyFn, {  });
   $grammar.addRule("with_filter", [new $runtime.NonTerminal("out_param_Any", ["type" as ("type" extends keyof ReturnType<typeof C.paramKeyFn> ? string : void), ((new Type.Measure('m2'))) as ReturnType<typeof C.paramKeyFn>["type"]]), "smaller", "than", new $runtime.NonTerminal("constant_Measure_m2", undefined)], ((p : C.ParamSlot, $0 : undefined, $1 : undefined, x : Ast.Value) : (C.FilterSlot)|null => C.makeFilter($loader, p, '<=', x)), C.filterKeyFn, {  });
   $grammar.addRule("with_filter", [new $runtime.NonTerminal("out_param_Any", ["type" as ("type" extends keyof ReturnType<typeof C.paramKeyFn> ? string : void), ((new Type.Measure('m3'))) as ReturnType<typeof C.paramKeyFn>["type"]]), new $runtime.Choice(["larger", "bigger"]), "than", new $runtime.NonTerminal("constant_Measure_m3", undefined)], ((p : C.ParamSlot, $0 : undefined, $1 : undefined, x : Ast.Value) : (C.FilterSlot)|null => C.makeFilter($loader, p, '>=', x)), C.filterKeyFn, {  });
   $grammar.addRule("with_filter", [new $runtime.NonTerminal("out_param_Any", ["type" as ("type" extends keyof ReturnType<typeof C.paramKeyFn> ? string : void), ((new Type.Measure('m3'))) as ReturnType<typeof C.paramKeyFn>["type"]]), "smaller", "than", new $runtime.NonTerminal("constant_Measure_m3", undefined)], ((p : C.ParamSlot, $0 : undefined, $1 : undefined, x : Ast.Value) : (C.FilterSlot)|null => C.makeFilter($loader, p, '<=', x)), C.filterKeyFn, {  });
   $grammar.addRule("with_filter", [new $runtime.NonTerminal("out_param_Any", ["type" as ("type" extends keyof ReturnType<typeof C.paramKeyFn> ? string : void), ((new Type.Measure('mps'))) as ReturnType<typeof C.paramKeyFn>["type"]]), new $runtime.Choice(["faster", "quicker", "speedier"]), "than", new $runtime.NonTerminal("constant_Measure_mps", undefined)], ((p : C.ParamSlot, $0 : undefined, $1 : undefined, x : Ast.Value) : (C.FilterSlot)|null => C.makeFilter($loader, p, '>=', x)), C.filterKeyFn, {  });
   $grammar.addRule("with_filter", [new $runtime.NonTerminal("out_param_Any", ["type" as ("type" extends keyof ReturnType<typeof C.paramKeyFn> ? string : void), ((new Type.Measure('mps'))) as ReturnType<typeof C.paramKeyFn>["type"]]), new $runtime.Choice(["slower", "more slowly"]), "than", new $runtime.NonTerminal("constant_Measure_mps", undefined)], ((p : C.ParamSlot, $0 : undefined, $1 : undefined, x : Ast.Value) : (C.FilterSlot)|null => C.makeFilter($loader, p, '<=', x)), C.filterKeyFn, {  });
   $grammar.addRule("with_filter", [new $runtime.NonTerminal("out_param_Any", ["type" as ("type" extends keyof ReturnType<typeof C.paramKeyFn> ? string : void), ((Type.Date)) as ReturnType<typeof C.paramKeyFn>["type"]]), new $runtime.Choice(["after", "later than"]), new $runtime.NonTerminal("constant_Date", undefined)], ((p : C.ParamSlot, $0 : undefined, x : Ast.Value) : (C.FilterSlot)|null => C.makeFilter($loader, p, '>=', x)), C.filterKeyFn, {  });
   $grammar.addRule("with_filter", [new $runtime.NonTerminal("out_param_Any", ["type" as ("type" extends keyof ReturnType<typeof C.paramKeyFn> ? string : void), ((Type.Date)) as ReturnType<typeof C.paramKeyFn>["type"]]), new $runtime.Choice(["before", "earlier than", "sooner than"]), new $runtime.NonTerminal("constant_Date", undefined)], ((p : C.ParamSlot, $0 : undefined, x : Ast.Value) : (C.FilterSlot)|null => C.makeFilter($loader, p, '<=', x)), C.filterKeyFn, {  });
   $grammar.addRule("with_filter", [new $runtime.NonTerminal("out_param_Any", ["type" as ("type" extends keyof ReturnType<typeof C.paramKeyFn> ? string : void), ((Type.Time)) as ReturnType<typeof C.paramKeyFn>["type"]]), new $runtime.Choice(["after", "later than"]), new $runtime.NonTerminal("constant_Time", undefined)], ((p : C.ParamSlot, $0 : undefined, x : Ast.Value) : (C.FilterSlot)|null => C.makeFilter($loader, p, '>=', x)), C.filterKeyFn, {  });
   $grammar.addRule("with_filter", [new $runtime.NonTerminal("out_param_Any", ["type" as ("type" extends keyof ReturnType<typeof C.paramKeyFn> ? string : void), ((Type.Time)) as ReturnType<typeof C.paramKeyFn>["type"]]), new $runtime.Choice(["before", "earlier than", "sooner than"]), new $runtime.NonTerminal("constant_Time", undefined)], ((p : C.ParamSlot, $0 : undefined, x : Ast.Value) : (C.FilterSlot)|null => C.makeFilter($loader, p, '<=', x)), C.filterKeyFn, {  });
   $grammar.addRule("with_filter", [new $runtime.Choice(["greater", "higher", "larger", "more"]), new $runtime.NonTerminal("out_param_Any", ["type" as ("type" extends keyof ReturnType<typeof C.paramKeyFn> ? string : void), ((Type.Number)) as ReturnType<typeof C.paramKeyFn>["type"]]), "than", new $runtime.NonTerminal("constant_Number", undefined)], (($0 : undefined, p : C.ParamSlot, $1 : undefined, x : Ast.NumberValue) : (C.FilterSlot)|null => C.makeFilter($loader, p, '>=', x)), C.filterKeyFn, {  });
   $grammar.addRule("with_filter", [new $runtime.Choice(["smaller", "lower", "less"]), new $runtime.NonTerminal("out_param_Any", ["type" as ("type" extends keyof ReturnType<typeof C.paramKeyFn> ? string : void), ((Type.Number)) as ReturnType<typeof C.paramKeyFn>["type"]]), "than", new $runtime.NonTerminal("constant_Number", undefined)], (($0 : undefined, p : C.ParamSlot, $1 : undefined, x : Ast.NumberValue) : (C.FilterSlot)|null => C.makeFilter($loader, p, '<=', x)), C.filterKeyFn, {  });
   $grammar.addRule("with_filter", [new $runtime.Choice(["more expensive", "more costly", "more"]), new $runtime.NonTerminal("out_param_Any", ["type" as ("type" extends keyof ReturnType<typeof C.paramKeyFn> ? string : void), ((Type.Currency)) as ReturnType<typeof C.paramKeyFn>["type"]]), "than", new $runtime.NonTerminal("constant_Currency", undefined)], (($0 : undefined, p : C.ParamSlot, $1 : undefined, x : Ast.Value) : (C.FilterSlot)|null => C.makeFilter($loader, p, '>=', x)), C.filterKeyFn, {  });
   $grammar.addRule("with_filter", [new $runtime.Choice(["less expensive", "cheaper", "less"]), new $runtime.NonTerminal("out_param_Any", ["type" as ("type" extends keyof ReturnType<typeof C.paramKeyFn> ? string : void), ((Type.Currency)) as ReturnType<typeof C.paramKeyFn>["type"]]), "than", new $runtime.NonTerminal("constant_Currency", undefined)], (($0 : undefined, p : C.ParamSlot, $1 : undefined, x : Ast.Value) : (C.FilterSlot)|null => C.makeFilter($loader, p, '<=', x)), C.filterKeyFn, {  });
   $grammar.addRule("with_filter", ["longer", new $runtime.NonTerminal("out_param_Any", ["type" as ("type" extends keyof ReturnType<typeof C.paramKeyFn> ? string : void), ((new Type.Measure('ms'))) as ReturnType<typeof C.paramKeyFn>["type"]]), "than", new $runtime.NonTerminal("constant_Measure_ms", undefined)], (($0 : undefined, p : C.ParamSlot, $1 : undefined, x : Ast.Value) : (C.FilterSlot)|null => C.makeFilter($loader, p, '>=', x)), C.filterKeyFn, {  });
   $grammar.addRule("with_filter", ["shorter", new $runtime.NonTerminal("out_param_Any", ["type" as ("type" extends keyof ReturnType<typeof C.paramKeyFn> ? string : void), ((new Type.Measure('ms'))) as ReturnType<typeof C.paramKeyFn>["type"]]), "than", new $runtime.NonTerminal("constant_Measure_ms", undefined)], (($0 : undefined, p : C.ParamSlot, $1 : undefined, x : Ast.Value) : (C.FilterSlot)|null => C.makeFilter($loader, p, '<=', x)), C.filterKeyFn, {  });
   $grammar.addRule("with_filter", [new $runtime.Choice(["larger", "more"]), new $runtime.NonTerminal("out_param_Any", ["type" as ("type" extends keyof ReturnType<typeof C.paramKeyFn> ? string : void), ((new Type.Measure('byte'))) as ReturnType<typeof C.paramKeyFn>["type"]]), "than", new $runtime.NonTerminal("constant_Measure_byte", undefined)], (($0 : undefined, p : C.ParamSlot, $1 : undefined, x : Ast.Value) : (C.FilterSlot)|null => C.makeFilter($loader, p, '>=', x)), C.filterKeyFn, {  });
   $grammar.addRule("with_filter", [new $runtime.Choice(["smaller", "less"]), new $runtime.NonTerminal("out_param_Any", ["type" as ("type" extends keyof ReturnType<typeof C.paramKeyFn> ? string : void), ((new Type.Measure('byte'))) as ReturnType<typeof C.paramKeyFn>["type"]]), "than", new $runtime.NonTerminal("constant_Measure_byte", undefined)], (($0 : undefined, p : C.ParamSlot, $1 : undefined, x : Ast.Value) : (C.FilterSlot)|null => C.makeFilter($loader, p, '<=', x)), C.filterKeyFn, {  });
   $grammar.addRule("with_filter", [new $runtime.Choice(["heavier", "larger"]), new $runtime.NonTerminal("out_param_Any", ["type" as ("type" extends keyof ReturnType<typeof C.paramKeyFn> ? string : void), ((new Type.Measure('kg'))) as ReturnType<typeof C.paramKeyFn>["type"]]), "than", new $runtime.NonTerminal("constant_Measure_kg", undefined)], (($0 : undefined, p : C.ParamSlot, $1 : undefined, x : Ast.Value) : (C.FilterSlot)|null => C.makeFilter($loader, p, '>=', x)), C.filterKeyFn, {  });
   $grammar.addRule("with_filter", [new $runtime.Choice(["lighter", "smaller"]), new $runtime.NonTerminal("out_param_Any", ["type" as ("type" extends keyof ReturnType<typeof C.paramKeyFn> ? string : void), ((new Type.Measure('kg'))) as ReturnType<typeof C.paramKeyFn>["type"]]), "than", new $runtime.NonTerminal("constant_Measure_kg", undefined)], (($0 : undefined, p : C.ParamSlot, $1 : undefined, x : Ast.Value) : (C.FilterSlot)|null => C.makeFilter($loader, p, '<=', x)), C.filterKeyFn, {  });
   $grammar.addRule("with_filter", [new $runtime.Choice(["hotter", "higher"]), new $runtime.NonTerminal("out_param_Any", ["type" as ("type" extends keyof ReturnType<typeof C.paramKeyFn> ? string : void), ((new Type.Measure('C'))) as ReturnType<typeof C.paramKeyFn>["type"]]), "than", new $runtime.NonTerminal("constant_Measure_C", undefined)], (($0 : undefined, p : C.ParamSlot, $1 : undefined, x : Ast.Value) : (C.FilterSlot)|null => C.makeFilter($loader, p, '>=', x)), C.filterKeyFn, {  });
   $grammar.addRule("with_filter", [new $runtime.Choice(["cooler", "colder", "lower"]), new $runtime.NonTerminal("out_param_Any", ["type" as ("type" extends keyof ReturnType<typeof C.paramKeyFn> ? string : void), ((new Type.Measure('C'))) as ReturnType<typeof C.paramKeyFn>["type"]]), "than", new $runtime.NonTerminal("constant_Measure_C", undefined)], (($0 : undefined, p : C.ParamSlot, $1 : undefined, x : Ast.Value) : (C.FilterSlot)|null => C.makeFilter($loader, p, '<=', x)), C.filterKeyFn, {  });
   $grammar.addRule("with_filter", [new $runtime.Choice(["farther", "more distant", "longer"]), new $runtime.NonTerminal("out_param_Any", ["type" as ("type" extends keyof ReturnType<typeof C.paramKeyFn> ? string : void), ((new Type.Measure('m'))) as ReturnType<typeof C.paramKeyFn>["type"]]), "than", new $runtime.NonTerminal("constant_Measure_m", undefined)], (($0 : undefined, p : C.ParamSlot, $1 : undefined, x : Ast.Value) : (C.FilterSlot)|null => C.makeFilter($loader, p, '>=', x)), C.filterKeyFn, {  });
   $grammar.addRule("with_filter", [new $runtime.Choice(["nearer", "closer", "shorter"]), new $runtime.NonTerminal("out_param_Any", ["type" as ("type" extends keyof ReturnType<typeof C.paramKeyFn> ? string : void), ((new Type.Measure('m'))) as ReturnType<typeof C.paramKeyFn>["type"]]), "than", new $runtime.NonTerminal("constant_Measure_m", undefined)], (($0 : undefined, p : C.ParamSlot, $1 : undefined, x : Ast.Value) : (C.FilterSlot)|null => C.makeFilter($loader, p, '<=', x)), C.filterKeyFn, {  });
   $grammar.addRule("with_filter", [new $runtime.Choice(["bigger", "larger"]), new $runtime.NonTerminal("out_param_Any", ["type" as ("type" extends keyof ReturnType<typeof C.paramKeyFn> ? string : void), ((new Type.Measure('m2'))) as ReturnType<typeof C.paramKeyFn>["type"]]), "than", new $runtime.NonTerminal("constant_Measure_m2", undefined)], (($0 : undefined, p : C.ParamSlot, $1 : undefined, x : Ast.Value) : (C.FilterSlot)|null => C.makeFilter($loader, p, '>=', x)), C.filterKeyFn, {  });
   $grammar.addRule("with_filter", ["smaller", new $runtime.NonTerminal("out_param_Any", ["type" as ("type" extends keyof ReturnType<typeof C.paramKeyFn> ? string : void), ((new Type.Measure('m2'))) as ReturnType<typeof C.paramKeyFn>["type"]]), "than", new $runtime.NonTerminal("constant_Measure_m2", undefined)], (($0 : undefined, p : C.ParamSlot, $1 : undefined, x : Ast.Value) : (C.FilterSlot)|null => C.makeFilter($loader, p, '<=', x)), C.filterKeyFn, {  });
   $grammar.addRule("with_filter", [new $runtime.Choice(["bigger", "larger"]), new $runtime.NonTerminal("out_param_Any", ["type" as ("type" extends keyof ReturnType<typeof C.paramKeyFn> ? string : void), ((new Type.Measure('m3'))) as ReturnType<typeof C.paramKeyFn>["type"]]), "than", new $runtime.NonTerminal("constant_Measure_m3", undefined)], (($0 : undefined, p : C.ParamSlot, $1 : undefined, x : Ast.Value) : (C.FilterSlot)|null => C.makeFilter($loader, p, '>=', x)), C.filterKeyFn, {  });
   $grammar.addRule("with_filter", ["smaller", new $runtime.NonTerminal("out_param_Any", ["type" as ("type" extends keyof ReturnType<typeof C.paramKeyFn> ? string : void), ((new Type.Measure('m3'))) as ReturnType<typeof C.paramKeyFn>["type"]]), "than", new $runtime.NonTerminal("constant_Measure_m3", undefined)], (($0 : undefined, p : C.ParamSlot, $1 : undefined, x : Ast.Value) : (C.FilterSlot)|null => C.makeFilter($loader, p, '<=', x)), C.filterKeyFn, {  });
   $grammar.addRule("with_filter", [new $runtime.Choice(["faster", "quicker", "speedier"]), new $runtime.NonTerminal("out_param_Any", ["type" as ("type" extends keyof ReturnType<typeof C.paramKeyFn> ? string : void), ((new Type.Measure('mps'))) as ReturnType<typeof C.paramKeyFn>["type"]]), "than", new $runtime.NonTerminal("constant_Measure_mps", undefined)], (($0 : undefined, p : C.ParamSlot, $1 : undefined, x : Ast.Value) : (C.FilterSlot)|null => C.makeFilter($loader, p, '>=', x)), C.filterKeyFn, {  });
   $grammar.addRule("with_filter", [new $runtime.Choice(["slower", "more slowly"]), new $runtime.NonTerminal("out_param_Any", ["type" as ("type" extends keyof ReturnType<typeof C.paramKeyFn> ? string : void), ((new Type.Measure('mps'))) as ReturnType<typeof C.paramKeyFn>["type"]]), "than", new $runtime.NonTerminal("constant_Measure_mps", undefined)], (($0 : undefined, p : C.ParamSlot, $1 : undefined, x : Ast.Value) : (C.FilterSlot)|null => C.makeFilter($loader, p, '<=', x)), C.filterKeyFn, {  });
   $grammar.addRule("with_filter", ["later", new $runtime.NonTerminal("out_param_Any", ["type" as ("type" extends keyof ReturnType<typeof C.paramKeyFn> ? string : void), ((Type.Date)) as ReturnType<typeof C.paramKeyFn>["type"]]), "than", new $runtime.NonTerminal("constant_Date", undefined)], (($0 : undefined, p : C.ParamSlot, $1 : undefined, x : Ast.Value) : (C.FilterSlot)|null => C.makeFilter($loader, p, '>=', x)), C.filterKeyFn, {  });
   $grammar.addRule("with_filter", [new $runtime.Choice(["earlier", "sooner"]), new $runtime.NonTerminal("out_param_Any", ["type" as ("type" extends keyof ReturnType<typeof C.paramKeyFn> ? string : void), ((Type.Date)) as ReturnType<typeof C.paramKeyFn>["type"]]), "than", new $runtime.NonTerminal("constant_Date", undefined)], (($0 : undefined, p : C.ParamSlot, $1 : undefined, x : Ast.Value) : (C.FilterSlot)|null => C.makeFilter($loader, p, '<=', x)), C.filterKeyFn, {  });
   $grammar.addRule("with_filter", ["later", new $runtime.NonTerminal("out_param_Any", ["type" as ("type" extends keyof ReturnType<typeof C.paramKeyFn> ? string : void), ((Type.Time)) as ReturnType<typeof C.paramKeyFn>["type"]]), "than", new $runtime.NonTerminal("constant_Time", undefined)], (($0 : undefined, p : C.ParamSlot, $1 : undefined, x : Ast.Value) : (C.FilterSlot)|null => C.makeFilter($loader, p, '>=', x)), C.filterKeyFn, {  });
   $grammar.addRule("with_filter", [new $runtime.Choice(["earlier", "sooner"]), new $runtime.NonTerminal("out_param_Any", ["type" as ("type" extends keyof ReturnType<typeof C.paramKeyFn> ? string : void), ((Type.Time)) as ReturnType<typeof C.paramKeyFn>["type"]]), "than", new $runtime.NonTerminal("constant_Time", undefined)], (($0 : undefined, p : C.ParamSlot, $1 : undefined, x : Ast.Value) : (C.FilterSlot)|null => C.makeFilter($loader, p, '<=', x)), C.filterKeyFn, {  });
   if (!$options.flags.turking) {
       if ($options.flags.range_filters) {
           $grammar.addRule("with_filter", [new $runtime.NonTerminal("range_with_filter", undefined)], ((f : C.FilterSlot) : (C.FilterSlot)|null => f), C.filterKeyFn, {  });
       }
       $grammar.addRule("with_filter", ["no", new $runtime.NonTerminal("out_param_Any", ["type" as ("type" extends keyof ReturnType<typeof C.paramKeyFn> ? string : void), ((Type.Number)) as ReturnType<typeof C.paramKeyFn>["type"]])], (($0 : undefined, param : C.ParamSlot) : (C.FilterSlot)|null => C.makeFilter($loader, param, '==', new Ast.Value.Number(0))), C.filterKeyFn, {  });
       $grammar.addRule("with_filter", ["zero", new $runtime.NonTerminal("out_param_Any", ["type" as ("type" extends keyof ReturnType<typeof C.paramKeyFn> ? string : void), ((Type.Number)) as ReturnType<typeof C.paramKeyFn>["type"]])], (($0 : undefined, param : C.ParamSlot) : (C.FilterSlot)|null => C.makeFilter($loader, param, '==', new Ast.Value.Number(0))), C.filterKeyFn, {  });
   }
   $grammar.addRule("with_filter", [new $runtime.NonTerminal("out_param_Array__Any", undefined), "containing", new $runtime.NonTerminal("constant_Any", undefined)], ((p : C.ParamSlot, $0 : undefined, x : Ast.Value) : (C.FilterSlot)|null => C.makeFilter($loader, p, 'contains', x)), C.filterKeyFn, {  });
   $grammar.addRule("with_filter", [new $runtime.NonTerminal("out_param_Array__Any", undefined), "containing no", new $runtime.NonTerminal("constant_Any", undefined)], ((p : C.ParamSlot, $0 : undefined, x : Ast.Value) : (C.FilterSlot)|null => C.makeFilter($loader, p, 'contains', x, true)), C.filterKeyFn, {  });
   $grammar.addRule("with_filter", [new $runtime.NonTerminal("constant_Any", undefined), "in", "the", new $runtime.NonTerminal("out_param_Array__Any", undefined)], ((x : Ast.Value, $0 : undefined, $1 : undefined, p : C.ParamSlot) : (C.FilterSlot)|null => C.makeFilter($loader, p, 'contains', x)), C.filterKeyFn, {  });
   if (!$options.flags.turking) {
       $grammar.addRule("with_filter", [new $runtime.NonTerminal("out_param_Array__Any", undefined), "containing", new $runtime.NonTerminal("both_prefix", undefined), new $runtime.NonTerminal("constant_pairs", ["type" as ("type" extends keyof ReturnType<typeof C.valueArrayKeyFn> ? string : void), 0, "elem" as ("elem" extends keyof ReturnType<typeof C.paramKeyFn> ? string : void)])], ((p : C.ParamSlot, $0 : undefined, $1 : string, pair : [Ast.Value, Ast.Value]) : (C.FilterSlot)|null => C.makeAndFilter($loader, p, 'contains', pair)), C.filterKeyFn, {  });
   }
   if (!$options.flags.turking) {
       $grammar.addRule("with_filter", [new $runtime.NonTerminal("both_prefix", undefined), new $runtime.NonTerminal("constant_pairs", undefined), "in", "the", new $runtime.NonTerminal("out_param_Array__Any", ["elem" as ("elem" extends keyof ReturnType<typeof C.paramKeyFn> ? string : void), 1, "type" as ("type" extends keyof ReturnType<typeof C.valueArrayKeyFn> ? string : void)])], (($0 : string, pair : [Ast.Value, Ast.Value], $1 : undefined, $2 : undefined, p : C.ParamSlot) : (C.FilterSlot)|null => C.makeAndFilter($loader, p, 'contains', pair)), C.filterKeyFn, {  });
   }
   $grammar.addRule("with_filter", [new $runtime.NonTerminal("out_param_Array__Any", undefined), "containing either", new $runtime.NonTerminal("constant_or_pairs", ["type" as ("type" extends keyof ReturnType<typeof C.valueArrayKeyFn> ? string : void), 0, "elem" as ("elem" extends keyof ReturnType<typeof C.paramKeyFn> ? string : void)])], ((p : C.ParamSlot, $0 : undefined, pair : [Ast.Value, Ast.Value]) : (C.FilterSlot)|null => C.makeOrFilter($loader, p, 'contains', pair)), C.filterKeyFn, {  });
   $grammar.addRule("with_filter", ["either", new $runtime.NonTerminal("constant_or_pairs", undefined), "in", "the", new $runtime.NonTerminal("out_param_Array__Any", ["elem" as ("elem" extends keyof ReturnType<typeof C.paramKeyFn> ? string : void), 1, "type" as ("type" extends keyof ReturnType<typeof C.valueArrayKeyFn> ? string : void)])], (($0 : undefined, pair : [Ast.Value, Ast.Value], $1 : undefined, $2 : undefined, p : C.ParamSlot) : (C.FilterSlot)|null => C.makeOrFilter($loader, p, 'contains', pair)), C.filterKeyFn, {  });
   if (!$options.flags.turking) {
       $grammar.addRule("with_filter", [new $runtime.NonTerminal("out_param_Any", ["type" as ("type" extends keyof ReturnType<typeof C.paramKeyFn> ? string : void), ((Type.Any)) as ReturnType<typeof C.paramKeyFn>["type"]]), new $runtime.Choice(["containing", "including"]), new $runtime.NonTerminal("constant_String", undefined), new $runtime.Choice(["but not", "without", "but no"]), new $runtime.NonTerminal("constant_String", undefined)], ((p : C.ParamSlot, $0 : undefined, x : Ast.Value, $1 : undefined, y : Ast.Value) : (C.FilterSlot)|null => C.makeButFilter($loader, p, 'contains', [x, y])), C.filterKeyFn, {  });
   }
   if (!$options.flags.turking) {
       $grammar.addRule("with_filter", [new $runtime.Choice(["no", "zero"]), new $runtime.NonTerminal("out_param_Array__Any", undefined)], (($0 : undefined, p : C.ParamSlot) : (C.FilterSlot)|null => C.makeAggregateFilter(p, 'count', '*', '==', new Ast.Value.Number(0))), C.filterKeyFn, {  });
   }
   $grammar.addRule("with_filter", [new $runtime.NonTerminal("constant_Number", undefined), new $runtime.NonTerminal("out_param_Array__Any", undefined)], ((x : Ast.NumberValue, p : C.ParamSlot) : (C.FilterSlot)|null => C.makeAggregateFilter(p, 'count', '*', '==', x)), C.filterKeyFn, {  });
   if (!$options.flags.turking) {
       $grammar.addRule("with_filter", [new $runtime.Choice(["no", "zero"]), new $runtime.NonTerminal("out_param_ArrayCount", undefined)], (($0 : undefined, p : C.ParamSlot) : (C.FilterSlot)|null => C.makeFilter($loader, p, '==', new Ast.Value.Number(0))), C.filterKeyFn, {  });
   }
   $grammar.addRule("with_filter", [new $runtime.NonTerminal("constant_Number", undefined), new $runtime.NonTerminal("out_param_ArrayCount", undefined)], ((x : Ast.NumberValue, p : C.ParamSlot) : (C.FilterSlot)|null => C.makeFilter($loader, p, '==', x)), C.filterKeyFn, {  });
   if (!$options.flags.turking) {
       $grammar.addRule("with_filter", [new $runtime.Choice(["no", "zero"]), new $runtime.NonTerminal("out_param_Array__Compound", undefined), new $runtime.Choice(["with", "having"]), new $runtime.NonTerminal("out_param_Any", ["is_numeric" as ("is_numeric" extends keyof ReturnType<typeof C.paramKeyFn> ? string : void), (true) as ReturnType<typeof C.paramKeyFn>["is_numeric"]]), new $runtime.Choice(["lower than", "below"]), new $runtime.NonTerminal("constant_Numeric", ["type" as ("type" extends keyof ReturnType<typeof C.valueKeyFn> ? string : void), 3, "type" as ("type" extends keyof ReturnType<typeof C.paramKeyFn> ? string : void)])], (($0 : undefined, p : C.ParamSlot, $1 : undefined, f : C.ParamSlot, $2 : undefined, x : Ast.Value) : (C.FilterSlot)|null => {
        const filter =  C.makeFilter($loader, f, '<=', x);
        return C.makeAggregateFilterWithFilter(p, filter, 'count', '*', '==', new Ast.Value.Number(0));
    }), C.filterKeyFn, {  });
   }
   if (!$options.flags.turking) {
       $grammar.addRule("with_filter", [new $runtime.Choice(["no", "zero"]), new $runtime.NonTerminal("out_param_Array__Compound", undefined), new $runtime.Choice(["with", "having"]), new $runtime.NonTerminal("out_param_Any", ["is_numeric" as ("is_numeric" extends keyof ReturnType<typeof C.paramKeyFn> ? string : void), (true) as ReturnType<typeof C.paramKeyFn>["is_numeric"]]), new $runtime.Choice(["greater than", "above"]), new $runtime.NonTerminal("constant_Numeric", ["type" as ("type" extends keyof ReturnType<typeof C.valueKeyFn> ? string : void), 3, "type" as ("type" extends keyof ReturnType<typeof C.paramKeyFn> ? string : void)])], (($0 : undefined, p : C.ParamSlot, $1 : undefined, f : C.ParamSlot, $2 : undefined, x : Ast.Value) : (C.FilterSlot)|null => {
        const filter = C.makeFilter($loader, f, '>=', x);
        return C.makeAggregateFilterWithFilter(p, filter, 'count', '*', '==', new Ast.Value.Number(0));
    }), C.filterKeyFn, {  });
   }
   $grammar.addRule("with_filter", [new $runtime.NonTerminal("out_param_Array__Compound", undefined), new $runtime.Choice(["with", "having"]), new $runtime.NonTerminal("out_param_Any", ["is_numeric" as ("is_numeric" extends keyof ReturnType<typeof C.paramKeyFn> ? string : void), (true) as ReturnType<typeof C.paramKeyFn>["is_numeric"]]), new $runtime.Choice(["greater than", "above"]), new $runtime.NonTerminal("constant_Numeric", ["type" as ("type" extends keyof ReturnType<typeof C.valueKeyFn> ? string : void), 2, "type" as ("type" extends keyof ReturnType<typeof C.paramKeyFn> ? string : void)])], ((p : C.ParamSlot, $0 : undefined, f : C.ParamSlot, $1 : undefined, x : Ast.Value) : (C.FilterSlot)|null => {
        const filter = C.makeFilter($loader, f, '<=', x);
        return C.makeAggregateFilterWithFilter(p, filter, 'count', '*', '>=', new Ast.Value.Number(1));

    }), C.filterKeyFn, {  });
   $grammar.addRule("with_filter", [new $runtime.NonTerminal("out_param_Array__Compound", undefined), new $runtime.Choice(["with", "having"]), new $runtime.NonTerminal("out_param_Any", ["is_numeric" as ("is_numeric" extends keyof ReturnType<typeof C.paramKeyFn> ? string : void), (true) as ReturnType<typeof C.paramKeyFn>["is_numeric"]]), new $runtime.Choice(["lower than", "below"]), new $runtime.NonTerminal("constant_Numeric", ["type" as ("type" extends keyof ReturnType<typeof C.valueKeyFn> ? string : void), 2, "type" as ("type" extends keyof ReturnType<typeof C.paramKeyFn> ? string : void)])], ((p : C.ParamSlot, $0 : undefined, f : C.ParamSlot, $1 : undefined, x : Ast.Value) : (C.FilterSlot)|null => {
        const filter = C.makeFilter($loader, f, '>=', x);
        return C.makeAggregateFilterWithFilter(p, filter, 'count', '*', '>=', new Ast.Value.Number(1));
    }), C.filterKeyFn, {  });
   $grammar.addRule("with_filter", [new $runtime.NonTerminal("out_param_Array__Compound", undefined), new $runtime.Choice(["with", "having"]), new $runtime.NonTerminal("with_filter", undefined)], ((p : C.ParamSlot, $0 : undefined, f : C.FilterSlot) : (C.FilterSlot)|null => {
        return C.makeAggregateFilterWithFilter(p, f, 'count', '*', '>=', new Ast.Value.Number(1));
    }), C.filterKeyFn, {  });
   $grammar.addRule("with_filter", [new $runtime.NonTerminal("constant_Number", undefined), new $runtime.NonTerminal("out_param_Array__Compound", undefined), new $runtime.Choice(["with", "having"]), new $runtime.NonTerminal("out_param_Any", ["is_numeric" as ("is_numeric" extends keyof ReturnType<typeof C.paramKeyFn> ? string : void), (true) as ReturnType<typeof C.paramKeyFn>["is_numeric"]]), new $runtime.Choice(["greater than", "above"]), new $runtime.NonTerminal("constant_Numeric", ["type" as ("type" extends keyof ReturnType<typeof C.valueKeyFn> ? string : void), 3, "type" as ("type" extends keyof ReturnType<typeof C.paramKeyFn> ? string : void)])], ((v : Ast.NumberValue, p : C.ParamSlot, $0 : undefined, f : C.ParamSlot, $1 : undefined, x : Ast.Value) : (C.FilterSlot)|null => {
        const filter = C.makeFilter($loader, f, '<=', x);
        return C.makeAggregateFilterWithFilter(p, filter, 'count', '*', '==', v);
    }), C.filterKeyFn, {  });
   $grammar.addRule("with_filter", [new $runtime.Choice(["more than", "at least", "no fewer than"]), new $runtime.NonTerminal("constant_Number", undefined), new $runtime.NonTerminal("out_param_Array__Any", undefined)], (($0 : undefined, x : Ast.NumberValue, p : C.ParamSlot) : (C.FilterSlot)|null => C.makeAggregateFilter(p, 'count', '*', '>=', x)), C.filterKeyFn, {  });
   $grammar.addRule("with_filter", [new $runtime.Choice(["fewer than", "at most", "no more than"]), new $runtime.NonTerminal("constant_Number", undefined), new $runtime.NonTerminal("out_param_Array__Any", undefined)], (($0 : undefined, x : Ast.NumberValue, p : C.ParamSlot) : (C.FilterSlot)|null => C.makeAggregateFilter(p, 'count', '*', '<=', x)), C.filterKeyFn, {  });
   $grammar.addRule("with_filter", [new $runtime.Choice(["more than", "at least", "no fewer than"]), new $runtime.NonTerminal("constant_Number", undefined), new $runtime.NonTerminal("out_param_ArrayCount", undefined)], (($0 : undefined, x : Ast.NumberValue, p : C.ParamSlot) : (C.FilterSlot)|null => C.makeFilter($loader, p, '>=', x)), C.filterKeyFn, {  });
   $grammar.addRule("with_filter", [new $runtime.Choice(["fewer than", "at most", "no more than"]), new $runtime.NonTerminal("constant_Number", undefined), new $runtime.NonTerminal("out_param_ArrayCount", undefined)], (($0 : undefined, x : Ast.NumberValue, p : C.ParamSlot) : (C.FilterSlot)|null => C.makeFilter($loader, p, '<=', x)), C.filterKeyFn, {  });
   $grammar.addRule("with_filter", ["no more than", new $runtime.NonTerminal("constant_Number", undefined), new $runtime.NonTerminal("out_param_Array__Compound", undefined), new $runtime.Choice(["with", "having"]), new $runtime.NonTerminal("out_param_Any", ["is_numeric" as ("is_numeric" extends keyof ReturnType<typeof C.paramKeyFn> ? string : void), (true) as ReturnType<typeof C.paramKeyFn>["is_numeric"]]), new $runtime.Choice(["lower than", "below"]), new $runtime.NonTerminal("constant_Numeric", ["type" as ("type" extends keyof ReturnType<typeof C.valueKeyFn> ? string : void), 4, "type" as ("type" extends keyof ReturnType<typeof C.paramKeyFn> ? string : void)])], (($0 : undefined, v : Ast.NumberValue, p : C.ParamSlot, $1 : undefined, f : C.ParamSlot, $2 : undefined, x : Ast.Value) : (C.FilterSlot)|null => {
        const filter = C.makeFilter($loader, f, '<=', x);
        return C.makeAggregateFilterWithFilter(p, filter, 'count', '*', '<=', v);
    }), C.filterKeyFn, {  });
   $grammar.addRule("with_filter", ["at most", new $runtime.NonTerminal("constant_Number", undefined), new $runtime.NonTerminal("out_param_Array__Compound", undefined), new $runtime.Choice(["with", "having"]), new $runtime.NonTerminal("out_param_Any", ["is_numeric" as ("is_numeric" extends keyof ReturnType<typeof C.paramKeyFn> ? string : void), (true) as ReturnType<typeof C.paramKeyFn>["is_numeric"]]), new $runtime.Choice(["lower than", "below"]), new $runtime.NonTerminal("constant_Numeric", ["type" as ("type" extends keyof ReturnType<typeof C.valueKeyFn> ? string : void), 4, "type" as ("type" extends keyof ReturnType<typeof C.paramKeyFn> ? string : void)])], (($0 : undefined, v : Ast.NumberValue, p : C.ParamSlot, $1 : undefined, f : C.ParamSlot, $2 : undefined, x : Ast.Value) : (C.FilterSlot)|null => {
        const filter = C.makeFilter($loader, f, '<=', x);
        return C.makeAggregateFilterWithFilter(p, filter, 'count', '*', '<=', v);
    }), C.filterKeyFn, {  });
   $grammar.addRule("with_filter", ["fewer than", new $runtime.NonTerminal("constant_Number", undefined), new $runtime.NonTerminal("out_param_Array__Compound", undefined), new $runtime.Choice(["with", "having"]), new $runtime.NonTerminal("out_param_Any", ["is_numeric" as ("is_numeric" extends keyof ReturnType<typeof C.paramKeyFn> ? string : void), (true) as ReturnType<typeof C.paramKeyFn>["is_numeric"]]), new $runtime.Choice(["lower than", "below"]), new $runtime.NonTerminal("constant_Numeric", ["type" as ("type" extends keyof ReturnType<typeof C.valueKeyFn> ? string : void), 4, "type" as ("type" extends keyof ReturnType<typeof C.paramKeyFn> ? string : void)])], (($0 : undefined, v : Ast.NumberValue, p : C.ParamSlot, $1 : undefined, f : C.ParamSlot, $2 : undefined, x : Ast.Value) : (C.FilterSlot)|null => {
        const filter = C.makeFilter($loader, f, '<=', x);
        return C.makeAggregateFilterWithFilter(p, filter, 'count', '*', '<=', v);
    }), C.filterKeyFn, {  });
   $grammar.addRule("with_filter", ["more than", new $runtime.NonTerminal("constant_Number", undefined), new $runtime.NonTerminal("out_param_Array__Compound", undefined), new $runtime.Choice(["with", "having"]), new $runtime.NonTerminal("out_param_Any", ["is_numeric" as ("is_numeric" extends keyof ReturnType<typeof C.paramKeyFn> ? string : void), (true) as ReturnType<typeof C.paramKeyFn>["is_numeric"]]), new $runtime.Choice(["greater than", "above"]), new $runtime.NonTerminal("constant_Numeric", ["type" as ("type" extends keyof ReturnType<typeof C.valueKeyFn> ? string : void), 4, "type" as ("type" extends keyof ReturnType<typeof C.paramKeyFn> ? string : void)])], (($0 : undefined, v : Ast.NumberValue, p : C.ParamSlot, $1 : undefined, f : C.ParamSlot, $2 : undefined, x : Ast.Value) : (C.FilterSlot)|null => {
        const filter = C.makeFilter($loader, f, '>=', x);
        return C.makeAggregateFilterWithFilter(p, filter, 'count', '*', '>=', v);
    }), C.filterKeyFn, {  });
   $grammar.addRule("with_filter", ["at least", new $runtime.NonTerminal("constant_Number", undefined), new $runtime.NonTerminal("out_param_Array__Compound", undefined), new $runtime.Choice(["with", "having"]), new $runtime.NonTerminal("out_param_Any", ["is_numeric" as ("is_numeric" extends keyof ReturnType<typeof C.paramKeyFn> ? string : void), (true) as ReturnType<typeof C.paramKeyFn>["is_numeric"]]), new $runtime.Choice(["greater than", "above"]), new $runtime.NonTerminal("constant_Numeric", ["type" as ("type" extends keyof ReturnType<typeof C.valueKeyFn> ? string : void), 4, "type" as ("type" extends keyof ReturnType<typeof C.paramKeyFn> ? string : void)])], (($0 : undefined, v : Ast.NumberValue, p : C.ParamSlot, $1 : undefined, f : C.ParamSlot, $2 : undefined, x : Ast.Value) : (C.FilterSlot)|null => {
        const filter = C.makeFilter($loader, f, '>=', x);
        return C.makeAggregateFilterWithFilter(p, filter, 'count', '*', '>=', v);
    }), C.filterKeyFn, {  });
   $grammar.addRule("with_filter", [new $runtime.Choice(["total", "sum"]), new $runtime.NonTerminal("out_param_Array__Any", ["is_numeric_elem" as ("is_numeric_elem" extends keyof ReturnType<typeof C.paramKeyFn> ? string : void), (true) as ReturnType<typeof C.paramKeyFn>["is_numeric_elem"]]), new $runtime.Choice(["of", "being"]), new $runtime.NonTerminal("constant_Numeric", ["type" as ("type" extends keyof ReturnType<typeof C.valueKeyFn> ? string : void), 1, "elem" as ("elem" extends keyof ReturnType<typeof C.paramKeyFn> ? string : void)])], (($0 : undefined, p : C.ParamSlot, $1 : undefined, x : Ast.Value) : (C.FilterSlot)|null => C.makeAggregateFilter(p, 'sum', null, '==', x)), C.filterKeyFn, {  });
   $grammar.addRule("with_filter", [new $runtime.Choice(["total", "sum"]), new $runtime.NonTerminal("out_param_Array__Any", ["is_numeric_elem" as ("is_numeric_elem" extends keyof ReturnType<typeof C.paramKeyFn> ? string : void), (true) as ReturnType<typeof C.paramKeyFn>["is_numeric_elem"]]), new $runtime.Choice(["greater than", "more than", "above"]), new $runtime.NonTerminal("constant_Numeric", ["type" as ("type" extends keyof ReturnType<typeof C.valueKeyFn> ? string : void), 1, "elem" as ("elem" extends keyof ReturnType<typeof C.paramKeyFn> ? string : void)])], (($0 : undefined, p : C.ParamSlot, $1 : undefined, x : Ast.Value) : (C.FilterSlot)|null => C.makeAggregateFilter(p, 'sum', null, '>=', x)), C.filterKeyFn, {  });
   $grammar.addRule("with_filter", [new $runtime.Choice(["total", "sum"]), new $runtime.NonTerminal("out_param_Array__Any", ["is_numeric_elem" as ("is_numeric_elem" extends keyof ReturnType<typeof C.paramKeyFn> ? string : void), (true) as ReturnType<typeof C.paramKeyFn>["is_numeric_elem"]]), new $runtime.Choice(["lower than", "less than", "below"]), new $runtime.NonTerminal("constant_Numeric", ["type" as ("type" extends keyof ReturnType<typeof C.valueKeyFn> ? string : void), 1, "elem" as ("elem" extends keyof ReturnType<typeof C.paramKeyFn> ? string : void)])], (($0 : undefined, p : C.ParamSlot, $1 : undefined, x : Ast.Value) : (C.FilterSlot)|null => C.makeAggregateFilter(p, 'sum', null, '<=', x)), C.filterKeyFn, {  });
   if (!$options.flags.turking) {
       $grammar.addRule("with_filter", [new $runtime.Choice(["total", "sum"]), new $runtime.NonTerminal("out_param_Any", ["is_numeric" as ("is_numeric" extends keyof ReturnType<typeof C.paramKeyFn> ? string : void), (true) as ReturnType<typeof C.paramKeyFn>["is_numeric"]]), "of", new $runtime.NonTerminal("out_param_Array__Compound", undefined), new $runtime.Choice(["of", "being"]), new $runtime.NonTerminal("constant_Numeric", ["type" as ("type" extends keyof ReturnType<typeof C.valueKeyFn> ? string : void), 1, "type" as ("type" extends keyof ReturnType<typeof C.paramKeyFn> ? string : void)])], (($0 : undefined, f : C.ParamSlot, $1 : undefined, p : C.ParamSlot, $2 : undefined, x : Ast.Value) : (C.FilterSlot)|null => C.makeAggregateFilter(p, 'sum', f, '==', x)), C.filterKeyFn, {  });
   }
   if (!$options.flags.turking) {
       $grammar.addRule("with_filter", [new $runtime.Choice(["total", "sum"]), new $runtime.NonTerminal("out_param_Any", ["is_numeric" as ("is_numeric" extends keyof ReturnType<typeof C.paramKeyFn> ? string : void), (true) as ReturnType<typeof C.paramKeyFn>["is_numeric"]]), "of", new $runtime.NonTerminal("out_param_Array__Compound", undefined), new $runtime.Choice(["greater than", "more than", "higher than"]), new $runtime.NonTerminal("constant_Numeric", ["type" as ("type" extends keyof ReturnType<typeof C.valueKeyFn> ? string : void), 1, "type" as ("type" extends keyof ReturnType<typeof C.paramKeyFn> ? string : void)])], (($0 : undefined, f : C.ParamSlot, $1 : undefined, p : C.ParamSlot, $2 : undefined, x : Ast.Value) : (C.FilterSlot)|null => C.makeAggregateFilter(p, 'sum', f, '>=', x)), C.filterKeyFn, {  });
   }
   if (!$options.flags.turking) {
       $grammar.addRule("with_filter", [new $runtime.Choice(["total", "sum"]), new $runtime.NonTerminal("out_param_Any", ["is_numeric" as ("is_numeric" extends keyof ReturnType<typeof C.paramKeyFn> ? string : void), (true) as ReturnType<typeof C.paramKeyFn>["is_numeric"]]), "of", new $runtime.NonTerminal("out_param_Array__Compound", undefined), new $runtime.Choice(["smaller than", "less than", "lower than"]), new $runtime.NonTerminal("constant_Numeric", ["type" as ("type" extends keyof ReturnType<typeof C.valueKeyFn> ? string : void), 1, "type" as ("type" extends keyof ReturnType<typeof C.paramKeyFn> ? string : void)])], (($0 : undefined, f : C.ParamSlot, $1 : undefined, p : C.ParamSlot, $2 : undefined, x : Ast.Value) : (C.FilterSlot)|null => C.makeAggregateFilter(p, 'sum', f, '>=', x)), C.filterKeyFn, {  });
   }
   $grammar.addRule("with_filter", ["average", new $runtime.NonTerminal("out_param_Array__Any", ["is_numeric_elem" as ("is_numeric_elem" extends keyof ReturnType<typeof C.paramKeyFn> ? string : void), (true) as ReturnType<typeof C.paramKeyFn>["is_numeric_elem"]]), new $runtime.Choice(["of", "being"]), new $runtime.NonTerminal("constant_Numeric", ["type" as ("type" extends keyof ReturnType<typeof C.valueKeyFn> ? string : void), 1, "elem" as ("elem" extends keyof ReturnType<typeof C.paramKeyFn> ? string : void)])], (($0 : undefined, p : C.ParamSlot, $1 : undefined, x : Ast.Value) : (C.FilterSlot)|null => C.makeAggregateFilter(p, 'avg', null, '==', x)), C.filterKeyFn, {  });
   $grammar.addRule("with_filter", ["average", new $runtime.NonTerminal("out_param_Array__Any", ["is_numeric_elem" as ("is_numeric_elem" extends keyof ReturnType<typeof C.paramKeyFn> ? string : void), (true) as ReturnType<typeof C.paramKeyFn>["is_numeric_elem"]]), new $runtime.Choice(["greater than", "more than", "above"]), new $runtime.NonTerminal("constant_Numeric", ["type" as ("type" extends keyof ReturnType<typeof C.valueKeyFn> ? string : void), 1, "elem" as ("elem" extends keyof ReturnType<typeof C.paramKeyFn> ? string : void)])], (($0 : undefined, p : C.ParamSlot, $1 : undefined, x : Ast.Value) : (C.FilterSlot)|null => C.makeAggregateFilter(p, 'avg', null, '>=', x)), C.filterKeyFn, {  });
   $grammar.addRule("with_filter", ["average", new $runtime.NonTerminal("out_param_Array__Any", ["is_numeric_elem" as ("is_numeric_elem" extends keyof ReturnType<typeof C.paramKeyFn> ? string : void), (true) as ReturnType<typeof C.paramKeyFn>["is_numeric_elem"]]), new $runtime.Choice(["lower than", "less than", "below"]), new $runtime.NonTerminal("constant_Numeric", ["type" as ("type" extends keyof ReturnType<typeof C.valueKeyFn> ? string : void), 1, "elem" as ("elem" extends keyof ReturnType<typeof C.paramKeyFn> ? string : void)])], (($0 : undefined, p : C.ParamSlot, $1 : undefined, x : Ast.Value) : (C.FilterSlot)|null => C.makeAggregateFilter(p, 'avg', null, '<=', x)), C.filterKeyFn, {  });
   if (!$options.flags.turking) {
       $grammar.addRule("with_filter", ["average", new $runtime.NonTerminal("out_param_Any", ["is_numeric" as ("is_numeric" extends keyof ReturnType<typeof C.paramKeyFn> ? string : void), (true) as ReturnType<typeof C.paramKeyFn>["is_numeric"]]), "of", new $runtime.NonTerminal("out_param_Array__Compound", undefined), new $runtime.Choice(["of", "being"]), new $runtime.NonTerminal("constant_Numeric", ["type" as ("type" extends keyof ReturnType<typeof C.valueKeyFn> ? string : void), 1, "type" as ("type" extends keyof ReturnType<typeof C.paramKeyFn> ? string : void)])], (($0 : undefined, f : C.ParamSlot, $1 : undefined, p : C.ParamSlot, $2 : undefined, x : Ast.Value) : (C.FilterSlot)|null => C.makeAggregateFilter(p, 'avg', f, '==', x)), C.filterKeyFn, {  });
   }
   if (!$options.flags.turking) {
       $grammar.addRule("with_filter", ["average", new $runtime.NonTerminal("out_param_Any", ["is_numeric" as ("is_numeric" extends keyof ReturnType<typeof C.paramKeyFn> ? string : void), (true) as ReturnType<typeof C.paramKeyFn>["is_numeric"]]), "of", new $runtime.NonTerminal("out_param_Array__Compound", undefined), new $runtime.Choice(["greater than", "more than", "higher than"]), new $runtime.NonTerminal("constant_Numeric", ["type" as ("type" extends keyof ReturnType<typeof C.valueKeyFn> ? string : void), 1, "type" as ("type" extends keyof ReturnType<typeof C.paramKeyFn> ? string : void)])], (($0 : undefined, f : C.ParamSlot, $1 : undefined, p : C.ParamSlot, $2 : undefined, x : Ast.Value) : (C.FilterSlot)|null => C.makeAggregateFilter(p, 'avg', f, '>=', x)), C.filterKeyFn, {  });
   }
   if (!$options.flags.turking) {
       $grammar.addRule("with_filter", ["average", new $runtime.NonTerminal("out_param_Any", ["is_numeric" as ("is_numeric" extends keyof ReturnType<typeof C.paramKeyFn> ? string : void), (true) as ReturnType<typeof C.paramKeyFn>["is_numeric"]]), "of", new $runtime.NonTerminal("out_param_Array__Compound", undefined), new $runtime.Choice(["smaller than", "less than", "lower than"]), new $runtime.NonTerminal("constant_Numeric", ["type" as ("type" extends keyof ReturnType<typeof C.valueKeyFn> ? string : void), 1, "type" as ("type" extends keyof ReturnType<typeof C.paramKeyFn> ? string : void)])], (($0 : undefined, f : C.ParamSlot, $1 : undefined, p : C.ParamSlot, $2 : undefined, x : Ast.Value) : (C.FilterSlot)|null => C.makeAggregateFilter(p, 'avg', f, '>=', x)), C.filterKeyFn, {  });
   }
   $grammar.addRule("with_filter", [new $runtime.Choice(["maximum", "highest"]), new $runtime.NonTerminal("out_param_Array__Any", ["is_numeric_elem" as ("is_numeric_elem" extends keyof ReturnType<typeof C.paramKeyFn> ? string : void), (true) as ReturnType<typeof C.paramKeyFn>["is_numeric_elem"]]), new $runtime.Choice(["of", "being"]), new $runtime.NonTerminal("constant_Numeric", ["type" as ("type" extends keyof ReturnType<typeof C.valueKeyFn> ? string : void), 1, "elem" as ("elem" extends keyof ReturnType<typeof C.paramKeyFn> ? string : void)])], (($0 : undefined, p : C.ParamSlot, $1 : undefined, x : Ast.Value) : (C.FilterSlot)|null => C.makeAggregateFilter(p, 'max', null, '==', x)), C.filterKeyFn, {  });
   $grammar.addRule("with_filter", [new $runtime.Choice(["maximum", "highest"]), new $runtime.NonTerminal("out_param_Array__Any", ["is_numeric_elem" as ("is_numeric_elem" extends keyof ReturnType<typeof C.paramKeyFn> ? string : void), (true) as ReturnType<typeof C.paramKeyFn>["is_numeric_elem"]]), new $runtime.Choice(["greater than", "more than", "above"]), new $runtime.NonTerminal("constant_Numeric", ["type" as ("type" extends keyof ReturnType<typeof C.valueKeyFn> ? string : void), 1, "elem" as ("elem" extends keyof ReturnType<typeof C.paramKeyFn> ? string : void)])], (($0 : undefined, p : C.ParamSlot, $1 : undefined, x : Ast.Value) : (C.FilterSlot)|null => C.makeAggregateFilter(p, 'max', null, '>=', x)), C.filterKeyFn, {  });
   $grammar.addRule("with_filter", [new $runtime.Choice(["maximum", "highest"]), new $runtime.NonTerminal("out_param_Array__Any", ["is_numeric_elem" as ("is_numeric_elem" extends keyof ReturnType<typeof C.paramKeyFn> ? string : void), (true) as ReturnType<typeof C.paramKeyFn>["is_numeric_elem"]]), new $runtime.Choice(["lower than", "less than", "below"]), new $runtime.NonTerminal("constant_Numeric", ["type" as ("type" extends keyof ReturnType<typeof C.valueKeyFn> ? string : void), 1, "elem" as ("elem" extends keyof ReturnType<typeof C.paramKeyFn> ? string : void)])], (($0 : undefined, p : C.ParamSlot, $1 : undefined, x : Ast.Value) : (C.FilterSlot)|null => C.makeAggregateFilter(p, 'max', null, '<=', x)), C.filterKeyFn, {  });
   if (!$options.flags.turking) {
       $grammar.addRule("with_filter", [new $runtime.Choice(["maximum", "highest"]), new $runtime.NonTerminal("out_param_Any", ["is_numeric" as ("is_numeric" extends keyof ReturnType<typeof C.paramKeyFn> ? string : void), (true) as ReturnType<typeof C.paramKeyFn>["is_numeric"]]), "of", new $runtime.NonTerminal("out_param_Array__Compound", undefined), new $runtime.Choice(["of", "being"]), new $runtime.NonTerminal("constant_Numeric", ["type" as ("type" extends keyof ReturnType<typeof C.valueKeyFn> ? string : void), 1, "type" as ("type" extends keyof ReturnType<typeof C.paramKeyFn> ? string : void)])], (($0 : undefined, f : C.ParamSlot, $1 : undefined, p : C.ParamSlot, $2 : undefined, x : Ast.Value) : (C.FilterSlot)|null => C.makeAggregateFilter(p, 'max', f, '==', x)), C.filterKeyFn, {  });
   }
   if (!$options.flags.turking) {
       $grammar.addRule("with_filter", [new $runtime.Choice(["maximum", "highest"]), new $runtime.NonTerminal("out_param_Any", ["is_numeric" as ("is_numeric" extends keyof ReturnType<typeof C.paramKeyFn> ? string : void), (true) as ReturnType<typeof C.paramKeyFn>["is_numeric"]]), "of", new $runtime.NonTerminal("out_param_Array__Compound", undefined), new $runtime.Choice(["greater than", "more than", "higher than"]), new $runtime.NonTerminal("constant_Numeric", ["type" as ("type" extends keyof ReturnType<typeof C.valueKeyFn> ? string : void), 1, "type" as ("type" extends keyof ReturnType<typeof C.paramKeyFn> ? string : void)])], (($0 : undefined, f : C.ParamSlot, $1 : undefined, p : C.ParamSlot, $2 : undefined, x : Ast.Value) : (C.FilterSlot)|null => C.makeAggregateFilter(p, 'max', f, '>=', x)), C.filterKeyFn, {  });
   }
   if (!$options.flags.turking) {
       $grammar.addRule("with_filter", [new $runtime.Choice(["maximum", "highest"]), new $runtime.NonTerminal("out_param_Any", ["is_numeric" as ("is_numeric" extends keyof ReturnType<typeof C.paramKeyFn> ? string : void), (true) as ReturnType<typeof C.paramKeyFn>["is_numeric"]]), "of", new $runtime.NonTerminal("out_param_Array__Compound", undefined), new $runtime.Choice(["smaller than", "less than", "lower than"]), new $runtime.NonTerminal("constant_Numeric", ["type" as ("type" extends keyof ReturnType<typeof C.valueKeyFn> ? string : void), 1, "type" as ("type" extends keyof ReturnType<typeof C.paramKeyFn> ? string : void)])], (($0 : undefined, f : C.ParamSlot, $1 : undefined, p : C.ParamSlot, $2 : undefined, x : Ast.Value) : (C.FilterSlot)|null => C.makeAggregateFilter(p, 'max', f, '>=', x)), C.filterKeyFn, {  });
   }
   $grammar.addRule("with_filter", [new $runtime.Choice(["minimum", "lowest"]), new $runtime.NonTerminal("out_param_Array__Any", ["is_numeric_elem" as ("is_numeric_elem" extends keyof ReturnType<typeof C.paramKeyFn> ? string : void), (true) as ReturnType<typeof C.paramKeyFn>["is_numeric_elem"]]), new $runtime.Choice(["of", "being"]), new $runtime.NonTerminal("constant_Numeric", ["type" as ("type" extends keyof ReturnType<typeof C.valueKeyFn> ? string : void), 1, "elem" as ("elem" extends keyof ReturnType<typeof C.paramKeyFn> ? string : void)])], (($0 : undefined, p : C.ParamSlot, $1 : undefined, x : Ast.Value) : (C.FilterSlot)|null => C.makeAggregateFilter(p, 'min', null, '==', x)), C.filterKeyFn, {  });
   $grammar.addRule("with_filter", [new $runtime.Choice(["minimum", "lowest"]), new $runtime.NonTerminal("out_param_Array__Any", ["is_numeric_elem" as ("is_numeric_elem" extends keyof ReturnType<typeof C.paramKeyFn> ? string : void), (true) as ReturnType<typeof C.paramKeyFn>["is_numeric_elem"]]), new $runtime.Choice(["greater than", "more than", "above"]), new $runtime.NonTerminal("constant_Numeric", ["type" as ("type" extends keyof ReturnType<typeof C.valueKeyFn> ? string : void), 1, "elem" as ("elem" extends keyof ReturnType<typeof C.paramKeyFn> ? string : void)])], (($0 : undefined, p : C.ParamSlot, $1 : undefined, x : Ast.Value) : (C.FilterSlot)|null => C.makeAggregateFilter(p, 'min', null, '>=', x)), C.filterKeyFn, {  });
   $grammar.addRule("with_filter", [new $runtime.Choice(["minimum", "lowest"]), new $runtime.NonTerminal("out_param_Array__Any", ["is_numeric_elem" as ("is_numeric_elem" extends keyof ReturnType<typeof C.paramKeyFn> ? string : void), (true) as ReturnType<typeof C.paramKeyFn>["is_numeric_elem"]]), new $runtime.Choice(["lower than", "less than", "below"]), new $runtime.NonTerminal("constant_Numeric", ["type" as ("type" extends keyof ReturnType<typeof C.valueKeyFn> ? string : void), 1, "elem" as ("elem" extends keyof ReturnType<typeof C.paramKeyFn> ? string : void)])], (($0 : undefined, p : C.ParamSlot, $1 : undefined, x : Ast.Value) : (C.FilterSlot)|null => C.makeAggregateFilter(p, 'min', null, '<=', x)), C.filterKeyFn, {  });
   if (!$options.flags.turking) {
       $grammar.addRule("with_filter", [new $runtime.Choice(["minimum", "lowest"]), new $runtime.NonTerminal("out_param_Any", ["is_numeric" as ("is_numeric" extends keyof ReturnType<typeof C.paramKeyFn> ? string : void), (true) as ReturnType<typeof C.paramKeyFn>["is_numeric"]]), "of", new $runtime.NonTerminal("out_param_Array__Compound", undefined), new $runtime.Choice(["of", "being"]), new $runtime.NonTerminal("constant_Numeric", ["type" as ("type" extends keyof ReturnType<typeof C.valueKeyFn> ? string : void), 1, "type" as ("type" extends keyof ReturnType<typeof C.paramKeyFn> ? string : void)])], (($0 : undefined, f : C.ParamSlot, $1 : undefined, p : C.ParamSlot, $2 : undefined, x : Ast.Value) : (C.FilterSlot)|null => C.makeAggregateFilter(p, 'min', f, '==', x)), C.filterKeyFn, {  });
   }
   if (!$options.flags.turking) {
       $grammar.addRule("with_filter", [new $runtime.Choice(["minimum", "lowest"]), new $runtime.NonTerminal("out_param_Any", ["is_numeric" as ("is_numeric" extends keyof ReturnType<typeof C.paramKeyFn> ? string : void), (true) as ReturnType<typeof C.paramKeyFn>["is_numeric"]]), "of", new $runtime.NonTerminal("out_param_Array__Compound", undefined), new $runtime.Choice(["greater than", "more than", "higher than"]), new $runtime.NonTerminal("constant_Numeric", ["type" as ("type" extends keyof ReturnType<typeof C.valueKeyFn> ? string : void), 1, "type" as ("type" extends keyof ReturnType<typeof C.paramKeyFn> ? string : void)])], (($0 : undefined, f : C.ParamSlot, $1 : undefined, p : C.ParamSlot, $2 : undefined, x : Ast.Value) : (C.FilterSlot)|null => C.makeAggregateFilter(p, 'min', f, '>=', x)), C.filterKeyFn, {  });
   }
   if (!$options.flags.turking) {
       $grammar.addRule("with_filter", [new $runtime.Choice(["minimum", "lowest"]), new $runtime.NonTerminal("out_param_Any", ["is_numeric" as ("is_numeric" extends keyof ReturnType<typeof C.paramKeyFn> ? string : void), (true) as ReturnType<typeof C.paramKeyFn>["is_numeric"]]), "of", new $runtime.NonTerminal("out_param_Array__Compound", undefined), new $runtime.Choice(["smaller than", "less than", "lower than"]), new $runtime.NonTerminal("constant_Numeric", ["type" as ("type" extends keyof ReturnType<typeof C.valueKeyFn> ? string : void), 1, "type" as ("type" extends keyof ReturnType<typeof C.paramKeyFn> ? string : void)])], (($0 : undefined, f : C.ParamSlot, $1 : undefined, p : C.ParamSlot, $2 : undefined, x : Ast.Value) : (C.FilterSlot)|null => C.makeAggregateFilter(p, 'min', f, '>=', x)), C.filterKeyFn, {  });
   }
   $grammar.declareSymbol("range_with_filter");
   $grammar.addRule("range_with_filter", [new $runtime.NonTerminal("out_param_Any", ["is_numeric" as ("is_numeric" extends keyof ReturnType<typeof C.paramKeyFn> ? string : void), (true) as ReturnType<typeof C.paramKeyFn>["is_numeric"]]), new $runtime.NonTerminal("range", undefined)], ((param : C.ParamSlot, range : [Ast.Value, Ast.Value]) : (C.FilterSlot)|null => {
        const [v1, v2] = range;
        const f1 = C.makeFilter($loader, param, '>=', v1);
        const f2 = C.makeFilter($loader, param, '<=', v2);
        if (!f1 || !f2)
            return null;
        return { schema: f1.schema, ptype: f1.ptype,
            ast: new Ast.BooleanExpression.And(null, [f1.ast, f2.ast]) };
    }), C.filterKeyFn, {  });
   $grammar.declareSymbol("both_prefix");
   $grammar.addRule("both_prefix", ["both"], (($0 : undefined) : (string)|null => "both"), undefined, { identity: (true) });
   $grammar.declareSymbol("nearby_filter");
   $grammar.addRule("nearby_filter", ["here"], (($0 : undefined) : (C.DomainIndependentFilterSlot)|null => C.makeDomainIndependentFilter('geo', '==', HERE)), undefined, {  });
   $grammar.addRule("nearby_filter", ["around here"], (($0 : undefined) : (C.DomainIndependentFilterSlot)|null => C.makeDomainIndependentFilter('geo', '==', HERE)), undefined, {  });
   $grammar.addRule("nearby_filter", ["near here"], (($0 : undefined) : (C.DomainIndependentFilterSlot)|null => C.makeDomainIndependentFilter('geo', '==', HERE)), undefined, {  });
   $grammar.addRule("nearby_filter", ["nearby"], (($0 : undefined) : (C.DomainIndependentFilterSlot)|null => C.makeDomainIndependentFilter('geo', '==', HERE)), undefined, {  });
   $grammar.addRule("nearby_filter", ["around me"], (($0 : undefined) : (C.DomainIndependentFilterSlot)|null => C.makeDomainIndependentFilter('geo', '==', HERE)), undefined, {  });
   $grammar.addRule("nearby_filter", ["in this area"], (($0 : undefined) : (C.DomainIndependentFilterSlot)|null => C.makeDomainIndependentFilter('geo', '==', HERE)), undefined, {  });
   $grammar.addRule("nearby_filter", [new $runtime.Choice(["in", "around", "near"]), new $runtime.NonTerminal("constant_Location", undefined)], (($0 : undefined, x : Ast.LocationValue) : (C.DomainIndependentFilterSlot)|null => C.makeDomainIndependentFilter('geo', '==', x)), undefined, {  });
   $grammar.declareSymbol("whose_npp_filter");
   $grammar.addRule("whose_npp_filter", [new $runtime.NonTerminal("out_param_Any", undefined), "is", new $runtime.NonTerminal("constant_Any", undefined)], ((p : C.ParamSlot, $0 : undefined, x : Ast.Value) : (C.FilterSlot)|null => C.makeFilter($loader, p, '==', x)), C.filterKeyFn, {  });
   $grammar.addRule("whose_npp_filter", [new $runtime.NonTerminal("out_param_Any", undefined), "is", new $runtime.NonTerminal("constant_Any", undefined)], ((p : C.ParamSlot, $0 : undefined, x : Ast.Value) : (C.FilterSlot)|null => C.makeFilter($loader, p, 'contains', x)), C.filterKeyFn, {  });
   $grammar.declareSymbol("quality_filter");
   $grammar.addRule("quality_filter", ["good"], (($0 : undefined) : (C.DomainIndependentFilterSlot)|null => C.makeDomainIndependentFilter('aggregateRating.ratingValue', '>=', new Ast.Value.Number(4))), undefined, {  });
   $grammar.addRule("quality_filter", ["great"], (($0 : undefined) : (C.DomainIndependentFilterSlot)|null => C.makeDomainIndependentFilter('aggregateRating.ratingValue', '>=', new Ast.Value.Number(4))), undefined, {  });
   $grammar.addRule("quality_filter", ["nice"], (($0 : undefined) : (C.DomainIndependentFilterSlot)|null => C.makeDomainIndependentFilter('aggregateRating.ratingValue', '>=', new Ast.Value.Number(4))), undefined, {  });
   $grammar.addRule("quality_filter", ["decent"], (($0 : undefined) : (C.DomainIndependentFilterSlot)|null => C.makeDomainIndependentFilter('aggregateRating.ratingValue', '>=', new Ast.Value.Number(4))), undefined, {  });
   $grammar.declareSymbol("within_filter");
   $grammar.addRule("within_filter", ["within", new $runtime.NonTerminal("constant_Measure_m", undefined), new $runtime.Choice(["", "radius", "of here", "from here", "from me", "of me", "of this area", "of my location", "from current location"])], (($0 : undefined, distance : Ast.Value, $1 : undefined) : (({ distance : Ast.Value, place : Ast.Value }))|null => ({ distance, place: HERE })), undefined, {  });
   $grammar.addRule("within_filter", ["within", new $runtime.NonTerminal("constant_Measure_m", undefined), new $runtime.Choice(["", "radius"]), new $runtime.Choice(["of", "from"]), new $runtime.NonTerminal("constant_Location", undefined)], (($0 : undefined, distance : Ast.Value, $1 : undefined, $2 : undefined, place : Ast.LocationValue) : (({ distance : Ast.Value, place : Ast.Value }))|null => ({ distance, place })), undefined, {  });
};
